note
	description: "Summary description for {SIT_CSV}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	SIT_CSV[G -> DB_SERVICE[DB_ENTITY]]

inherit
	FALLIBLE

feature {NONE} -- Initialization

	make (a_rest_request: like rest_request; a_db_service: like db_service)
		do
			db_service := a_db_service
			rest_request := a_rest_request
			current_row_index := 1
			default_create
		ensure
			rest_request = a_rest_request
			db_service = a_db_service
		end

	make_from_date_format (a_date_time_format: attached like date_time_format; a_rest_request: like rest_request; a_db_service: like db_service)
		do
			date_time_format := a_date_time_format
			make (a_rest_request, a_db_service)
		ensure
			attached date_time_format and date_time_format = a_date_time_format
		end

feature -- Access

	file_path: detachable STRING

	date_time_format: detachable STRING

	db_service: G

	rest_request: REST_REQUEST

	current_row_index: INTEGER -- starting @ start_row_index

	start_row_index: INTEGER = 2

	header_row_index: INTEGER
		deferred
		end

feature -- Parsing

	before
			-- Redefine if needed
		do
		end

	process
			-- calls for_each_csv_line
		require
			attached file_path
			attached date_time_format
		do
			before
			import_from_csv_impl
			after
		end

	after
			-- updating items into DB
			-- Redefine if needed
		do
			if attached db_service as l_dbs and then
				attached l_dbs.item as l_item
			then
				l_dbs.set_insert_if_not_exists (True)
				l_dbs.update_item_s_relationships
				l_dbs.set_insert_if_not_exists (False)
			else
				check
					item_not_void: False
				end
			end
		end

feature {NONE} -- Lines processing

	process_row (a_csv_row: DS_ARRAYED_LIST [STRING])
		deferred
		ensure
			current_row_index = old current_row_index + 1
		end

feature {NONE} -- Implementation

	csv_header_is_valid: BOOLEAN
	csv_header: detachable DS_ARRAYED_LIST [STRING]

	set_csv_header (a_csv_row: attached like csv_header)
		require
			current_row_index < start_row_index
		do
			if current_row_index = header_row_index then
				csv_header := a_csv_row.deep_twin
				csv_header_is_valid := not a_csv_row.is_empty
			end
		end


	import_from_csv_impl
			--
		require
			attached file_path
		local
			l_csv_handler: SIT_CSV_HANDLER
			l_is: KL_STRING_INPUT_STREAM
			l_file: KL_TEXT_INPUT_FILE
			l_to_read_count_at_once: INTEGER
--			l_partially_closed: PROCEDURE[DS_ARRAYED_LIST[STRING]]
		do
			if attached file_path as l_fp then
				create l_file.make (l_fp)
				l_to_read_count_at_once := l_file.count
				l_file.open_read
				create l_csv_handler.make_with_separator (',')
				if attached l_file.last_string as l_file_s then
	--				check
	--					attached item
	--				end
	--				if attached item then

	--					l_partially_closed := agent (ia_l: DS_ARRAYED_LIST[STRING];
	--							ia_file_path: STRING;
	--							ia_proc: PROCEDURE[DS_ARRAYED_LIST[STRING], INTEGER_64, STRING])
	--						do
	--							ia_proc.call (ia_l, ia_file_path)
	--						end (?, l_file_path, process_csv_line)
	--					l_csv_handler.read_file (l_is, l_partially_closed)
	--				else
	--					set_last_error (create {INVALID_REQUEST}.make("Unable to find measuring point with id:" + l_measuring_point_id.out))
	--				end

					-- read file
					l_file.read_string (l_to_read_count_at_once)
					create l_is.make (l_file.last_string)
					l_csv_handler.read_file (l_is, agent process_row)
				else
					check
						file_empty: False
					end
				end
				l_file.close
			end
		end

invariant
	start_row_index >= 1
	current_row_index >= 1
	header_row_index >= 1
end


