note
	description: "Summary description for {DB_CONNECTION}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	DB_CONNECTION

inherit
	DATABASE_APPL [ODBC]
		redefine
			out,
			default_create
		end

	FALLIBLE
		redefine
			out,
			default_create
		end

	GLOBAL_SETTINGS
		redefine
			out,
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
		do
			Precursor {DATABASE_APPL}
			Precursor {FALLIBLE}
			Precursor {GLOBAL_SETTINGS}
			set_use_extended_types (True) --| Use extended types: INTEGER_64, STRING_32 etc. https://stackoverflow.com/questions/54463744/eiffel-odbc-dealing-with-bigint
			set_map_zero_null_value (False) --
		end

feature -- Status report

	out: STRING
		do
			Result := Precursor {FALLIBLE}
			Result.append (Precursor {DATABASE_APPL})
		end

feature {NONE} -- Access for Database Operations

	session_control: detachable DB_CONTROL

feature -- Database Connection

	disconnect
		require
			is_connected
		do
			if attached session_control as sc then
				sc.disconnect
				logger.write_information("DB Connection disconnected")
			else
				logger.write_error("Unable to disconnect, session_control is Void")
			end
		end

	is_connected: BOOLEAN
		do
			Result := attached session_control as l_sc and then l_sc.is_connected
		end

	connect
		local
			l_msg: STRING
		do
			if attached {APP_CONFIGURATION}.app_instance.configuration_value ({APP_CONFIGURATION}.Datasource_name_json_key) as l_dn then
				logger.write_debug ("connect->setting datasource with:" + l_dn)
				set_data_source(l_dn)
				if
					attached {APP_CONFIGURATION}.app_instance.configuration_value ({APP_CONFIGURATION}.Database_username_json_key) as l_un and
					attached {APP_CONFIGURATION}.app_instance.configuration_value ({APP_CONFIGURATION}.Database_password_json_key) as l_pwd
				then
					-- l_connection_string := "DRIVER={Devart ODBC Driver for PostgreSQL};Data Source=" + l_dn + ";User ID=" + l_un + ";Password=" + l_pwd + ";Database=sit_entities_db;Schema=prod"
					-- Set user's name and password
					login (l_un, l_pwd)
					-- login_with_connection_string(l_connection_string) -- NOT WORKING
						-- Initialization of the Relational Database:
						-- This will set various informations to perform a correct
						-- Connection to the Relational database
					set_base

						-- Create usefull classes
						-- 'session_control' provides informations control access and
						--  the status of the database.
					if not attached session_control then
						create session_control.make
					end
						-- Start session
					if attached session_control as l_sc then
						if {APP_CONFIGURATION}.app_instance.trace_enabled then
							l_sc.set_trace
						end

						if not l_sc.is_connected then
							l_msg := "connect->connecting session_control for DB_CONNECTION, it wasn't connected"
							logger.write_information(l_msg)
							l_sc.connect
						end
						check
							-- check with # isql -v PODBC
							is_connected: l_sc.is_connected
						end
						set_used_schema
					else
						l_msg := "Unable to connect to database, session_control is Void"
						set_last_error (create {ODBC_ERROR}.make (l_msg))
						(create {EXCEPTIONS}).raise (l_msg)
					end
				else
					l_msg := {APP_CONFIGURATION}.Database_username_json_key + "|" + {APP_CONFIGURATION}.Database_password_json_key + " not found into configuration file"
					logger.write_error (l_msg)
					(create {EXCEPTIONS}).raise (l_msg)
				end
			else
				l_msg := {APP_CONFIGURATION}.Datasource_name_json_key + " not found into configuration file"
				logger.write_error (l_msg)
				(create {EXCEPTIONS}).raise (l_msg)
			end
		ensure
			is_connected -- ensure 'isql -v PODBC'
					-- Something went wrong, and the connection failed
--				session_control.raise_error
--			end
		end

	set_used_schema
		require
			is_connected
		local
			l_base_update: DB_CHANGE
			l_qry, l_msg : STRING
		do
			l_qry := "SET search_path TO " + {APP_CONFIGURATION}.app_instance.current_db_schema + ";"
			create l_base_update.make
			if l_base_update.is_ok then
				l_base_update.modify (l_qry)
				if l_base_update.is_ok then
					l_msg := "set_used_schema with (" + l_qry + ")=> Updated successfully"
					set_last_success_message (l_msg)
					logger.write_information ("set_used_schema->Success:" + l_msg + " qry:" + l_qry)
				else
					set_last_error (create {ODBC_ERROR}.make("Error updating qry:" + l_qry + "-code:" + l_base_update.error_code.out + "-msg:" + l_base_update.error_message_32))
				end
				if not l_base_update.warning_message_32.is_empty then
					logger.write_warning ("set_used_schema:warning_message_32:" + l_base_update.warning_message_32)
				end
			else
				set_last_error (create {ODBC_ERROR}.make("Error updating qry:" + l_qry + "-code:" + l_base_update.error_code.out + "-msg:" + l_base_update.error_message_32))
			end
		ensure
			not has_error
		end

	manage_errors_and_warnings
			-- Manage errors and warnings that may have
			-- occurred during last operation.
		do
			if attached session_control as l_session_control then
				if l_session_control.is_ok then
						-- There was an error!
					l_session_control.raise_error
					l_session_control.reset
				else
					if l_session_control.warning_message_32.count /= 0 then
						logger.write_warning (l_session_control.warning_message_32)
					end
				end
			else
				set_last_error (create {ATTACHED_OBJECT_EXPECTED}.make ("session_control is Void"))
			end
		end


end
