note
	description: "G is the type of setter's argument, for set_name(name:STRING_8) it'll be STRING_8"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	DB_FIELD

inherit
	LOGGABLE

create
	make

feature -- Initialization

	make (a_field_name: like db_field_name; a_setter: like setter; a_value_to_set: ANY; a_setter_argument_type: TYPE[ANY])
			-- a_value_to_set has to be of the type of the value to set. It's used @runtime to define the type to transform to, and used with to_json function
		do
			setter := a_setter
			db_field_name := a_field_name
			value_to_set := a_value_to_set
			setter_argument_type := a_setter_argument_type
		end

feature -- Status Report

	setter: PROCEDURE[TUPLE[ANY]]
		-- The eiffel setter for this field

	value_to_set: ANY
		-- needed to define the type of value to transform @runtime

	db_field_name: STRING

	eiffel_field_name: like db_field_name
			-- DB field name as a convention
		do
			Result := db_field_name
		end

	setter_argument_type: TYPE[ANY]

	call_setter (a_value: ANY)
		do
		--	l_setter_argument_type := (create {REFLECTOR}).type_of_type (setter.open_operand_type(1))
			if attached {JSON_VALUE} a_value as l_val then
				logger.write_debug ("call_setter->" + db_field_name + " := " + l_val.representation)
			else
				logger.write_debug ("call_setter->" + db_field_name + " := " + a_value.out)
			end
			if setter_argument_type.conforms_to({INTEGER_64}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_integer_64 (a_value)])
			elseif setter_argument_type.conforms_to({INTEGER_32}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_integer_32 (a_value)])
			elseif setter_argument_type.conforms_to({RUT}) then
				if attached {JSON_STRING} a_value as l_v then
					setter.call ([create {RUT}.make_from_string (l_v.unescaped_string_8)])
				else
					setter.call ([create {RUT}.make_from_string (a_value.out)])
				end

			elseif setter_argument_type.conforms_to({STRING}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_string (a_value)])
			elseif setter_argument_type.conforms_to({STRING_32}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_string (a_value).to_string_32])
			elseif setter_argument_type.conforms_to({WATT}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_watt (a_value)])
			elseif setter_argument_type.conforms_to({VOLT}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_volt (a_value)])
			elseif setter_argument_type.conforms_to({AMPERE}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_ampere (a_value)])
			elseif setter_argument_type.conforms_to({K_WATT_H}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_k_w_h (a_value)])
			elseif setter_argument_type.conforms_to({DATE_TIME}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_date_time (a_value)])
			elseif setter_argument_type.conforms_to({DATE}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_date (a_value)])
			elseif setter_argument_type.conforms_to({REAL_32}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_real (a_value)])
			elseif setter_argument_type.conforms_to({REAL_64}) then
				if attached {REAL_32_REF} a_value as l_val then
					setter.call ([l_val.to_double])
				else
					setter.call ([{SIT_TYPE_CONVERTER}.force_as_real_64 (a_value)])
				end
			elseif setter_argument_type.conforms_to({NATURAL_64}) then
				setter.call ([{SIT_TYPE_CONVERTER}.force_as_natural_64 (a_value)])
			else
				logger.write_error ("Type of setter argument not treated:" + setter_argument_type.out)
				check
					setter_argument_type_unknown: False
				end
			end
		end

end
