note
	description: "Summary description for {COMPOSED_SQL_QUERY}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	COMPOSED_SQL_QUERY

inherit
	SQL_QUERY
		redefine
			make,
			to_sql
		end

create
	make

feature {NONE} -- Initialization

	make (a_base_entity: like base_entity)
		do
			create select_clauses.make
			create from_clauses.make
			create join_clauses.make
			Precursor (a_base_entity)
		end

feature -- Access

	Coma_clauses_sep: STRING = ", "


	select_clauses: LINKED_LIST[STRING]

	from_clauses: LINKED_LIST[STRING]

	join_clauses: LINKED_LIST[JOIN]


feature -- Element change

	add_api_exportable_fields_to_select (
			a_join_alias,
			an_table_field_alias_prefix: STRING;
			a_rest_req_relationship: like {REST_REQUEST}.relationships.item)
            -- Adds fieldnames AS alias to a_select_from
        require
        	attached {PK_DB_ENTITY} a_rest_req_relationship.secondary_prototype
        local
            l_secondary_alias: STRING
		do
			-- Exported fields
			if attached {API_EXPORTABLE_ENTITY} a_rest_req_relationship.secondary_prototype as l_s_ee and then
				attached {PK_DB_ENTITY} a_rest_req_relationship.secondary_prototype as l_s_pkdbe
			then
				if l_s_ee.api_exported_fields.is_empty then
					logger.write_warning ("query_from_relationships: beware, no exported field were found for " + l_s_ee.generating_type.out)
				end
				-- add default id as join field
				l_secondary_alias := a_rest_req_relationship.secondary_join_as_field_name (l_s_pkdbe.Primary_key_db_column_name)
--				select_clauses.extend (a_join_alias + "." + l_s_pkdbe.Primary_key_db_column_name + " AS " + l_secondary_alias) -- not needed into curl --header "Content-Type: application/json"   --request POST --data '{ "action": { "feature": "pdf_data", "arg1": "1"} }' http://localhost:9997/customer_invoice
				-- add api_exported_fields to join
				across
					l_s_ee.api_exported_fields as l_exported_field_name
				loop
					l_secondary_alias := an_table_field_alias_prefix + "_" + l_exported_field_name.item
					select_clauses.extend (a_join_alias + "." + l_exported_field_name.item + " AS " + l_secondary_alias)
				end
			else
				logger.write_debug ("add_api_exportable_fields_to_select-> Not adding relationship because not exportable or doesn't have a pk:" + a_rest_req_relationship.generating_type.out)
			end
		end

feature -- Out

	to_sql: STRING
		do
			Result := select_clauses_to_sql +
				from_clauses_to_sql +
				join_clauses_to_sql +
				constraints_to_sql +
				order_clauses_to_sql +
				limit_to_sql
		end

	select_clauses_to_sql: STRING
		do
			Result := "SELECT "
			if select_clauses.is_empty then
				Result.append ("* ")
			else
				across
					select_clauses as l_v
				loop
					Result.append ("%N%T" + l_v.item)
					if not l_v.is_last then
						 Result.append (Coma_clauses_sep)
					end
				end
			end
		end

	from_clauses_to_sql: STRING
		do
			Result := ""
			Result.append ("%NFROM ONLY ")
			if from_clauses.is_empty then
				Result.append (base_entity.table_name + Coma_clauses_sep)
			end
			across
				from_clauses as l_v
			loop
				Result.append ("%N%T" + l_v.item)
				if not l_v.is_last then
					 Result.append (Coma_clauses_sep)
				end
			end

			if Result.ends_with (Coma_clauses_sep) then
				Result.remove_tail (Coma_clauses_sep.count)
			end
		ensure
			not Result.ends_with (Coma_clauses_sep)
		end

	join_clauses_to_sql: STRING
		do
			Result := ""
			across
				join_clauses as l_v
			loop
				Result.append ("%N%T" + l_v.item.to_sql)
			end
		end

	order_clauses_to_sql: STRING
		do
			Result := ""
			if not order_clauses.is_empty then
				Result.append ("%NORDER BY ")
			end
			across
				order_clauses as l_v
			loop
				Result.append ("%N%T" + l_v.item.to_sql)
				if not l_v.is_last then
					 Result.append (Coma_clauses_sep)
				end
			end
		end

end
