note
	description: "Summary description for {SQL_INSERT}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	SQL_INSERT

inherit
	SQL_MODIFY
		redefine
			make
		end

create
	make

feature {NONE} -- Initialization

	make (a_base_entity: like base_entity)
		do
			base_entity := a_base_entity
			create confict_fields_list.make
			default_create
		end

feature -- Access

	on_conflict_action: detachable STRING
	On_conflict_action_do_nothing: STRING = "DO NOTHING"
	On_conflict_action_update: STRING = "UPDATE"

	confict_fields_list: LINKED_LIST[DB_FIELD]

feature -- Status setting

	set_on_conflict_action (v: attached like on_conflict_action)
		require
			v = On_conflict_action_do_nothing or
				v = On_conflict_action_update
		do
			on_conflict_action := v
		ensure
			on_conflict_action = v
		end

feature -- Output

	to_sql: STRING
		local
			l_keys, l_values: LINKED_SET[STRING]
		do
			create l_keys.make
			create l_values.make
			-- base_entity fields
			across
				base_entity.fields_mapping as l_field
			loop
				if not base_entity.db_unstored_field_names.has (l_field.item.db_field_name) then
					l_keys.extend (l_field.item.db_field_name)
					l_values.extend ({SQL_UTIL}.from_value_to_db_field_string (
						l_field.item.value_to_set,
						l_field.item.db_field_name.is_case_insensitive_equal ({PK_DB_ENTITY}.Primary_key_db_column_name)))
				end
			end
			-- relationships fields
			if attached {RELATED_PK_DB_ENTITY} base_entity as l_reldbe then
				across
					l_reldbe.relationships as l_rel
				loop
					if attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rel.item as l_rel_a then
						if attached {PK_DB_ENTITY} l_rel_a.secondary as l_pkdbe then
							if attached {IDENTIFIABLE} l_rel.item as l_rel_i then
								l_keys.extend (l_rel_i.foreign_key)
							else
								l_keys.extend (l_rel_a.secondary_foreign_key)
							end
							l_values.extend (l_pkdbe.primary_key.out)
						end
					else
						check
							untreated_relationship_type: False
						end
					end
				end
			end
			-- qry construction
			Result := "INSERT INTO "
			Result.append (base_entity.table_name)
			-- field names
			Result.append ("%N%T(")
			across
				l_keys as l_v
			loop
				Result.append (l_v.item)
				if not l_v.is_last then
					Result.append (", ")
				end
			end
			Result.append (")")
			-- values
			Result.append ("%NVALUES")
			Result.append ("%N%T(")
			across
				l_values as l_v
			loop
				Result.append (l_v.item)
				if not l_v.is_last then
					Result.append (", ")
				end
			end
			Result.append (")")
			-- conflict
			if attached on_conflict_action as l_act then
				Result.append ("%NON CONFLICT ")
				if not confict_fields_list.is_empty then
					Result.append ("(")
					across
						confict_fields_list as l_cf
					loop
						if not base_entity.db_unstored_field_names.has (l_cf.item.db_field_name) then
							Result.append (l_cf.item.db_field_name)
							if not l_cf.is_last then
								Result.append (", ")
							end
						end
					end
					Result.append (")")
				end
				Result.append (l_act)
			end
		end

end
