note
	description: "Summary description for {DB_ENTITY}."
	author: "Philippe Gachoud"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	DB_ENTITY

inherit
	LOGGABLE
		redefine
			default_create,
			out
		end

	FALLIBLE
		redefine
			default_create,
			out
		end

	ENTITY
		redefine
			default_create,
			out
		end

	JSON_SIT_ENTITY
		rename
			unexported_field_names as json_unexported_field_names
		redefine
			default_create,
			out
		end


feature {NONE} -- Initialize

	default_create
		do
			create invalid_field_names_for_setter.make
			invalid_field_names_for_setter.compare_objects
			create db_unstored_field_names.make
			db_unstored_field_names.compare_objects

			Precursor {ENTITY}
			Precursor {JSON_SIT_ENTITY}
			Precursor {LOGGABLE}
			Precursor {FALLIBLE}

			set_invalid_field_names_for_setter
		ensure then
			not has_error
			not exists_in_db
		end

	make_from_db_service (a_db_service: DB_SERVICE[like Current])
			-- Creates entity with given DB_SERVICE and sets its attributes
		do
			default_create
			set_from_db_service (a_db_service)
		end

feature -- Access

	invalid_field_names_for_setter: LINKED_LIST[STRING]

feature -- Status Report

	exists_in_db: BOOLEAN
		-- Is Current object linked to DB

	valid_for_insert: BOOLEAN
		-- Is Current object valid to be inserted into DB
		-- Redefine me if you need more business
		do
			Result := True
		end

	valid_for_delete: BOOLEAN
		-- Is Current object valid to be inserted into DB
		-- Redefine me if you need more business
		do
			Result := True
		end

	valid_for_update: BOOLEAN
		-- Is Current object valid to be inserted into DB
		-- Redefine me if you need more business
		do
			Result := True
		end

	fields_mapping_has (a_db_field_s: STRING): BOOLEAN
		do
			across
				fields_mapping as l_db_field
			until
				Result
			loop
				Result := l_db_field.item.db_field_name.is_equal (a_db_field_s)
			end
		end

	class_name_lowercase: STRING
		do
			Result := {SIT_UTIL}.class_name_lowercase (({like Current}).out)
		ensure
			instance_free: class
		end

feature -- DB Field names

	fields_mapping: LINKED_SET[DB_FIELD]
			-- Fields mappings for queries
			-- Each DB_FIELD is used for JSON export and import, and for setters from DB fields read from query into DB_SERVICE
		do
			create Result.make
			Result.compare_objects
		end

feature -- Database

	table_name: STRING
			--Current DB_Entity table name
		do
			Result := {SIT_UTIL}.class_name_lowercase (generating_type.out)
		end

	db_unstored_field_names: LINKED_LIST[STRING]
			-- Fields mostly contained into fields_mapping not to be stored during update or insert

feature -- Status setting


	set_from_db_service (a_db_service: DB_SERVICE[like Current])
			-- Calls all setters from a_db_service resultset
		require else
			attached a_db_service.last_column_names
				and then attached a_db_service.cursor
		local
			i: INTEGER
			l_db_cursor: DB_TUPLE
			l_column_name: STRING
			l_value: detachable ANY
		do
			if attached a_db_service.last_column_names as l_column_names then
				create l_db_cursor.copy(a_db_service.cursor)
				if l_db_cursor.count = 0 then --TODO remove me because of performances
					create {ENTITY_NOT_FOUND} last_error.make("Given resultset is empty")
				else
					from
						i := 1
					until
						i > l_db_cursor.count
					loop
						if attached l_db_cursor.column_name (i) as l_a_column_name then
							l_column_name := l_a_column_name
							if attached l_db_cursor.item (i) as l_attached_value then
								l_value := l_attached_value.deep_twin
								if not invalid_field_names_for_setter.has (l_column_name) then -- TODO: LOW remove this concept of adding automatically the '_' fields and invalid_field_names_for_setter.prune_all (a_field)
									call_setter_from_db_field_name (l_column_name, l_value)
								end
							else
								logger.write_debug ("set_from_db_service->wont Call setter from Void value:" + l_column_name)
							end
							i := i + 1
						else
							check
								column_name_not_null: False
							end
						end
					end
					logger.write_debug ("set_from_db_service:" + out)
				end
			else
				create {ENTITY_NOT_FOUND} last_error.make("Column names are empty")
			end
		ensure
			cursor_not_moved: a_db_service.cursor.is_equal (old a_db_service.cursor)
		end

	set_invalid_field_names_for_setter
			-- Default create invalid_field_names_for_setter and adds the field_names ending with "_id"
			-- Sets invalid_field_names_for_setter for object_comparison
			-- Redefine if needed
		require
			attached fields_mapping and then fields_mapping.count > 0
		do
			across
				fields_mapping as l_f
			loop
				if l_f.item.db_field_name.ends_with ("_id") then
					invalid_field_names_for_setter.extend (l_f.item.db_field_name)
				end
			end
		end

	set_random_for_insert
			-- setting random values for insert
			-- not setting id
		do
			set_random_for_update
		ensure
			valid_for_insert
		end

	set_random_for_update
		local
			l_random: RANDOM
		do
			create l_random.make
			l_random.start
			across
				fields_mapping as l_field_c
			loop
				if attached l_field_c.item as l_field then
					l_field.call_setter ({RANDOMIZER}.random_from_any_to_any (l_field))
				end
			end
		ensure
			valid_for_update
		end

feature -- JSON Form

	to_json: JSON_OBJECT
			-- Just call the representation to get it as STRING
			-- TODO: LOW, nothing really more than I did... refactor with JSON_SERIALIZABLE https://www.youtube.com/watch?v=G9AiHaYdqzU&t=24s&list=WL&index=3
		local
			l_field_name: like {DB_FIELD}.db_field_name
			l_field_value: like {DB_FIELD}.value_to_set
		do
			create Result.make
			across
				fields_mapping as l_field
			loop
				l_field_name := l_field.item.db_field_name
				if not json_unexported_field_names.has (l_field_name) then
					check
						valid_key_for_json: l_field_name.count > 0
					end
					l_field_value := l_field.item.value_to_set
					if attached {INTEGER_REF} l_field_value as l_int then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_int:" + l_int.out)
						Result.put_integer (l_int.to_integer_64, l_field_name)
					elseif attached {INTEGER_64} l_field_value as l_int then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_int:" + l_int.out)
						Result.put_integer (l_int, l_field_name)
					elseif attached {REAL} l_field_value as l_real then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_int:" + l_int.out)
						Result.put_real ({SIT_TYPE_CONVERTER}.real_32_to_real_64(l_real), l_field_name) -- https://stackoverflow.com/questions/54374115/eiffel-real-32-to-double-gives-a-strange-value
					elseif attached {REAL_64} l_field_value as l_real then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_int:" + l_int.out)
						Result.put_real (l_real, l_field_name) -- https://stackoverflow.com/questions/54374115/eiffel-real-32-to-double-gives-a-strange-value
					elseif attached {TUPLE} l_field_value as l_tuple and then attached {INTEGER_64} l_tuple.item (1) as l_int then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_int:" + l_int.out)
						Result.put_integer (l_int, l_field_name)
					elseif attached {STRING} l_field_value as l_s then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_s:" + l_s.out)
						Result.put_string (l_s, l_field_name)
					elseif attached {STRING_32} l_field_value as l_s then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_s:" + l_s.out)
						Result.put_string (l_s.to_string_8, l_field_name)
					elseif attached {DATE_TIME} l_field_value as l then
--						logger.write_debug("l_field_name:" + l_field_name + "-l_s:" + {JSON_UTIL}.date_time_to_api_export (l))
						Result.put_string ({JSON_UTIL}.date_time_to_api_export (l), l_field_name)
					elseif attached {DATE} l_field_value as l then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_s:" + l_s.out)
						Result.put_string ({JSON_UTIL}.date_to_api_export (l), l_field_name)
					elseif attached {WATT} l_field_value as l then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_s:" + l_s.out)
						Result.put_string (l.out, l_field_name)
					elseif attached {K_WATT_H} l_field_value as l_real then
	--					logger.write_debug("l_field_name:" + l_field_name + "-l_int:" + l_int.out)
						Result.put_real (l_real.item, l_field_name) -- https://stackoverflow.com/questions/54374115/eiffel-real-32-to-double-gives-a-strange-value
					elseif attached {VOLT} l_field_value as l then
						Result.put_string (l.out, l_field_name)
					elseif attached {AMPERE} l_field_value as l then
						Result.put_string (l.out, l_field_name)
					elseif attached {NATURAL_64} l_field_value as l_v then
						Result.put_natural (l_v, l_field_name)
					else
						logger.write_error ("to_json-> Type not found in matching:" + l_field_value.out)
						check
							not_found_item_type: False
						end
					end
				end
			end
		end

	set_from_json (a_json_s: STRING)
		local
--			l_json_serialization: JSON_SERIALIZATION -- AUTOMATIC
			l_json_parser: JSON_PARSER
			l_json_util: JSON_UTIL
		do
			create l_json_util
			logger.write_information ("set_from_json->Received object:" + a_json_s + "-")
			create l_json_parser.make_with_string (a_json_s)
			l_json_parser.parse_content
			if l_json_parser.is_valid and then
					attached l_json_parser.parsed_json_value as l_parsed_json_value and then
					attached {JSON_OBJECT} l_parsed_json_value as l_parsed_json_obj
			then
				set_from_json_object (l_parsed_json_obj)
			else
				create {PARSE_ERROR} last_error.make ("Error parsing Json object from string, parser is not valid:%N" + a_json_s + "%NError:" + l_json_parser.errors_as_string)
			end
		end

	set_from_json_object (a_json_object: JSON_OBJECT)
		local
			l_setter_procedure: PROCEDURE[TUPLE]
			l_field_name: STRING
		do
			across
				fields_mapping as l_field
			loop
				l_setter_procedure := l_field.item.setter
				l_field_name := l_field.item.db_field_name
				if not invalid_field_names_for_setter.has (l_field_name) then
					if attached a_json_object.item (l_field_name) as l_field_val then
						call_setter_from_eiffel_field_name (l_field_name, l_field_val)
					else
						logger.write_debug ("set_from_json_object-> fieldname '" + l_field_name + "' not found into json object:" + a_json_object.representation) -- could be removed or let as trace
					end
				end
			end
		end

feature -- Agents for runtime setting of properties

	call_setter_from_eiffel_field_name (an_eiffel_field_name: STRING; a_value: ANY)
			--Calls corresponding setter with agent
		require
			not invalid_field_names_for_setter.has (an_eiffel_field_name)
		local
			l_found: BOOLEAN
		do
			across
				fields_mapping as l_db_field
			until
				l_found
			loop
				l_found := l_db_field.item.eiffel_field_name.is_equal (an_eiffel_field_name)
				if l_found then
					l_db_field.item.call_setter (a_value)
				end
			end
			if not l_found then
				logger.write_error ("call_setter_from_eiffel_field_name->Field:" + an_eiffel_field_name + ":" + a_value.out + "<- has not been found into the setters")
			end
			check
				field_name_added_to_fields_mapping: l_found
			end
		end

	call_setter_from_db_field_name (a_db_field_name: like {DB_FIELD}.db_field_name; a_value: ANY)
			--Calls corresponding setter with agent
		require
			not invalid_field_names_for_setter.has (a_db_field_name)
		local
			l_found: BOOLEAN
		do
			across
				fields_mapping as l_db_field
			until
				l_found
			loop
				l_found := l_db_field.item.db_field_name.is_equal (a_db_field_name)
				if l_found then
					l_db_field.item.call_setter (a_value)
				end
			end
			if not l_found then -- just debug because fields for JOINS are constructed into the DB_ENTITY
				logger.write_debug ("call_setter_from_db_field_name->Field:" + a_db_field_name + ":" + a_value.out + "<- has not been found into the setters")
			end
--			check
--				field_name_added_to_fields_mapping: l_found
--			end
		end

feature -- Out

	out: STRING
		local
			l_field: like fields_mapping.item
		do
			Result := ""
			across
				fields_mapping as field
			loop
				l_field := field.item
				Result.append (l_field.db_field_name + ":" + l_field.value_to_set.out + "|")
			end
		end

invariant
	invalid_field_names_for_setter.object_comparison
	db_unstored_field_names.object_comparison
	fields_mapping.object_comparison

end
