note
	description: "Summary description for {CHARGE_UNIT}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CHARGE_UNIT

inherit
	MEASURE_UNIT
		redefine
			value,
			set_relationships,
			default_create,
			valid_for_insert
		end

create
	default_create,
	make_from_date_time,
	make_from_json,
	make_from_json_object,
	make_from_db_service,
	make_from_csv_line

feature {NONE} -- Initialize

	default_create
		do
			create value
			Precursor
		end

	make_from_csv_line(
			a_csv_line,
			a_csv_header: DS_ARRAYED_LIST[STRING];
			a_csv_line_index: INTEGER;
			a_dt_format,
			a_csv_header_date_time_key,
			a_csv_header_value_key: STRING;
			a_measuring_point: attached like measuring_point_relationship.secondary;
			a_name: STRING;
			an_index: attached like an_index_relationship.secondary;
			a_transformation_function: detachable FUNCTION[STRING, like value];
			)
		do
			default_create
			impl_set_date_time (a_csv_line, a_csv_header, a_csv_header_date_time_key, a_csv_line_index, a_dt_format)
			-- Value
			if attached col_from_csv_line (a_csv_line, a_csv_header, a_csv_header_value_key) as l_csv_data then
				if attached a_transformation_function then
					set_value (a_transformation_function.item (l_csv_data))
				elseif l_csv_data.is_integer_64 then
					set_value (l_csv_data.to_integer_64)
				elseif l_csv_data.is_real_64 then
					set_value (l_csv_data.to_real_64.truncated_to_integer_64)
				else
					logger.write_warning ("make_from_csv_line->Error in line " + a_csv_line_index.out + " column " + a_csv_header_value_key + " value is not integer:" + l_csv_data)
					is_valid := False
				end
			else
				logger.write_warning ("make_from_csv_line->Unable to find key into header:" + a_csv_header_date_time_key)
				is_valid := False
			end

			if is_valid then
				measuring_point_relationship.set_secondary (a_measuring_point)
				an_index_relationship.set_secondary (an_index)
				name := a_name
			end
		end

feature -- DB Field names

	Charge_template_relationship_join_table_name: STRING = "charge_template_charge_unit"

feature -- Access

	value: WATT

	charge_template_relationship: MANY_TO_ONE_JOIN[like Current, CHARGE_TEMPLATE]
		attribute
			create Result.make_from_join_table_name (Current, False, Charge_template_relationship_join_table_name)
		end

feature -- Status setting	

	set_relationships
		do
			Precursor
			add_relationship (charge_template_relationship)
		end


feature -- Status report

	valid_for_insert: BOOLEAN
		-- <Precursor>
		do
			Result := Precursor

			if Result and not charge_template_relationship.primaries.is_empty then
				across
					charge_template_relationship.primaries as l_primary
				until
					not Result
				loop
					if attached {PK_DB_ENTITY} l_primary as l_pk_primary then
						Result := l_pk_primary.valid_for_insert
					end
				end
			end
		end
end
