note
	description: "Summary description for {COMPANY}."
	author: "Philippe Gachoud"
	date: "$Date$"
	revision: "$Revision$"

class
	COMPANY

inherit
	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping
		end

	API_EXPORTABLE_ENTITY
		undefine
			out
		redefine
			default_create
		end

create
	default_create,
	make_from_json,
	make_from_json_object,
	make_from_db_service

feature {NONE} -- Initialization

	default_create
		do
			name := ""
			main_email := ""
			main_phone := ""
			rut := ""
			billing_chapter := ""
			billing_address := ""

			Precursor {API_EXPORTABLE_ENTITY}
			Precursor {RELATED_PK_DB_ENTITY}
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Name_db_column_name)
			api_exported_fields.extend (Main_email_db_column_name)
			api_exported_fields.extend (Main_phone_db_column_name)
			api_exported_fields.extend (Rut_db_column_name)
			api_exported_fields.extend (Billing_chapter_db_column_name)
			api_exported_fields.extend (Billing_address_db_column_name)
		end

feature -- DB Field names

	Name_db_column_name: STRING = "name"
	Main_email_db_column_name: STRING = "main_email"
	Main_phone_db_column_name: STRING = "main_phone"
	Rut_db_column_name: STRING = "rut"
	Billing_chapter_db_column_name: STRING = "billing_chapter"
	Billing_address_db_column_name: STRING = "billing_address"

	Billing_city_identifier: STRING = "billing_city"
	Billing_city_fk: STRING = "billing_city_id"

	fields_mapping: like {DB_ENTITY}.fields_mapping
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Name_db_column_name, agent set_name (?), name, {like name}))
			Result.extend (create {DB_FIELD}.make (Main_email_db_column_name, agent set_main_email (?), main_email, {like main_email}))
			Result.extend (create {DB_FIELD}.make (Main_phone_db_column_name, agent set_main_phone (?), main_phone, {like main_phone}))
			Result.extend (create {DB_FIELD}.make (Rut_db_column_name, agent set_rut (?), rut, {like rut}))
			Result.extend (create {DB_FIELD}.make (Billing_chapter_db_column_name, agent set_billing_chapter (?), billing_chapter, {like billing_chapter}))
			Result.extend (create {DB_FIELD}.make (Billing_address_db_column_name, agent set_billing_address (?), billing_address, {like billing_address}))
		end

feature -- Access

	name: STRING

	main_email: STRING

	main_phone: STRING

	rut: RUT

	billing_chapter: STRING

	billing_address: STRING

	branches_relationship: ONE_TO_MANY[like Current, BRANCH]
		attribute
			create Result.make(Current, False)
		end

	billing_city_relationship: IDENTIFIABLE_MANY_TO_ONE[like Current, ENUMERATE]
			attribute
				create Result.make_identifiable (Current, False, Billing_city_identifier, Billing_city_fk)
			end

feature -- Status setting

	set_name (v: like name)
		do
			name := v
		ensure
			name = v
		end

	set_main_email (v: like main_email)
		do
			main_email := v
		ensure
			main_email = v
		end

	set_main_phone (v: like main_phone)
		do
			main_phone := v
		ensure
			main_phone = v
		end

	set_rut (v: like rut)
		do
			rut := v
		ensure
			rut = v
		end

	set_billing_chapter (v: like billing_chapter)
		do
			billing_chapter := v
		ensure
			billing_chapter = v
		end

	set_billing_address (v: like billing_address)
		do
			billing_address := v
		ensure
			billing_address = v
		end

	set_relationships
		do
			add_relationship (branches_relationship)
			add_relationship (billing_city_relationship)
		end


end
