note
	description: "Summary description for {CUSTOMER_INVOICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CUSTOMER_INVOICE

inherit
	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping
		end

	API_EXPORTABLE_ENTITY
		undefine
			out
		redefine
			default_create
		end

create
	default_create,
	make_from_json,
	make_from_json_object,
	make_from_db_service

feature {NONE} -- Initialization

	default_create
		do
			create a_date.make_now
			name := ""
			create consumption_amount
			Precursor {API_EXPORTABLE_ENTITY}
			Precursor {RELATED_PK_DB_ENTITY}
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Name_column_name)
			api_exported_fields.extend (Date_column_name)
			api_exported_fields.extend (Consumption_amount_db_column_name)
		end

feature -- DB Field names

	Name_column_name: STRING = "name"
	Date_column_name: STRING = "a_date"
	Consumption_amount_db_column_name: STRING = "consumption_amount"
	Branch_id_db_column_name: STRING = "branch_id"


	fields_mapping: LINKED_SET[DB_FIELD]
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Name_column_name, agent set_name (?), name, {like name}))
			Result.extend (create {DB_FIELD}.make (Date_column_name, agent set_date (?), a_date, {like a_date}))
			Result.extend (create {DB_FIELD}.make (Consumption_amount_db_column_name, agent set_consumption_amount (?), consumption_amount, {like consumption_amount}))
		end

feature -- Access


	name: STRING

	consumption_amount: K_WATT_H

	a_date: DATE

	uf_relationship: MANY_TO_ONE[like Current, UF]
		attribute
			create Result.make(Current, True)
		end

	electricity_rate_relationship: MANY_TO_ONE[like Current, ELECTRICITY_RATE]
		attribute
			create Result.make(Current, False)
		end

	branch_relationship: MANY_TO_ONE[like Current, BRANCH]
		attribute
			create Result.make(Current, True)
		end


	uf: UF
		require
			attached uf_relationship.secondary
			is_san_ignacio implies attached {APP_CONFIGURATION}.app_instance.configuration_value ("s-ignacio-uf-value")
		do
			if is_san_ignacio then
				create Result
				if attached {APP_CONFIGURATION}.app_instance.configuration_value ("s-ignacio-uf-value") as l_v then
					Result.set_item (l_v.to_double)
					Result.set_date (a_date)
				else
					check
						s_ignacio_uf_value_not_found: False
					end
				end
			else
				if attached uf_relationship.secondary as l_res then
					Result := l_res
				else
					create Result -- for void Safety but covered by require
				end
			end
		end

feature -- Status setting

	set_name (v: like name)
		do
			name := v
		ensure
			name = v
		end

	set_date (v: like a_date)
		do
			a_date := v
		ensure
			a_date = v
		end

	set_consumption_amount (v: like consumption_amount)
		do
			consumption_amount := v
		ensure
			consumption_amount = v
		end

	set_relationships
		do
			add_relationship (uf_relationship)
			add_relationship (electricity_rate_relationship)
			add_relationship (branch_relationship)
		end

feature -- Status report

	branch_pk: INTEGER_64
		require
			attached branch_relationship.secondary
		do
			if attached branch_relationship.secondary as l_v then
				Result := l_v.primary_key
			end
		end

	is_san_ignacio: BOOLEAN
			-- id configured into json file
		require
			attached {APP_CONFIGURATION}.app_instance.configuration_value ("s-ignacio-branch-pk")
		do
			if attached {APP_CONFIGURATION}.app_instance.configuration_value ("s-ignacio-branch-pk") as s_i_pk_s then
				Result := primary_key = s_i_pk_s.to_integer_64
			else
				logger.write_error ("Key s-ignacio-branch-pk not found into json file")
			end
		end

	discount_factor: like branch_relationship.secondary.discount_factor
			-- percentage
		require
			attached branch_relationship.secondary
		do
			if attached branch_relationship.secondary as l_br then
				Result := l_br.discount_factor
			end
		ensure
			Result <= 100 and Result > 0
		end

	electricity_rate_value: REAL_64
		require
			attached electricity_rate_relationship.secondary
		do
			if attached electricity_rate_relationship.secondary as l_v then
				Result := l_v.energy_rate + l_v.public_service_rate + l_v.main_transmission_rate
			end
		end

	uf_for_kWh: REAL_64
		do
			Result := clp_for_kWh / uf.item
		end

	clp_for_kWh: REAL_64
		do
			Result := electricity_rate_value * (1 - discount_factor)
		end

	total_vat: REAL_64
		do
			Result := {APP_CONFIGURATION}.app_instance.vat * total_without_vat
		end

	total_without_vat: REAL_64
		do
			Result := consumption_amount * clp_for_kwh
		end

	total_with_vat: REAL_64
		do
			Result := total_without_vat + total_vat
		end

	due_to_date: DATE
			-- 15 days after current invoice date
		do
			Result := a_date.deep_twin
			Result.month_add (1)
			Result.set_day (15)
		end

	distribution_company_price: REAL_64
		do
			Result := electricity_rate_value
		end

	sit_discount: REAL_64
		do
			Result := consumption_amount * electricity_rate_value * discount_factor
		end

invariant
	consumption_amount >= 0.0
	-- uf_month_same_as_invoice_date: a_date.month = uf.a_date.month -- sometimes date is setted improperly because of loaded from json and not from DB
end
