note
	description: "Summary description for {ELECTRICITY_RATE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	ELECTRICITY_RATE

inherit
	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping
		end

	API_EXPORTABLE_ENTITY
		undefine
			out
		redefine
			default_create
		end

create
	default_create,
	make_from_json,
	make_from_json_object,
	make_from_db_service

feature {NONE} -- Initialization

	default_create
		do
			create validity_start_date.make_now_utc
			create validity_end_date.make_now_utc
			Precursor {API_EXPORTABLE_ENTITY}
			Precursor {RELATED_PK_DB_ENTITY}
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Energy_rate_column_name)
			api_exported_fields.extend (Main_transmission_rate_db_column_name)
			api_exported_fields.extend (Public_service_rate_db_column_name)
			api_exported_fields.extend (Validity_start_date_db_column_name)
			api_exported_fields.extend (Validity_end_date_db_column_name)
		end

feature -- DB Field names

	Energy_rate_column_name: STRING = "energy_rate"
	Main_transmission_rate_db_column_name: STRING = "main_transmission_rate"
	Public_service_rate_db_column_name: STRING = "public_service_rate"

	Validity_start_date_db_column_name: STRING = "validity_start_date"
	Validity_end_date_db_column_name: STRING = "validity_end_date"

	-- relationships to Enumerates
	Electricity_type_identifier: STRING = "electricity_type"
	Electricity_type_fk: STRING = "electricity_type_id"
	City_identifier: STRING = "city"
	City_fk: STRING = "city_id"


	fields_mapping: like {DB_ENTITY}.fields_mapping
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Energy_rate_column_name, agent set_energy_rate (?), energy_rate, {like energy_rate}))
			Result.extend (create {DB_FIELD}.make (Main_transmission_rate_db_column_name, agent set_main_transmission_rate (?), main_transmission_rate, {like main_transmission_rate}))
			Result.extend (create {DB_FIELD}.make (Public_service_rate_db_column_name, agent set_public_service_rate (?), public_service_rate, {like public_service_rate}))

			Result.extend (create {DB_FIELD}.make (Validity_start_date_db_column_name, agent set_validity_start_date (?), validity_start_date, {like validity_start_date}))
			Result.extend (create {DB_FIELD}.make (Validity_end_date_db_column_name, agent set_validity_end_date (?), validity_end_date, {like validity_end_date}))
		end

feature -- Access

	energy_rate: REAL
	main_transmission_rate: like energy_rate
	public_service_rate: like energy_rate

	validity_start_date, validity_end_date: DATE

	distribution_company_relationship: MANY_TO_ONE[like Current, DISTRIBUTION_COMPANY]
		attribute
			create Result.make(Current, False)
		end

	electricity_type_relationship: IDENTIFIABLE_MANY_TO_ONE[like Current, ENUMERATE]
		attribute
			create Result.make_identifiable (Current, False, Electricity_type_identifier, Electricity_type_fk)
		end

	city_relationship: IDENTIFIABLE_MANY_TO_ONE[like Current, ENUMERATE]
		attribute
			create Result.make_identifiable (Current, False, City_identifier, City_fk)
		end

	consumption_sector_relationship: MANY_TO_ONE[like Current, CONSUMPTION_SECTOR]
		attribute
			create Result.make(Current, False)
		end

feature -- Status setting

	set_energy_rate (v: like energy_rate)
		do
			energy_rate := v
		ensure
			energy_rate = v
		end

	set_main_transmission_rate (v: like main_transmission_rate)
		do
			main_transmission_rate := v
		ensure
			main_transmission_rate = v
		end

	set_public_service_rate (v: like public_service_rate)
		do
			public_service_rate := v
		ensure
			public_service_rate = v
		end

	set_validity_start_date (v: like validity_start_date)
		do
			validity_start_date := v
		ensure
			validity_start_date = v
		end

	set_validity_end_date (v: like validity_end_date)
		do
			validity_end_date := v
		ensure
			validity_end_date = v
		end

	set_electricity_type (v: like electricity_type_relationship.secondary)
		do
			electricity_type_relationship.set_secondary (v)
		ensure
			electricity_type_relationship.secondary = v
		end

	set_city (v: like city_relationship.secondary)
		do
			city_relationship.set_secondary (v)
		ensure
			city_relationship.secondary = v
		end

feature -- Relationships

	set_relationships
			-- from deferred RELATED_DB_ENTITY called by default_create
		do
			add_relationship (city_relationship)
			add_relationship (electricity_type_relationship)

			add_relationship (distribution_company_relationship)
			--add_relationship (consumption_sector_relationship)
		end

invariant
	energy_rate >= 0.0
	main_transmission_rate >= 0.0
	public_service_rate >= 0.0

end
