note
	description: "An {ENUMERATE} has a name and belongs to a category of enumerates. If category is Void, then its a root category."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	ENUMERATE

inherit
	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping
		select
			default_create
		end

	API_EXPORTABLE_ENTITY
		rename
			default_create as default_create_aee
		undefine
			out
		end

create
	default_create,
	make,
	make_from_json,
	make_from_json_object,
	make_from_db_service


feature {NONE} -- Initialization

	default_create
		do
			name := ""
			default_create_aee
			Precursor {RELATED_PK_DB_ENTITY}

			json_unexported_field_names.extend (Category_db_column_name)
			json_unexported_field_names.extend (Category_name_join_column_name)
			json_unexported_field_names.extend (Category_id_join_column_name)

			db_unstored_field_names.extend (Category_name_join_column_name)
			db_unstored_field_names.extend (Category_id_join_column_name)

			invalid_field_names_for_setter.prune_all (Category_id_join_column_name)
			invalid_field_names_for_setter.prune_all (Category_name_join_column_name)
			invalid_field_names_for_setter.prune_all (Category_db_column_name)
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Name_db_column_name)
		end

feature -- DB Field names

	Name_db_column_name: STRING = "name"
	Category_db_column_name: STRING = "enumerate_id"
	-- JOIN
	Category_name_join_column_name: STRING = "category_name"
	Category_id_join_column_name: STRING = "category_id"

	fields_mapping: like {DB_ENTITY}.fields_mapping
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Name_db_column_name, agent set_name (?), name, {like name}))
			Result.extend (create {DB_FIELD}.make (Category_db_column_name, agent set_category_id (?), category_id, {like category_id})) -- if has an '_' prune it from invalid_field_names_for_setter
			-- JOIN
			if attached category as l_cat then -- needed for json export
				Result.extend (create {DB_FIELD}.make (Category_name_join_column_name, agent set_category_name (?), l_cat.name, {like name}))
				Result.extend (create {DB_FIELD}.make (Category_id_join_column_name, agent set_category_id (?), l_cat.primary_key, {like primary_key}))
			else -- needed for setter
				Result.extend (create {DB_FIELD}.make (Category_name_join_column_name, agent set_category_name (?), name, {like name}))
				Result.extend (create {DB_FIELD}.make (Category_id_join_column_name, agent set_category_id (?), primary_key, {like primary_key})) --dont forget to add then into valid_field_name_for_setter
			end
		end

feature -- Access

	name: STRING

	category: detachable like Current
		do
			Result := category_relationship.secondary
		end

	category_relationship: MANY_TO_ONE[like Current, like Current]
		attribute
			create Result.make(Current, False)
		end

	category_id: like primary_key
		do
			if attached category_relationship.secondary as l_sec then
				Result := l_sec.primary_key
			end
		end

feature -- Status setting

	set_name (v: like name)
		do
			name := v
		ensure
			name = v
		end

	set_category (v: like category)
		do
			category_relationship.set_secondary (v)
		ensure
			valid_category: category = v
		end

	set_category_id (v: like primary_key)
		do
			if v > 0 then
				if not attached category then
					set_category(create {like Current})
				end
				if attached category as l_cat then
					l_cat.set_primary_key(v)
				end
			end
		ensure
			v > 0 implies attached category as l_cat and then
				l_cat.primary_key = v
		end

	set_category_name (v: like name)
		do
			if not attached category then
				set_category(create {like Current})
			end
			if attached category as l_cat then
				l_cat.set_name(v)
			end
		ensure
			attached category as l_cat and then
				l_cat.name = v
		end

	set_relationships
		do
			add_relationship (category_relationship)
		end

end
