note
	description: "Summary description for {MEASURE_UNIT}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	MEASURE_UNIT


inherit
	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping,
			valid_for_insert
		end

	API_EXPORTABLE_ENTITY
		undefine
			out
		redefine
			default_create
		end

	DATE_TIME_CHARTABLE
		rename
			y_dimension_value as value
		undefine
			out
		redefine
			default_create
		end


feature {NONE} -- Initialization

	default_create
		do
			name := ""
			is_valid := True
			Precursor {DATE_TIME_CHARTABLE}
			Precursor {API_EXPORTABLE_ENTITY}
			Precursor {RELATED_PK_DB_ENTITY}
		end

	make_from_date_time (a_date_time: DATE_TIME)
		do
			default_create
			date_time := a_date_time
		ensure
			date_time = a_date_time
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Name_db_column_name)
			api_exported_fields.extend (Value_db_column_name)
			api_exported_fields.extend (Date_time_db_column_name)
			api_exported_fields.extend (An_index_identifier)
		end

	make_from_csv_line (
			a_csv_line,
			a_csv_header: DS_ARRAYED_LIST[STRING];
			a_csv_line_index: INTEGER;
			a_dt_format,
			a_csv_header_date_time_key,
			a_csv_header_value_key: STRING;
			a_measuring_point: attached like measuring_point_relationship.secondary;
			a_name: STRING;
			an_index: attached like an_index_relationship.secondary;
			a_transformation_function: detachable FUNCTION[STRING, like value];
			)
		require
			not a_csv_header.is_empty
			valid_csv_line_and_header: a_csv_line.count = a_csv_header.count
		deferred
		ensure
			is_valid implies measuring_point_relationship.secondary = a_measuring_point
			is_valid implies an_index_relationship.secondary = an_index
			is_valid implies name = a_name
		end

feature -- DB Field names

	Name_db_column_name: STRING = "name"
	Value_db_column_name: STRING = "value"
	Date_time_db_column_name: STRING = "timestamp"
	Measuring_point_id_column_name: STRING = "measuring_point_id"
	An_index_identifier: STRING = "an_index"
	An_index_fk: STRING = "an_index_id"

	fields_mapping: LINKED_SET[DB_FIELD]
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Name_db_column_name, agent set_name (?), name, {like name}))
			Result.extend (create {DB_FIELD}.make (Value_db_column_name, agent set_value (?), value, {like value}))
			Result.extend (create {DB_FIELD}.make (Date_time_db_column_name, agent set_date_time (?), date_time, {like date_time}))
		end

feature -- Access

	name: STRING

	measuring_point_relationship: MANY_TO_ONE[like Current, MEASURING_POINT]
		attribute
			create Result.make (Current, False)
		end

	an_index_relationship: IDENTIFIABLE_MANY_TO_ONE[like Current, ENUMERATE]
		attribute
			create Result.make_identifiable (Current, False, An_index_identifier, An_index_fk)
		end

	is_valid: BOOLEAN

feature -- Status setting

	set_name (v: like name)
		do
			name := v
		ensure
			name = v
		end

	set_value (v: like value)
		do
			value := v
		ensure
			value = v
		end

	set_time_zone_offset (a_time_zone_offset: INTEGER)
			-- a_time_zone_offset in minutes (West is negative, East is positive)
		do
			date_time.minute_add (a_time_zone_offset)
		end

	set_measuring_point_relationship (a_relationship: like measuring_point_relationship)
		do
			measuring_point_relationship := a_relationship
		end

	set_an_index (v: like an_index_relationship.secondary)
		do
			an_index_relationship.set_secondary (v)
		ensure
			an_index_relationship.secondary = v
		end

	set_relationships
		do
			add_relationship (measuring_point_relationship)
			add_relationship (an_index_relationship)
		end

feature -- Validation

	valid_for_insert: BOOLEAN
		do
			Result := Precursor and then
				is_valid and then
				date_time > create {DATE_TIME}.make (2018, 12, 12, 0, 0, 0)
		end
feature {NONE} -- Implementation

	impl_set_date_time (a_csv_line, a_csv_header_line: DS_ARRAYED_LIST [STRING]; a_csv_header_date_time_key: STRING; a_csv_line_index: INTEGER; a_date_time_format: STRING )
		local
			l_date_time_format: like a_date_time_format
			l_dt: like date_time
		do
			-- Datetime
			if attached col_from_csv_line (a_csv_line, a_csv_header_line, a_csv_header_date_time_key) as l_csv_data then
				l_date_time_format := a_date_time_format.deep_twin
				if a_date_time_format.ends_with ("Z") then
					l_date_time_format.replace_substring_all ("Z", "")
					l_csv_data.replace_substring_all ("Z", "")
				end
				if a_date_time_format.has ('T') then
					l_date_time_format.replace_substring_all ("T", " ")
					l_csv_data.replace_substring_all ("T", " ")
				end
				create l_dt.make_now_utc
				if l_dt.date_time_valid (l_csv_data, l_date_time_format) then
					create l_dt.make_from_string (l_csv_data, l_date_time_format)
					set_date_time (l_dt)
				else
					logger.write_warning ("Error in line " + a_csv_line_index.out + " datetime column invalid data:" + l_csv_data)
					is_valid := False
				end
			else
				logger.write_warning ("Unable to find key into header:" + a_csv_header_date_time_key)
				is_valid := False
			end
		end

	col_from_csv_line (a_csv_line, a_header_line: DS_ARRAYED_LIST [STRING]; a_key: STRING): detachable STRING
		local
			l_found_index: INTEGER
		do
			l_found_index := -1
			across
				a_header_line as l_i
			until
				l_found_index >= 1
			loop
				if l_i.item.is_equal (a_key) then
					l_found_index := l_i.index
				end
			end
			if l_found_index >= 1 and then l_found_index <= a_header_line.count then
				Result := a_csv_line.at (l_found_index)
			else
				do_nothing
			end
		end

end
