note
	description: "Summary description for {MEASURING_POINT}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	MEASURING_POINT
inherit
	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping
		end

	API_EXPORTABLE_ENTITY
		undefine
			out
		redefine
			default_create
		end

create
	default_create,
	make,
	make_from_json,
	make_from_json_object,
	make_from_db_service


feature {NONE} -- Initialization

	default_create
		do
			name := ""
			comment := ""
			Precursor {API_EXPORTABLE_ENTITY}
			Precursor {RELATED_PK_DB_ENTITY}
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Name_db_column_name)
--			api_exported_fields.extend (Comment_db_column_name)
		end

feature -- DB Field names

	Name_db_column_name: STRING = "name"
	Comment_db_column_name: STRING = "comment"

	Type_identifier: STRING = "type"
	Type_fk: STRING = "a_type_id"

	fields_mapping: like {DB_ENTITY}.fields_mapping
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Name_db_column_name, agent set_name (?), name, {like name}))
			Result.extend (create {DB_FIELD}.make (Comment_db_column_name, agent set_comment (?), comment, {like comment}))
		end

feature -- Access

	name: STRING

	comment: STRING

	consumption_sector_relationship: MANY_TO_ONE[like Current, CONSUMPTION_SECTOR]
		attribute
			create Result.make(Current, True)
		end

	charge_unit_relationship: ONE_TO_MANY[like Current, CHARGE_UNIT]
		attribute
			create Result.make(Current, False)
		end

	tension_unit_relationship: ONE_TO_MANY[like Current, TENSION_UNIT]
		attribute
			create Result.make(Current, False)
		end

	current_unit_relationship: ONE_TO_MANY[like Current, CURRENT_UNIT]
		attribute
			create Result.make(Current, False)
		end

	energy_unit_relationship: ONE_TO_MANY[like Current, ENERGY_UNIT]
		attribute
			create Result.make (Current, False)
		end

	type_relationship: IDENTIFIABLE_MANY_TO_ONE[like Current, ENUMERATE]
		attribute
			create Result.make_identifiable (Current, False, Type_identifier, Type_fk)
		end

feature -- Status report

	has_charge_units: BOOLEAN
		do
			Result := not charge_unit_relationship.secondaries.is_empty
		end

	has_tension_units: BOOLEAN
		do
			Result := not tension_unit_relationship.secondaries.is_empty
		end

feature -- Status setting

	set_name (v: STRING)
		do
			name := v
		ensure
			name = v
		end

	set_comment (v: STRING)
		do
			comment := v
		ensure
			comment = v
		end

	set_relationships
		do
			add_relationship (consumption_sector_relationship)
			add_relationship (charge_unit_relationship)
			add_relationship (tension_unit_relationship)
			add_relationship (current_unit_relationship)
			add_relationship (energy_unit_relationship)
			add_relationship (type_relationship)
		end

feature -- Element change

	extend_measure_unit (a_measure_unit: MEASURE_UNIT)
			-- Adding measure_unit only if is_valid
		do
			if a_measure_unit.is_valid then
				if attached {CHARGE_UNIT} a_measure_unit as l_u then
					charge_unit_relationship.secondaries.extend (l_u)
				elseif attached {CURRENT_UNIT} a_measure_unit as l_u then
					current_unit_relationship.secondaries.extend (l_u)
				elseif attached {TENSION_UNIT} a_measure_unit as l_u then
					tension_unit_relationship.secondaries.extend (l_u)
				else
					check
						not_implemented: False
					end
				end
			end
		end

end
