note
	description: "Summary description for {BRANCH}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	SIT_EVENT

inherit
	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping,
			valid_for_insert,
			table_name,
			set_from_json
		end

	API_EXPORTABLE_ENTITY
		undefine
			out
		redefine
			default_create
		end

create
	default_create,
	make,
	make_from_json,
	make_from_json_object,
	make_from_db_service


feature {NONE} -- Initialization

	default_create
		do
			title := ""
			Content := ""
			create submited_date_time.make_now_utc
			create happening_date_time.make_now_utc
			Precursor {API_EXPORTABLE_ENTITY}
			Precursor {RELATED_PK_DB_ENTITY}
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Title_db_column_name)
			api_exported_fields.extend (Content_db_column_name)
			api_exported_fields.extend (Submited_timestamp_db_column_name)
			api_exported_fields.extend (Happening_timestamp_db_column_name)
		end

feature -- Status report

	valid_for_insert: BOOLEAN
		-- Is Current object valid to be inserted into DB
		do
			Result := Precursor and then
				title.count > 0

			if Result then
				if attached status_relationship.secondary as l_sec then
					Result := l_sec.primary_key > 0
				else
					Result := False -- Mandatory secondary status
				end
			end
			if Result and then
				attached entities_relationship.secondaries as l_sec
			then
				across
					l_sec as l_item
				until
					not Result
				loop
					Result := l_item.item.primary_key > 0
				end
			end
		end

feature -- DB Field names

	Title_db_column_name: STRING = "title"
	Content_db_column_name: STRING = "content"
	Submited_timestamp_db_column_name: STRING = "submited_timestamp"
	Happening_timestamp_db_column_name: STRING = "happening_timestamp"

	Status_identifier: STRING = "status"
	Status_fk: STRING = "status_id"

	Event_entity_join_table_name: STRING = "event_entity"
	Entities_identifier: STRING = "entities"
	Entity_class_json_key: STRING = "class"


	fields_mapping: like {DB_ENTITY}.fields_mapping
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Title_db_column_name, agent set_title (?), title, {like title}))
			Result.extend (create {DB_FIELD}.make (Content_db_column_name, agent set_content (?), content, {like content}))
			Result.extend (create {DB_FIELD}.make (Submited_timestamp_db_column_name, agent set_submited_date_time (?), submited_date_time, {like submited_date_time}))
			Result.extend (create {DB_FIELD}.make (Happening_timestamp_db_column_name, agent set_happening_date_time (?), happening_date_time, {like happening_date_time}))
		end

feature -- Access

	title: STRING
	content: STRING_32
	happening_date_time: DATE_TIME
	submited_date_time: DATE_TIME

	table_name: STRING = "event"


	status_relationship: IDENTIFIABLE_MANY_TO_ONE[like Current, ENUMERATE]
		attribute
			create Result.make_identifiable (Current, False, Status_identifier, Status_fk)
		end


	entities_relationship: ONE_TO_MANY_JOIN[like Current, PK_DB_ENTITY]
		attribute
			create Result.make_from_join_table_name (Current, False, Event_entity_join_table_name)
		end


feature -- Status setting

	set_title (v: like title)
		do
			title := v
		ensure
			title = v
		end

	set_content (v: like content)
		do
			content := v
		ensure
			content = v
		end

	add_entity (v: like entities_relationship.secondaries.item)
		do
			entities_relationship.secondaries.extend (v)
		ensure
			entities_relationship.secondaries.count = old entities_relationship.secondaries.count + 1
		end

	set_submited_date_time (v: like submited_date_time)
		do
			submited_date_time := v
		ensure
			submited_date_time = v
		end

	set_happening_date_time (v: like happening_date_time)
		do
			happening_date_time := v
		ensure
			happening_date_time = v
		end

	set_status (a_status: attached like status_relationship.secondary)
		do
			status_relationship.set_secondary (a_status)
		end

	set_relationships
		do
			add_relationship (status_relationship)
			add_relationship (entities_relationship)
		end

	set_from_json (a_json_s: STRING)
			-- Adds the parsing of relationships and sets them
		local
			l_json_parser: JSON_PARSER
			l_status: ENUMERATE
		do
			Precursor {RELATED_PK_DB_ENTITY} (a_json_s) -- TODO: LOW Improvement: object is parsed 2 times with precursor, parse it only one time
			create l_json_parser.make_with_string (a_json_s)
			l_json_parser.parse_content
			if l_json_parser.is_valid
					and then attached l_json_parser.parsed_json_value as l_parsed_json_val then
				if attached {JSON_OBJECT} l_parsed_json_val as l_json_object then
					-- Status
					if attached {JSON_OBJECT} l_json_object.item (status_identifier) as l_status_json_o then
						if attached {ENUMERATE_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable ENUMERATE}) as l_enum_dbs then
							if attached {JSON_STRING} l_status_json_o.item ({ENUMERATE}.name_db_column_name) as l_enum_name then
								l_status := l_enum_dbs.enumerate_cash_from_name (l_enum_name.unescaped_string_8)
								if l_status.primary_key > 0 then
									status_relationship.set_secondary (l_status)
								else
									check
										not_happening_pk_zero: False
									end
								end
							end
						else
							check
								enumerate_db_service_not_found: False
							end
						end
					end
					-- Entities
					if attached {JSON_OBJECT} l_json_object.item (Entities_identifier) as l_entity then
						add_entity_from_json_o (l_entity)
					elseif attached {JSON_ARRAY} l_json_object.item (Entities_identifier) as l_entities_array then
						across
							l_entities_array as l_item
						loop
							if attached {JSON_OBJECT} l_item.item as l_entity then
								add_entity_from_json_o (l_entity)
							else
								check
									should_not_be_something_else: False
								end
							end
						end
					end
				end
			end
		end

feature {NONE} -- Implementation

	add_entity_from_json_o (a_json_o: JSON_OBJECT)
		local
			l_class_s: STRING
			l_pkdbe: PK_DB_ENTITY
		do
			if attached {JSON_NUMBER} a_json_o.item ({PK_DB_ENTITY}.Primary_key_db_column_name) as l_pk and then
				attached {JSON_STRING} a_json_o.item (Entity_class_json_key) as l_class
			then
				l_class_s := l_class.unescaped_string_8
				l_pkdbe := {SIT_UTIL}.pk_db_entity_from_class_name (l_class_s, l_pk.integer_64_item)
				entities_relationship.secondaries.extend (l_pkdbe)
			else
				check
					wrong_object_passed: False
				end
			end
		ensure
			old entities_relationship.secondaries.count = entities_relationship.secondaries.count - 1
		end

end
