note
	description: "Summary description for {UF}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	UF


inherit
	RANDOMABLE
		redefine
			default_create,
			out
		end

	RELATED_PK_DB_ENTITY
		redefine
			default_create,
			fields_mapping,
			out
		end


	API_EXPORTABLE_ENTITY
		undefine
			out
		redefine
			default_create
		end

create
	default_create,
	make,
	make_from_real_64,
	make_from_string,
	make_random,
	make_from_json,
	make_from_json_object,
	make_from_db_service

convert
	make_from_real_64 ({REAL_64}),
	make_from_string ({STRING}),
	to_real_64: {REAL_64}

feature {NONE} -- Initialization

	default_create
		do
			set_item(0)
			create a_date.make_now_utc
			name := ""
			Precursor {RANDOMABLE}
			Precursor {API_EXPORTABLE_ENTITY}
			Precursor {RELATED_PK_DB_ENTITY}
		end

	make_from_real_64 (v: REAL_64)
		require
			valid_value (v)
		do
			set_item (v)
			default_create
		end

	make_from_string (v: STRING)
		require
			v_is_integer: v.is_real_64
			valid_v_value: valid_value (v.to_real_64)
		do
			set_item (v.to_real_64)
			default_create
		end

	make_random
		local
			l_random: RANDOM
		do
			create l_random.make
			default_create
			set_item (l_random.item.to_double)
		end

	add_exported_fields
		do
			api_exported_fields.extend (Primary_key_db_column_name)
			api_exported_fields.extend (Name_db_column_name)
			api_exported_fields.extend (A_date_db_column_name)
			api_exported_fields.extend (Value_db_column_name)
		end

feature -- DB Field names

	Name_db_column_name: STRING = "name"
	A_date_db_column_name: STRING = "a_date"
	Value_db_column_name: STRING = "value"

	fields_mapping: like {DB_ENTITY}.fields_mapping
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Name_db_column_name, agent set_name (?), name, {like name}))
			Result.extend (create {DB_FIELD}.make (A_date_db_column_name, agent set_date (?), a_date, {like a_date}))
			Result.extend (create {DB_FIELD}.make (Value_db_column_name, agent set_item (?), item, {like item}))
		end

feature -- Access

	name: STRING

	item: REAL_64

	a_date: DATE

	customer_invoices_relationship: ONE_TO_MANY[like Current, CUSTOMER_INVOICE]
		attribute
			create Result.make (Current, False)
		end

feature -- Status setting

	set_item (v: like item)
		require
			valid_value (v)
		do
			item := v
		ensure
			item = v
		end

	set_date (v: like a_date)
		do
			a_date := v
		ensure
			a_date = v
		end

	set_name (v: like name)
		do
			name := v
		ensure
			name = v
		end

	set_relationships
		do
			relationships.extend (customer_invoices_relationship)
		end

feature -- Status report

	to_real_64: REAL_64
		do
			Result := item
		end

	valid_value (v: REAL_64): BOOLEAN
		do
			Result := v >= 0.0
		end

	out: STRING
		do
			Result := item.out + " " + Precursor {RELATED_PK_DB_ENTITY}
		end

invariant
	valid_value (item)

end
