note
	description: "Summary description for {PK_DB_ENTITY}. Created to be able to have an instance of DB_ENTITY without making not deferred DB_ENTITY class"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	PK_DB_ENTITY

inherit
	DB_ENTITY
		redefine
			set_random_for_update,
			set_random_for_insert,
			fields_mapping,
			set_from_db_service,
			valid_for_insert,
			valid_for_delete,
			valid_for_update
		end

	CONSTRAINTABLE
		rename
			constraint as primary_key_constraint
		undefine
			default_create,
			out
		end

	CONSTRAINTABLE
		rename
			constraint as primary_key_constraint_join
		undefine
			default_create,
			out
		select
			primary_key_constraint_join
		end

create
	default_create,
	make,
	make_from_db_service,
	make_from_json

feature {NONE} -- Initialize

	make (a_pk: like primary_key)
			-- Creates entity with given DB_SERVICE and sets its attributes
		require
			valid_primary_key: valid_primary_key (a_pk)
		do
			primary_key := a_pk
			default_create
		end


feature -- Access

	sequence_name: STRING
			-- Used to get latest inserted ID
			-- Beware that with postgresql the sequence name is the table name from which you inherit
		do
			Result := table_name
		end

	primary_key: INTEGER_64 -- most of the time it's the id field

feature -- DB Field names

	Primary_key_db_column_name: STRING = "id"

	fields_mapping: LINKED_SET[DB_FIELD]
			-- Fields mappings for queries
			-- Each DB_FIELD is used for JSON export and import, and for setters from DB fields read from query into DB_SERVICE
		do
			Result := Precursor
			Result.extend (create {DB_FIELD}.make (Primary_key_db_column_name, agent set_primary_key (?), primary_key, {like primary_key}))
		end

feature -- Status setting


	set_primary_key (v: like primary_key)
		do
			primary_key := v
		ensure
			primary_key = v
		end

	set_from_db_service (a_db_service: DB_SERVICE[like Current])
			-- Calls all setters from a_db_service resultset
		do
			Precursor (a_db_service)
		ensure then
			valid_last_error: not valid_primary_key (primary_key) implies attached last_error
			valid_last_error: primary_key <= 0 implies attached last_error
		end

	set_random_for_update
		local
			l_random: RANDOM
		do
			create l_random.make
			l_random.start
			across
				fields_mapping as l_field_c
			loop
				if attached l_field_c.item as l_field and then
					not l_field.db_field_name.is_equal (Primary_key_db_column_name)
				then
					l_field.call_setter ({RANDOMIZER}.random_from_any_to_any (l_field))
				end
			end
		ensure then
			pk_unchanged: primary_key = old primary_key
		end


	set_random_for_insert
			-- setting random values for insert
			-- not setting id
		do
			Precursor
			set_primary_key (0)
		ensure then
			primary_key = 0
		end

feature -- Status report

	primary_key_constraint: CONSTRAINT
		require else
			valid_primary_key (primary_key)
		do
			create {FIELD_CONSTRAINT} Result.make (table_name, Primary_key_db_column_name, {CONSTRAINT}.Equal_operator, primary_key)
		end

	primary_key_constraint_join: CONSTRAINT
		require else
			valid_primary_key (primary_key)
		do
			create {FIELD_CONSTRAINT} Result.make (table_name + "_join", Primary_key_db_column_name, {CONSTRAINT}.Equal_operator, primary_key)
		end


	valid_for_insert: BOOLEAN
		-- Is Current object valid to be inserted into DB
		-- Redefine me if you need more business
		do
			Result := primary_key = 0
		end

	valid_for_delete: BOOLEAN
		-- Is Current object valid to be inserted into DB
		-- Redefine me if you need more business
		do
			Result := primary_key > 0
		end

	valid_for_update: BOOLEAN
		-- Is Current object valid to be updated from DB
		-- Redefine me if you need more business
		do
			Result := primary_key > 0
		end

	valid_primary_key (a_primary_key: like primary_key): BOOLEAN
		do
			Result := a_primary_key >= 0
		end

invariant
--	valid_primary: valid_primary_key (primary_key)
	not_both_valid: not (valid_for_insert and valid_for_update)
end
