note
	description: "Summary description for {RELATED_DB_ENTITY}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	RELATED_DB_ENTITY

inherit
	DB_ENTITY
		redefine
			default_create,
			make_from_db_service,
			set_from_db_service,
			to_json
		end

feature {NONE} -- Initialize

	default_create
		do
			to_json_relationships_level_max := 1
			create relationships.make
			Precursor
			set_relationships
		end

	make_from_db_service (a_db_service: DB_SERVICE[like Current])
			-- If identifiable relationships are found sets them
			-- a_db_service.item_prototype should contain the relationships setted to .is_loadable_from_resultset to look for before calling this			
		do
			Precursor (a_db_service)
		ensure then
			cursor_not_moved: a_db_service.cursor.is_equal (old a_db_service.cursor)
		end

feature -- Access

	relationships: LINKED_SET[RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]]

	to_json_relationships_level_max: INTEGER

	to_json_relationships_level_index: INTEGER

feature -- Status report

	relationship_from_type_conformance (a_type: TYPE[detachable like relationships.item]): detachable like relationships.item
			-- Returns first instance of found relationship which conforms to given one
		do
			across
				relationships as l_rel
			until
				Result /= Void
			loop
				if attached (a_type / l_rel.item) then
					Result := l_rel.item
				end
			end
		end

	relationship_from_secondary_type_equal (a_type: like relationships.item.secondary_type): detachable like relationships.item
			-- Returns first instance of found relationship secondary type which equals given one
			-- New implementation with
		local
			l_f, l_s: like {SIT_UTIL}.class_name_lowercase
--			l_f, l_s: like a_type
		do
			across
				relationships as l_rel
			until
				Result /= Void
			loop
--				if attached (a_type / l_rel.item.secondary_type) then -- TODO remove if no problem with new definition

--				if a_type.is_equal (l_rel.item.secondary_type) then -- tried but as is_equal needs a like Current => Catcall
--				if a_type.equal (a_type, l_rel.item.secondary_type) then -- Catcall because b signature is like a

				l_f := {SIT_UTIL}.class_name_lowercase (a_type.out)
				l_s := {SIT_UTIL}.class_name_lowercase (l_rel.item.secondary_type.out)
				if l_f.is_equal (l_s) then

--				l_f := a_type
--				l_s := l_rel.item.secondary_type
--				if l_f.conforms_to (l_s) and l_s.conforms_to (l_f) then -- tried but as is_equal needs a like Current => Catcall
					Result := l_rel.item
				end
			end
			check
				not_found_relationship: Result /= Void
			end
		end

	relationship_where_secondary_is_type (a_type_s: STRING): detachable like relationships.item
		require
			not a_type_s.is_empty
		do
			across
				relationships as l_rel
			until
				Result /= Void
			loop
				if {SIT_UTIL}.class_name_lowercase (l_rel.item.secondary_type.out).is_case_insensitive_equal (a_type_s) then
					Result := l_rel.item
				end
			end
		ensure
			valid_result: Result /= Void implies {SIT_UTIL}.class_name_lowercase (Result.secondary_type.out).is_case_insensitive_equal (a_type_s)
		end

	relationship_where_secondary_is_type_identifiable (a_type_s, an_identifier: STRING): detachable RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]
			-- returns the relationship identified by givent a_type_s and identifier is an_identifier
		require
			not a_type_s.is_empty
			not an_identifier.is_empty
		do
			across
				relationships as l_rel
			until
				Result /= Void
			loop
				logger.write_debug ("relationship_where_secondary_is_type_identifiable:" + {SIT_UTIL}.class_name_lowercase (l_rel.item.secondary_type.out) + "=>a_type_s:" + a_type_s)
				if {SIT_UTIL}.class_name_lowercase (l_rel.item.secondary_type.out).is_case_insensitive_equal (a_type_s) and then
					attached {IDENTIFIABLE} l_rel.item as l_rel_ident and then
					l_rel_ident.identifier.is_equal (an_identifier)
				then
					Result := l_rel.item
				end
			end
		ensure
			attached Result implies
				attached {IDENTIFIABLE} Result as res_ident and then
				res_ident.identifier.is_equal (an_identifier)
		end

	relationship_where_secondary_is_identifiable (an_identifier: STRING): detachable like relationships.item
			-- Returns the found relationship which identifies with given an_identifier
		require
			not an_identifier.is_empty
		do
			across
				relationships as l_rel
			until
				Result /= Void
			loop
				if
					attached {IDENTIFIABLE} l_rel.item as l_rel_ident and then
					l_rel_ident.identifier.is_equal (an_identifier)
				then
					Result := l_rel.item
				end
			end
		ensure
			attached Result implies attached {IDENTIFIABLE} Result as res_i and then res_i.identifier.is_equal (an_identifier)
		end

feature -- Status setting

	set_to_json_relationships_level_index (v: like to_json_relationships_level_index)
		do
			to_json_relationships_level_index := v
		ensure
			to_json_relationships_level_index = v
		end

	set_to_json_relationships_level_max (v: like to_json_relationships_level_max)
		do
			to_json_relationships_level_max := v
		ensure
			to_json_relationships_level_max = v
		end

	set_relationships_from_db_service (a_db_service: DB_SERVICE[like Current])
			-- Sets relationships with identifiable relationships on given a_db_service. If not found do_nothing
			-- a_db_service.item_prototype should contain the relationships setted to .is_loadable_from_resultset to look for before calling this
		require
			attached a_db_service.last_column_names
				and then attached a_db_service.cursor
		local
			l_column_name, l_id_column_name: STRING
			l_entity_id_setted: BOOLEAN
			l_s_proto: DB_ENTITY
		do
			-- across relationships
			across
				relationships as l_rel_c
			loop
				l_s_proto := l_rel_c.item.secondary_prototype -- avoid creating an instance each time
				if attached {API_EXPORTABLE_ENTITY} l_s_proto as l_exportable_entity then -- If exportable search for relationships
					set_relationship_loadable_from_resultset (a_db_service.item_prototype)
					if l_rel_c.item.loadable_from_resultset then
						across
							l_exportable_entity.api_exported_fields as l_exported_fields_c
						loop
							--setting column_name to call
							if attached {IDENTIFIABLE} l_rel_c.item as l_ident_rel then
								l_column_name := l_ident_rel.identifier + "_" + l_exported_fields_c.item
								logger.write_debug ("set_relationships_from_db_service-> column_name from identifiable:" + l_column_name)
							elseif attached {ONE_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rel_c.item as l_loadable_rel then
								l_column_name := l_loadable_rel.secondary_join_as_field_name (l_exported_fields_c.item)
								logger.write_debug ("set_relationships_from_db_service-> column_name from one_to_one:" + l_column_name)
							elseif attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rel_c.item as l_rel_1 then
								l_column_name := l_rel_1.secondary_join_as_field_name (l_exported_fields_c.item)
								logger.write_debug ("set_relationships_from_db_service-> column_name from many_to_one:" + l_column_name)
							elseif attached {ONE_TO_MANY_JOIN[RELATED_PK_DB_ENTITY, PK_DB_ENTITY]} l_rel_c.item as l_rel then -- CUSTOMER_INVOICE->
								l_column_name := l_rel.secondary_join_as_field_name (l_exported_fields_c.item)
								logger.write_debug ("set_relationships_from_db_service-> column_name from one_to_many_join:" + l_column_name)
							else
								l_column_name := "" -- for next call to data_from_column_name to be attached
								logger.write_debug ("set_relationships_from_db_service->Untreated relationship, seems to be neither loadable, and present into resultset, relationship which could be required to load it from query:" + l_rel_c.item.generating_type.out)
								check
									untreated_relationship_type: False
								end
							end
							-- find relationship into db_result and call setter for secondary
							if attached {TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rel_c.item as l_to_one_rel then
								if l_to_one_rel.loadable_from_resultset then
									if attached a_db_service.data_from_column_name (l_column_name) as l_data then
										if not attached l_to_one_rel.secondary then
											l_to_one_rel.create_default_secondary
										end
										if attached {PK_DB_ENTITY} l_to_one_rel.secondary as l_rel_sec then
											-- set id
											if not l_entity_id_setted then
												if attached {IDENTIFIABLE} l_to_one_rel as l_ident_sec then
													l_id_column_name := l_ident_sec.foreign_key
												else
													l_id_column_name := l_to_one_rel.secondary_join_as_field_name (l_rel_sec.Primary_key_db_column_name)
												end
												if attached a_db_service.data_from_column_name (l_id_column_name) as l_pk_data then
													l_rel_sec.set_primary_key ({SIT_TYPE_CONVERTER}.force_as_integer_64 (l_pk_data))
													l_entity_id_setted := True
												else
													logger.write_error ("set_relationships_from_db_service-> query doesn't contain the primary key of secondary which column_name is " + l_id_column_name)
													check
														not_found_id_for_secondary: False
													end
												end
											end
											-- set current field
											logger.write_debug ("set_relationships_from_db_service-> setting secondary " + l_rel_sec.generating_type.out + " fieldname:" + l_exported_fields_c.item + " to:" + l_data.out)
											l_rel_sec.call_setter_from_db_field_name (l_exported_fields_c.item, l_data)
										else
											check
												do_i_have_to_implement_something_else: False
											end
										end
--									else -- data for column is empty
									end
								else
									check
										not_happening: False
									end
								end

							elseif attached {ONE_TO_MANY_JOIN[RELATED_PK_DB_ENTITY, PK_DB_ENTITY]} l_rel_c.item as l_rel then
								set_otm_relationship_from_db_service_impl (a_db_service, l_rel, l_column_name, l_exported_fields_c.item, l_exported_fields_c.is_last)
							elseif attached {MANY_TO_ONE_JOIN[RELATED_PK_DB_ENTITY, PK_DB_ENTITY]} l_rel_c.item as l_rel then
								set_mto_relationship_from_db_service_impl (a_db_service, l_rel, l_column_name, l_exported_fields_c.item, l_exported_fields_c.is_last)
							else
								logger.write_debug ("set_relationships_from_db_service->relationship with key:" + l_column_name + " not found into db results of " + l_rel_c.item.generating_type.out + " %NlastQry:" + a_db_service.last_query)
--								check
--									not_found_relationship_in_db_even_if_loadable_or_identifiable: False -- Happening with SIT_EVENT and associated entities
--								end
							end
						end -- across
						l_entity_id_setted := False
					end -- if loadable
				else
					logger.write_debug ("set_relationships_from_db_service->Untreated relationship not an API_EXPORTABLE_ENTITY for secondary:" + l_rel_c.item.generating_type.out)
--					check
--						secondary_entity_cannot_be_retreived_from_db_fields_if_not_exportable: False
--					end
				end
			end -- loop
		ensure
			cursor_unmoved: a_db_service.cursor.is_equal (old a_db_service.cursor)
		end

	set_from_db_service (a_db_service: DB_SERVICE[like Current])
		do
			Precursor (a_db_service)
			set_relationships_from_db_service (a_db_service)
		end

	set_relationships
			-- called by default_create to add relationships
		deferred
		end

	add_relationship (a_relationship: like relationships.item)
		do
			relationships.extend (a_relationship)
		end

	set_relationship_loadable_from_resultset (other: RELATED_DB_ENTITY)
			-- if found into other.relationships sets loadable_from_resultset
		do
			set_relationship_loadable_from_resultset_list (other.relationships)
		end

	set_relationship_loadable_from_resultset_list (a_list: LINKED_LIST[RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]])
		local
			l_loadable: BOOLEAN
		do
			across
				relationships as l_rel
			loop
				across
					a_list as l_o_rel
				loop
					if l_rel.item.is_types_equal (l_o_rel.item) then
						l_loadable := l_o_rel.item.loadable_from_resultset
						l_rel.item.set_loadable_from_resultset (l_loadable)
					end
				end
			end
		end

feature -- JSON Form

	to_json: JSON_OBJECT
			-- Just call the representation to get it as STRING
		local
			l_sec_as_json_o: like to_json
			l_json_s_to_put: STRING
			l_json_array: JSON_ARRAY
		do
			Result := Precursor
			across
				relationships as l_relationship
			until
				to_json_relationships_level_index = to_json_relationships_level_max
			loop
				create l_json_array.make_empty
				if attached {ONE_TO_MANY[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship.item as l_rel then
					across
						l_rel.secondaries as l_secondary
					loop
						if attached {RELATED_DB_ENTITY} l_secondary.item as l_sec_rdbe then
							l_sec_rdbe.set_to_json_relationships_level_index (to_json_relationships_level_index + 1)
						end
						l_sec_as_json_o := l_secondary.item.to_json
						l_json_array.extend (l_sec_as_json_o)
					end
					if l_json_array.count > 0 then
						Result.put (l_json_array, l_rel.secondary_prototype.json_object_name)
					end
				elseif attached {ONE_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship.item as l_rel then
					if attached l_rel.secondary as l_sec then
						if attached {RELATED_DB_ENTITY} l_sec as l_sec_rdbe then
							l_sec_rdbe.set_to_json_relationships_level_index (to_json_relationships_level_index + 1)
						end
						l_sec_as_json_o := l_sec.to_json
						l_json_s_to_put := l_sec.json_object_name
						if attached {IDENTIFIABLE} l_relationship.item as l_rel_ident then
							l_json_s_to_put := l_rel_ident.identifier
						end
						Result.put (l_sec_as_json_o, l_json_s_to_put)
					end
				elseif attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship.item as l_rel then
					if attached l_rel.secondary as l_sec then
						if attached {RELATED_DB_ENTITY} l_sec as l_sec_rdbe then
							l_sec_rdbe.set_to_json_relationships_level_index (to_json_relationships_level_index + 1)
						end
						l_sec_as_json_o := l_sec.to_json
						if attached {IDENTIFIABLE} l_relationship.item as l_ident then
							l_json_s_to_put := l_ident.identifier
						else
							l_json_s_to_put := l_sec.json_object_name
						end
						Result.put (l_sec_as_json_o, l_json_s_to_put)
					end
				else
					check
						untreated_relationship: False
					end
				end
			end
			if to_json_relationships_level_index <= to_json_relationships_level_max then -- For invariant
				to_json_relationships_level_index := to_json_relationships_level_index + 1
			end
		ensure then
			to_json_relationships_level_index <= to_json_relationships_level_max implies to_json_relationships_level_index = old to_json_relationships_level_index + 1
		end

feature {NONE} -- Implementation

	to_add_dbe: detachable PK_DB_ENTITY

	set_otm_relationship_from_db_service_impl (
		a_db_service: DB_SERVICE[like Current];
		a_rel: ONE_TO_MANY_JOIN[RELATED_PK_DB_ENTITY, PK_DB_ENTITY];
		a_column_name: STRING;
		an_exported_field: like {API_EXPORTABLE_ENTITY}.api_exported_fields.item;
		l_add_to_relationships: BOOLEAN)
			-- uses of to_add_dbe, creates it if void
		do
			if to_add_dbe = Void then
				to_add_dbe := a_rel.default_secondary
				-- set pk
				if attached to_add_dbe as l_to_add_dbe then
					if attached a_db_service.data_from_column_name (a_rel.secondary_join_as_field_name (a_rel.primary_prototype.Primary_key_db_column_name)) as l_pk_data then
						l_to_add_dbe.set_primary_key ({SIT_TYPE_CONVERTER}.force_as_integer_64 (l_pk_data))
					else
						logger.write_error ("set_relationship_from_db_service_impl-> query doesn't contain the primary key which column_name is: " + a_rel.secondary_join_as_field_name (a_rel.primary_prototype.Primary_key_db_column_name))
						check
							not_found_pk_into_db_result: False
						end
					end
					to_add_dbe := l_to_add_dbe
				end
			end
			if attached a_db_service.data_from_column_name (a_column_name) as l_data and then
				attached to_add_dbe as l_to_add_dbe
			then
				-- set current field
				logger.write_debug ("set_relationship_from_db_service_impl-> setting secondary to_add_dbe " + l_to_add_dbe.generating_type.out + " fieldname:" + an_exported_field + " to:" + l_data.out)
				l_to_add_dbe.call_setter_from_db_field_name (an_exported_field, l_data)
				to_add_dbe := l_to_add_dbe
			else
				logger.write_error ("set_relationship_from_db_service_impl-> query doesn't contain the column_name which is: " + a_column_name)
				check
					not_found_id_for_secondary: False
				end
			end
			if l_add_to_relationships and then attached to_add_dbe as l_to_add_dbe then
				a_rel.secondaries.extend (l_to_add_dbe)
				to_add_dbe := Void
			end
		ensure
			relationship_added_correctly: l_add_to_relationships implies to_add_dbe = Void and a_rel.secondaries.count = old a_rel.secondaries.count + 1
		end

	set_mto_relationship_from_db_service_impl (
		a_db_service: DB_SERVICE[like Current];
		a_rel: MANY_TO_ONE_JOIN[RELATED_PK_DB_ENTITY, PK_DB_ENTITY];
		a_column_name: STRING;
		an_exported_field: like {API_EXPORTABLE_ENTITY}.api_exported_fields.item;
		l_add_to_relationships: BOOLEAN)
			-- uses of to_add_dbe, creates it if void
			-- TODO: merge with set_otm_relationship
		do
			if to_add_dbe = Void then
				to_add_dbe := a_rel.default_secondary
				-- set pk
				if attached to_add_dbe as l_to_add_dbe then
					if attached a_db_service.data_from_column_name (a_rel.secondary_join_as_field_name (a_rel.primary_prototype.Primary_key_db_column_name)) as l_pk_data then
						l_to_add_dbe.set_primary_key ({SIT_TYPE_CONVERTER}.force_as_integer_64 (l_pk_data))
					else
						logger.write_error ("set_relationship_from_db_service_impl-> query doesn't contain the primary key which column_name is: " + a_rel.secondary_join_as_field_name (a_rel.primary_prototype.Primary_key_db_column_name))
						check
							not_found_pk_into_db_result: False
						end
					end
					to_add_dbe := l_to_add_dbe
				end
			end
			if attached a_db_service.data_from_column_name (a_column_name) as l_data and then
				attached to_add_dbe as l_to_add_dbe
			then
				-- set current field
				logger.write_debug ("set_relationship_from_db_service_impl-> setting secondary to_add_dbe " + l_to_add_dbe.generating_type.out + " fieldname:" + an_exported_field + " to:" + l_data.out)
				l_to_add_dbe.call_setter_from_db_field_name (an_exported_field, l_data)
				to_add_dbe := l_to_add_dbe
			else
				logger.write_error ("set_relationship_from_db_service_impl-> query doesn't contain the column_name which is: " + a_column_name)
				check
					not_found_id_for_secondary: False
				end
			end
			if l_add_to_relationships and then attached to_add_dbe as l_to_add_dbe then
				a_rel.set_secondary (l_to_add_dbe)
				to_add_dbe := Void
			end
		ensure
			relationship_added_correctly: l_add_to_relationships implies to_add_dbe = Void and attached a_rel.secondary
		end


invariant
	related_entity_has_at_least_one_relationship: not relationships.is_empty
	to_json_relationships_level_index <= to_json_relationships_level_max + 1
end
