note
	description: "Summary description for {RELATED_PK_DB_ENTITY}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	RELATED_PK_DB_ENTITY

inherit
	PK_DB_ENTITY
		undefine
			to_json
		redefine
			default_create,
			make_from_db_service,
			set_from_db_service,
			set_from_json,
			valid_for_insert,
			valid_for_update,
			valid_for_delete
		end

	RELATED_DB_ENTITY
		undefine
			fields_mapping,
			set_random_for_update,
			set_random_for_insert
		redefine
			default_create,
			set_from_db_service,
			make_from_db_service,
			valid_for_insert,
			valid_for_update,
			valid_for_delete,
			set_from_json
		end

feature {NONE} -- Initialization

	default_create
		do
			Precursor {RELATED_DB_ENTITY}
			Precursor {PK_DB_ENTITY}
		end

	make_from_db_service (a_db_service: DB_SERVICE[like Current])
		do
			Precursor {RELATED_DB_ENTITY} (a_db_service)
		--	Precursor {PK_DB_ENTITY} (a_db_service) -- Otherwise set_relationships_from_db_service will be called 2 times
		end



feature -- Status report

	valid_for_update: BOOLEAN
		do
			Result := Precursor {PK_DB_ENTITY} and Precursor {RELATED_DB_ENTITY}
		end

	valid_for_delete: BOOLEAN
		do
			Result := Precursor {PK_DB_ENTITY} and Precursor {RELATED_DB_ENTITY}
		end


	valid_for_insert: BOOLEAN
		-- <Precursor>
		-- ManyToOne has a valid pk
		do
			Result := Precursor {PK_DB_ENTITY} and Precursor {RELATED_DB_ENTITY}
			if Result then
				across
					relationships as l_relationship
				until
					not Result
				loop
					if attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship.item as l_m_t_o then
						Result := attached {PK_DB_ENTITY} l_m_t_o.secondary as l_sec and then
							l_sec.primary_key > 0 or
							not attached l_m_t_o.secondary and then
							primary_key = 0
					end
				end
			end
		end

feature -- Status setting

	set_from_db_service (a_db_service: DB_SERVICE[like Current])
		do
			Precursor {RELATED_DB_ENTITY} (a_db_service)
			Precursor {PK_DB_ENTITY} (a_db_service)
		end

	set_from_json (a_json_s: STRING)
			-- Adds the parsing of relationships and sets them
		local
			l_json_parser: JSON_PARSER
			l_json_util: JSON_UTIL
			l_type_or_ident_s: STRING
			l_rel: like relationships.item
		do
			Precursor {RELATED_DB_ENTITY} (a_json_s) -- TODO: Improvment: object is parsed 2 times with precursor, parse it only one time
			Precursor {PK_DB_ENTITY} (a_json_s)
			create l_json_util
			logger.write_information ("set_from_json-> with relationships now->Received object:" + a_json_s + "-")
			create l_json_parser.make_with_string (a_json_s)
			l_json_parser.parse_content
			if l_json_parser.is_valid
					and then attached l_json_parser.parsed_json_value as l_parsed_json_val then
				if attached {JSON_OBJECT} l_parsed_json_val as l_json_object then
					across
						l_json_object as l_json_sub_val
					until
						has_error
					loop
						l_type_or_ident_s := l_json_sub_val.key.unescaped_string_32
						l_rel := relationship_where_secondary_is_type (l_type_or_ident_s)
						if not attached l_rel then
							l_rel := relationship_where_secondary_is_identifiable (l_type_or_ident_s)
						end
						if attached l_rel then
							if l_json_sub_val.item.is_object and then
								attached {JSON_OBJECT} l_json_sub_val.item as l_json_sub_obj
							then
								logger.write_debug ("set_from_json->loading_relationship:" + l_rel.out + " with json object:" + l_json_sub_obj.representation)
								if attached {JSON_VALUE} l_json_sub_obj.item (Primary_key_db_column_name) as l_secondary_pk_value and then
									l_secondary_pk_value.is_number
								then
									load_relationship_from_json_obj (l_json_sub_obj, l_rel)
									if l_rel.has_error then
										set_last_error_from_fallible (l_rel)
									end
								end
							else
								logger.write_warning ("set_from_json->Strange json string, the inside object is not an object but seems into relationships of current Entity:" + l_json_sub_val.item.out)
							end
						else
							logger.write_debug ("set_from_json->relationship:" + l_json_sub_val.key.unescaped_string_32 + " not found into relationships of :" + Current.generating_type.out)
						end
					end
				end
			else
				create {PARSE_ERROR} last_error.make ("Error parsing Json object from string, parser is not valid:%N" + a_json_s + "%NError:" + l_json_parser.errors_as_string)
			end
		end

	load_relationship_from_json_obj (a_json_obj: JSON_OBJECT; a_relationship: like relationships.item)
			-- Loads given a_relationship with given json_object
		require
			json_obj_has_pk: attached {JSON_VALUE} a_json_obj.item (Primary_key_db_column_name) as l_secondary_pk_value
				and then l_secondary_pk_value.is_number
				and then valid_primary_key (l_secondary_pk_value.representation.to_integer_64)
		do
			if attached {JSON_VALUE} a_json_obj.item (Primary_key_db_column_name) as l_secondary_pk_value
					and then l_secondary_pk_value.is_number
					and then valid_primary_key (l_secondary_pk_value.representation.to_integer_64) then
				if attached {TO_ONE[like Current, DB_ENTITY]} a_relationship as l_m_t_o then
					l_m_t_o.set_secondary_from_json_obj (a_json_obj)
					if l_m_t_o.has_error then
						set_last_error_from_fallible (l_m_t_o)
					end
				else
					check
						not_implemented: False
					end
				end
			end
		end


	load_relationship (a_secondary_type: TYPE[detachable DB_ENTITY]; a_db_connection: DB_CONNECTION)
			-- Loads relationships specified into relationships from DB when Current entity exists in DB (primary_key > 0)
		require
			setted_primary_key: primary_key > 0
			not_empty_relationships: not relationships.is_empty
		local
			l_relationship: like relationships.item
			l_s_const: STRING_CONSTRAINT
		do
			across
				relationships as l_relationship_c
			loop
				l_relationship := l_relationship_c.item

				if attached {ONE_TO_MANY[like Current, DB_ENTITY]} l_relationship as l_one_to_many then
					logger.write_debug ("a_secondary_type:" + a_secondary_type.out + " l_one_to_many.secondaries.generating_type:" + l_one_to_many.secondaries.generating_type.out)
					if a_secondary_type.is_equal (l_one_to_many.secondary_type) then -- do we have to load it as parameter specified it? does it conform? https://stackoverflow.com/a/53015196/2118777
						create l_s_const.make (l_one_to_many.primary_foreign_key + "=" + primary_key.out)
						l_one_to_many.set_secondary_db_service (a_db_connection) -- Could have been avoided with reflexion
						if attached l_one_to_many.secondary_db_service as l_secondary_db_service then
							l_secondary_db_service.load_entities (create {SQL_QUERY}.make_with_constraint (l_secondary_db_service.item_prototype, l_s_const))
							if l_secondary_db_service.has_error then
								set_last_error_from_fallible (l_secondary_db_service)
							else
								l_one_to_many.set_secondaries (l_secondary_db_service.items)
							end
--						else -- not happening setted before
						end
					end
				elseif attached {MANY_TO_ONE[like Current, DB_ENTITY]} l_relationship as l_one_to_many then
					do_nothing -- should not be called from here /branch/company/1 but /branch/1
				else
					check -- implement other relationships
						not_yet_implemented: false
					end
				end
			end
			-- if company to branches
			-- SELECT * FROM branches WHERE company_id = primary_key
		end

end
