note
	description: "Capable of making mistakes or being wrong"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	FALLIBLE

inherit
	LOGGABLE
		redefine
			default_create,
			out
		end

		
feature {NONE} -- Initialization

	default_create
		do
			Precursor
			create warnings.make
		end

feature -- Access

	warnings: LINKED_LIST[STRING]

	last_error: detachable SIT_ERROR
		-- Last error

	last_success_message: detachable STRING
		-- If any


feature -- Status report

	has_error: BOOLEAN
			-- Based on last_error
		do
			Result := last_error /= Void
		end

	has_warnings: BOOLEAN
		do
			Result := warnings.is_empty
		end

feature -- Output

	out: STRING
		do
			Result := "Fallible, error_status:" + has_error.out
			if attached last_error as l_err then
				Result.append (l_err.out)
			end
			if attached last_success_message as l_msg then
				Result.append (l_msg)
			end
		end

feature -- Status setting

	set_last_error_from_fallible (o: FALLIBLE)
		require
			attached o.last_error
		do
			last_error := o.last_error
			last_success_message := Void
		ensure
			attached o.last_error implies last_success_message = Void
		end

	set_last_error (an_error: attached like last_error)
		do
			last_error:= an_error
			last_success_message := Void
			logger.write_error ("set_last_error-> Error setted:" + an_error.message)
		ensure
			attached an_error implies last_success_message = Void
			has_error
		end

	set_last_success_message_from_fallible (o: attached like Current)
		require
			attached o.last_success_message
		do
			last_success_message := o.last_success_message
			last_error := Void
		ensure
			attached o.last_success_message implies last_error = Void
		end

	set_last_success_message (a_msg: attached like last_success_message)
		do
			last_success_message := a_msg
			last_error := Void
		ensure
			attached a_msg implies last_error = Void
		end

	wipe_last_error
			-- Once you consider having treated it
		do
			last_error := Void
		ensure
			last_error = Void
		end

	wipe_last_success_message
		do
			last_success_message := Void
		ensure
			last_success_message = Void
		end

invariant
	never_both_attached: {SIT_UTIL}.never_both (attached last_error, attached last_success_message)

end
