note
	description: "Summary description for {SIT_HANDLER}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	SIT_HANDLER[G -> DB_ENTITY create default_create, make_from_db_service, make_from_json end]

inherit
	WSF_FILTER

	WSF_URI_TEMPLATE_HANDLER

	WSF_RESOURCE_HANDLER_HELPER
		redefine
			do_get,
			do_post,
			do_put,
			do_delete
		end

	REFACTORING_HELPER

	WSF_SELF_DOCUMENTED_HANDLER

	LOGGABLE

create
	make,
	make_from_db_service

feature {NONE} -- Initialize

	make (a_db_connection: DB_CONNECTION; a_router: WSF_ROUTER)
			--adds it to the router after creation
		require
			a_db_connection.is_connected
		do
			create item_prototype
			db_connection := a_db_connection
			router := a_router
			create db_service.make (a_db_connection)
			set_handler -- has to be after creation of other objects
		ensure
			db_connection = a_db_connection
			router = a_router
		end

	make_from_db_service (a_db_connection: DB_CONNECTION; a_router: WSF_ROUTER; a_db_service: DB_SERVICE[G])
			--adds it to the router after creation
		do
			make (a_db_connection, a_router)
			db_service := a_db_service
		ensure
			db_connection = a_db_connection
			router = a_router
			db_service = a_db_service
		end

	set_handler
		-- Setting the methods this service will answer to
		-- based on https://stackoverflow.com/questions/256349/what-are-the-best-common-restful-url-verbs-and-actions/256359#256359 and https://stackoverflow.com/questions/630453/put-vs-post-in-rest
		require
			setted_router: attached router
		local
			l_http_methods: WSF_REQUEST_METHODS
		do
			-- Adds the handler for '/'
			create l_http_methods
			l_http_methods.enable_options
			l_http_methods.enable_get
			l_http_methods.enable_post -- to create, modify and update (NOT CREATE), only on a url/object/ request
			logger.write_debug (class_for_logger + "->Listening to base URL:" + api_base_url)
			router.handle (api_base_url, create {WSF_URI_TEMPLATE_AGENT_HANDLER}.make (execute_options_filter), l_http_methods)
			-- Adds handler for '/{id}'
			create l_http_methods
			l_http_methods.enable_options
			l_http_methods.enable_get
			l_http_methods.enable_delete
			l_http_methods.enable_put -- create a resource(I'll choose only the post option for that on the client side), or overwrite it (idempotent), Post method will be an error here
			logger.write_debug (class_for_logger + "->Listening to URL with {id}:" + api_entity_by_id_url)
			router.handle (api_entity_by_id_url, create {WSF_URI_TEMPLATE_AGENT_HANDLER}.make (execute_options_filter), l_http_methods) -- as {parameter_name} is said, the attached {WSF_STRING} req.path_parameter ("parameter_name") will contain the value
			set_handler_from_relationships
		end

feature -- Execute

	execute (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
			-- Execute request handler
		local
			l_msg: WSF_CORS_OPTIONS_RESPONSE
		do
			check
				db_service_items_empty: db_service.item = Void and db_service.items.count = 0
			end
			if a_request.is_request_method ({HTTP_REQUEST_METHODS}.method_options) then
				logger.write_debug ("Option request")
				create l_msg.make (a_request, router)
				a_response.send (l_msg)
			else
				logger.write_debug ("Other methods, executing them")
				execute_methods (a_request, a_response)
				execute_next (a_request, a_response)
			end
		end


feature -- Documentation

	mapping_documentation (m: WSF_ROUTER_MAPPING; a_request_methods: detachable WSF_REQUEST_METHODS): WSF_ROUTER_MAPPING_DOCUMENTATION
			-- TODO: Review me
		do
			create Result.make (m)
			if a_request_methods /= Void then
				if a_request_methods.has_method_post then
					Result.add_description ("URI:/tasks METHOD: POST GET")
				elseif a_request_methods.has_method_get
						or a_request_methods.has_method_put
						or a_request_methods.has_method_delete then
					Result.add_description ("URI:/" + api_base_url + "/{" + {PK_DB_ENTITY}.Primary_key_db_column_name + "} METHOD: GET, PUT, DELETE")
				end
			end
		end

feature -- Constants

	Json_base_items_response: STRING = "[
						[$items]
				]"


	Json_base_item_response : STRING_32 = "[
						{"content" : $item }
					]"


feature -- Access

	db_connection: DB_CONNECTION

	db_service: DB_SERVICE[like item_prototype]

	router: WSF_ROUTER

	api_base_url: STRING
		  -- /class_name_lowercase
		do
			Result := "/" + {SIT_UTIL}.class_name_lowercase (({like item_prototype}).out)
		end

	api_entity_by_id_url: STRING
		do
			Result := api_base_url + "/{" + {PK_DB_ENTITY}.Primary_key_db_column_name + "}"
		end

feature -- Element change

	execute_options_filter: like {WSF_EXECUTE_AGENT_HANDLER}.action
			-- local procedure which executes the do_get, do_post, etc. defined methods from http_methods
		local
			l_options_filter: WSF_CORS_OPTIONS_FILTER
		do
			create l_options_filter.make (router)
			l_options_filter.set_next (Current)

			Result := agent l_options_filter.execute
			logger.write_debug ("execute_options_filter: returning new option filter with execution methods from Current")
		end

feature -- Items

	item_prototype: attached G -- has to be attached because of relationships handling

feature -- HTTP Methods

	do_get (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
			-- <Precursor>
			-- to create, modify and update (NOT CREATE), only on a url/object/ request
		require else
			empty_service_error: not db_service.has_error
			empty_db_service_result: db_service.items_all_emtpy
		local
			l_rest_request: REST_REQUEST
		do
			create l_rest_request.make (a_request, item_prototype)
			if l_rest_request.has_primary_key_parameter then -- Get by id
				logger.write_information ("do_get-> With Id:" + l_rest_request.primary_key.out)
				if l_rest_request.has_valid_primary_key then
					if l_rest_request.has_relationships_in_path then
						do_get_relationships (l_rest_request, a_response) -- /db_entity/db_entity/{id}
					else
						db_service.load_from_primary_key (l_rest_request.primary_key) -- /db_entity/{id}
						if attached db_service.last_error as l_error then
							finalize_response_with_error (a_request, a_response, l_error )
							db_service.wipe_last_error
						else
							compute_response_get_or_get_all (a_request, a_response)
						end
					end
				elseif not l_rest_request.has_valid_primary_key then
					finalize_response_with_error (a_request, a_response, create {INVALID_REQUEST}.make (
						"Invalid request, your parameter doesn't seem to be valid, url:" + a_request.request_uri + " data:-" + l_rest_request.request_data + "-"))
				else
					check
						shouldn_t_reach_here: False
					end
				end
			else -- Get all
				logger.write_information ("do_get-> no Id -> getAll")
				db_service.load_all
				compute_response_get_all (a_request, a_response, db_service.items)
			end
		ensure then
			wiped_service_error: not db_service.has_error
		end

	do_post (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
			-- to create, modify and update, only on a url/object/ request
		require else
			empty_service_error: not db_service.has_error
			empty_db_service_result: db_service.items_all_emtpy
			no_path_parameter: attached a_request.path_parameters as l_pp and then
				l_pp.new_cursor.after
		do
			compute_response_post (a_request, a_response)
		ensure then
			wiped_service_error: not db_service.has_error
		end

	do_put (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
			-- Updating a resource with PUT
			-- /{id} and request as json
		require else
			empty_service_error: not db_service.has_error
			empty_db_service_result: db_service.items_all_emtpy
		local
			l_rest_request: REST_REQUEST
		do
			create l_rest_request.make (a_request, item_prototype)
			if l_rest_request.has_primary_key_parameter then
				logger.write_debug ("do_put-> With primary_key")
				if l_rest_request.has_valid_primary_key then -- update
					db_service.update_entity (l_rest_request.primary_key, l_rest_request.request_data)
					if attached db_service.last_error as l_error then
						finalize_response_with_error (a_request, a_response, l_error )
						db_service.wipe_last_error
					else
						if attached db_service.last_success_message as l_msg then
							finalize_response_put (a_request, a_response)
						-- else covered by FALLIBLE class invariant
						end
						db_service.wipe_last_success_message
					end
				elseif not l_rest_request.has_valid_primary_key then
					finalize_response_with_error (a_request, a_response, create {INVALID_REQUEST}.make ("Invalid request, your parameter doesn't seem to be valid"))
				else
					check
						shouldn_t_reach_here: False
					end
				end
			else
				finalize_response_with_error (a_request, a_response, create {INVALID_REQUEST}.make("Invalid request, your parameter is not recognized") )
			end
		ensure then
			wiped_service_error: not db_service.has_error
		end

	do_delete (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
		require else
			empty_service_error: not db_service.has_error
			empty_db_service_result: db_service.items_all_emtpy
		local
			l_rest_request: REST_REQUEST
		do
			create l_rest_request.make (a_request, item_prototype)
			if l_rest_request.has_primary_key_parameter then
				logger.write_debug ("do_delete-> With primary_key")
				if l_rest_request.has_valid_primary_key then
					db_service.delete_entity (l_rest_request.primary_key)
					if attached db_service.last_error as l_error then
						finalize_response_with_error (a_request, a_response, l_error )
						db_service.wipe_last_error
					else
						if attached db_service.last_success_message as l_msg then
							finalize_response_delete (a_request, a_response)
--							finalize_response (l_msg, {HTTP_STATUS_CODE}.ok, a_request, a_response, False)
						-- else covered by FALLIBLE class invariant
						end
						db_service.wipe_last_success_message
					end
				elseif not l_rest_request.has_valid_primary_key then
					finalize_response_with_error (a_request, a_response, create {INVALID_REQUEST}.make ("Invalid request, your parameter doesn't seem to be valid"))
				else
					check
						shouldn_t_reach_here: False
					end
				end
			else
				finalize_response_with_error (a_request, a_response, create {INVALID_REQUEST}.make("Invalid request, your parameter is not recognized") )
			end
		ensure then
			wiped_service_error: not db_service.has_error
		end

feature {NONE} -- Response Computing

	compute_response_get_or_get_all (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
		require
			not db_service.has_error
		do
			if attached db_service.item as l_item then
				compute_response_get (a_request, a_response, l_item)
			else
				compute_response_get_all (a_request, a_response, db_service.items)
			end
		end

	compute_response_get (a_request: WSF_REQUEST; a_response: WSF_RESPONSE; item: detachable like db_service.item)
			-- Resource not found if item is Void
		local
			l_to_put_string: STRING
		do
			create l_to_put_string.make_from_string (json_base_item_response)
			if attached item and then -- Item found
					attached item.to_json as l_json and then
					attached l_json.representation as l_json_repr then
				a_response.set_status_code ({HTTP_STATUS_CODE}.ok)
				create l_to_put_string.make_from_string (json_base_item_response)
				l_to_put_string.replace_substring_all ("$item", l_json_repr)
				finalize_response (l_to_put_string, {HTTP_STATUS_CODE}.ok, a_request, a_response, True)
			else
				if not attached item then -- Found entities can be empty
					finalize_response (l_to_put_string, {HTTP_STATUS_CODE}.ok, a_request, a_response, True)
				else -- entity not found but attached... strange situation
					if attached item and then attached item.to_json as l_json  then
						logger.write_debug ("attached to_json @ least")
					end
					if attached item and then -- Item found
						attached item.to_json as l_json and then
						attached l_json.representation as l_json_repr then
						logger.write_debug ("attached representation @ least")
					end
					check
						should_not_reach: False -- test is done by caller with last_error check and invariant of DB_ENTITY class
					end
				end
			end
		end

	compute_response_get_all (a_request: WSF_REQUEST; a_response: WSF_RESPONSE; some_items: like {DB_SERVICE[DB_ENTITY]}.items)
			-- With given items formats the response message
		require
			valid_db_service: db_service.ready_for_select
		do
			finalize_response (items_as_json_string (some_items), {HTTP_STATUS_CODE}.ok, a_request, a_response, True)
		end

	compute_response_post (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
			-- to create, modify and update, only on a url/object/ request
			-- Request data could be either:
			-- 		{ "name": "name_20181009_023832", "main_email": "email_20181009_023832", "main_phone": "phone_20181009_023832" } for create or update
			-- 		{ "action": { "feature": "get_default" }, "measuring_point": { "id": 1 } } which will be passed to db_service
			-- sets item or items with post feature
		require
			valid_db_service: not db_service.has_error -- last error was not cleaned
		local
			l_response_s: STRING
			l_status_code: like {HTTP_STATUS_CODE}.created
			l_rest_request: REST_REQUEST
		do
			create l_rest_request.make (a_request, item_prototype)
			if l_rest_request.has_request_feature then -- request feature
				db_service.call_associated_feature (l_rest_request)
			elseif attached {RELATED_DB_ENTITY} item_prototype as l_rel_prototype and then -- get_all
					l_rest_request.has_request_data and then
					l_rest_request.is_get_all
			then
				db_service.set_item_prototype (item_prototype)
				db_service.load_with_relationships (l_rest_request)
			else
				logger.write_debug ("compute_response_post->" + l_rest_request.out) -- create or update
				db_service.create_or_update_entity (l_rest_request)
			end

			if attached db_service.last_error as l_err then
				finalize_response_with_error (a_request, a_response, l_err)
				db_service.wipe_last_error
			else
				if attached db_service.callable_feature_result as l_res then
					l_status_code := {HTTP_STATUS_CODE}.created
					l_response_s := l_res
					finalize_response (l_response_s, {HTTP_STATUS_CODE}.ok, a_request, a_response, True)
				elseif attached db_service.item as l_item then
					l_status_code := {HTTP_STATUS_CODE}.created
					l_response_s := l_item.to_json_string
					finalize_response (l_response_s, {HTTP_STATUS_CODE}.ok, a_request, a_response, True)
				else
					finalize_response (items_as_json_string (db_service.items), {HTTP_STATUS_CODE}.ok, a_request, a_response, True)
				end
			end
		end

feature {NONE} -- URI helper methods

	finalize_response_with_error (a_request: WSF_REQUEST; a_response: WSF_RESPONSE; an_error: SIT_ERROR)
		local
			l_header: HTTP_HEADER
		do
			create l_header.make
			l_header.put_content_length (an_error.out.count)
			if attached a_request.request_time as time then
				l_header.put_utc_date (time)
			end
			l_header.put_content_type_text_plain
			a_response.set_status_code (an_error.http_status_code)
			a_response.put_header_text (l_header.string)
			a_response.put_string (an_error.out)
		end

	finalize_response (a_response_s: STRING_32; a_status_code: like {HTTP_STATUS_CODE}.ok; a_request: WSF_REQUEST; a_response: WSF_RESPONSE; a_response_is_json: BOOLEAN)
			--Either json or plain
		local
			l_http_header: HTTP_HEADER
		do
			create l_http_header.make
			l_http_header.add_header ("Cache-Control: max-age=0,must-revalidate") -- in seconds
			l_http_header.put_content_length (a_response_s.count)
			if attached a_request.request_time as time then
				l_http_header.put_utc_date (time)
			end
			if a_response_is_json then
				l_http_header.put_content_type_application_json
			else
				l_http_header.put_content_type_text_plain
			end
			a_response.set_status_code (a_status_code)
			a_response.put_header_text (l_http_header.string)
			a_response.put_string (a_response_s)
		end

	finalize_response_put (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
		do
			finalize_response_delete (a_request, a_response)
		end

	finalize_response_post (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
		do
			finalize_response_put (a_request, a_response)
		end

	finalize_response_delete (a_request: WSF_REQUEST; a_response: WSF_RESPONSE)
		do
			finalize_response ("", {HTTP_STATUS_CODE}.ok, a_request, a_response, False)
		end


feature {NONE} -- Implementation

	items_as_json_string (some_items: like {DB_SERVICE[DB_ENTITY]}.items): STRING_32
		local
			l_items_as_string: STRING_32
		do
			create l_items_as_string.make_empty
			across
				some_items as entity
			loop
				if attached entity.item as l_entity_item and then
					attached l_entity_item.to_json as l_entity_json and then
					attached l_entity_json.representation as l_entity_json_string then
					l_items_as_string.append (l_entity_json_string)
					if entity.cursor_index < entity.last_index then
						l_items_as_string.append (",%N")
					end
				else
					logger.write_error("entity as json failed because of Void")
				end
			end

			create Result.make_from_string (Json_base_items_response)
			Result.replace_substring_all ("$items", l_items_as_string)
			logger.write_debug ("items_as_json_string-> itemsCount:" + db_service.items.count.out + " result:" + Result.to_string_8)
		end

	set_handler_from_relationships
			-- Adds handler for '/current/relationship.secondaries/{id}' i.e. '/company/branch/{id}'
		require
			setted_router: attached router
		local
			l_http_methods: WSF_REQUEST_METHODS
			l_url: STRING
		do
			if attached {RELATED_DB_ENTITY} item_prototype as l_r_entity then -- current item has relationships
				across
					l_r_entity.relationships as l_r_c
				loop
					if attached {ONE_TO_MANY[RELATED_DB_ENTITY, DB_ENTITY]} l_r_c.item as l_o_t_m then
						create l_http_methods
						l_http_methods.enable_options
						l_http_methods.enable_get
						l_url := api_base_url + "/" + {SIT_UTIL}.class_name_lowercase (l_o_t_m.secondary_type.out) + "/{id}"
						logger.write_debug (class_for_logger + "->Listening to URL:" + l_url)
						-- as {parameter_name} is said, the attached {WSF_STRING} req.path_parameter ("parameter_name") will contain the value
						router.handle (l_url, create {WSF_URI_TEMPLATE_AGENT_HANDLER}.make (execute_options_filter), l_http_methods)
					elseif attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_r_c.item as l_m_t_o then
						create l_http_methods
						l_http_methods.enable_options
						l_http_methods.enable_get
						l_url := api_base_url + "/" + {SIT_UTIL}.class_name_lowercase (l_m_t_o.secondary_type.out) + "/{id}"
						logger.write_debug (class_for_logger + "->Listening to URL:" + l_url)
						-- as {parameter_name} is said, the attached {WSF_STRING} req.path_parameter ("parameter_name") will contain the value
						router.handle (l_url, create {WSF_URI_TEMPLATE_AGENT_HANDLER}.make (execute_options_filter), l_http_methods)
					end
				end
			end
		end

	do_get_relationships (a_request: REST_REQUEST; a_response: WSF_RESPONSE)
			-- Handles for '/relationship.secondaries/{id}' and modifies relationship_handled if handled
			-- Working item so
		require
			a_request.has_valid_primary_key
			a_request.has_primary_key_parameter
		local
			l_secondary_entity_name: STRING
			l_msg: STRING
		do
			if attached {RELATED_DB_ENTITY} item_prototype as l_r_entity then
				across
					l_r_entity.relationships as l_relationship
				loop
					if attached {ONE_TO_MANY[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship.item as l_o_t_m then -- relationship is a one_to_many
						l_secondary_entity_name := {SIT_UTIL}.class_name_lowercase (l_o_t_m.secondary_type.out)
						if a_request.has_as_secondary (l_secondary_entity_name) then -- query is '/primary/secondary_entity/{id}'
							logger.write_debug ("do_get /parent/ -> With Id")
							db_service.load_from_primary_key (a_request.primary_key)
							if attached db_service.last_error as l_error then -- error occured during load of primary entity
								finalize_response_with_error (a_request.request, a_response, l_error )
								db_service.wipe_last_error
							elseif attached {RELATED_PK_DB_ENTITY} db_service.item as l_item then -- entity has relationships
								l_item.load_relationship (l_o_t_m.secondary_type, db_connection)
								if not l_item.has_error
										and then attached l_item.relationship_from_type_conformance ({ONE_TO_MANY[RELATED_DB_ENTITY, DB_ENTITY]}) as l_item_relationship -- same as l_relationship but loaded
										and then attached {ONE_TO_MANY[RELATED_DB_ENTITY, DB_ENTITY]} l_item_relationship as l_one_to_many_relationship then
									compute_response_get_all (a_request.request, a_response, chain_to_items (l_one_to_many_relationship.secondaries))
								else
									l_msg := "Entity " + l_secondary_entity_name + " could not be set"
									if attached l_item.last_error as l_err then
										l_msg.append(": " + l_err.out)
									end
									finalize_response_with_error (a_request.request, a_response, create {ENTITY_NOT_FOUND}.make (l_msg) )
								end
							else
								check
									primary_load_error: False
								end
							end
						end
					elseif attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship.item as l_m_t_o then -- relationship is a many_to_one
						l_secondary_entity_name := {SIT_UTIL}.class_name_lowercase (l_m_t_o.secondary_type.out)
						if a_request.has_as_secondary (l_secondary_entity_name) then -- query is '/primary/secondary_entity/{id}'
							logger.write_debug ("do_get /parent/ -> With Id")
							db_service.load_from_primary_key (a_request.primary_key)
							if attached db_service.last_error as l_error then -- error occured during load of primary entity
								finalize_response_with_error (a_request.request, a_response, l_error )
								db_service.wipe_last_error
							elseif attached {RELATED_PK_DB_ENTITY} db_service.item as l_item then -- entity has relationships
								l_item.load_relationship (l_m_t_o.secondary_type, db_connection)
								if not l_item.has_error
										and then attached l_item.relationship_from_type_conformance ({MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]}) as l_item_relationship -- same as l_relationship but loaded
										and then attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_item_relationship as l_m_t_o_relationship then
									compute_response_get_all (a_request.request, a_response, chain_to_items (l_m_t_o_relationship.primaries))
								else
									l_msg := "Entity " + l_secondary_entity_name + " could not be set"
									if attached l_item.last_error as l_err then
										l_msg.append(": " + l_err.out)
									end
									finalize_response_with_error (a_request.request, a_response, create {ENTITY_NOT_FOUND}.make (l_msg) )
								end
							else
								check
									primary_load_error: False
								end
							end
						end
					else
						check
							not_happening_not_handled_relationship: False
						end
					end
				end
			end
		end

	chain_to_items (a_chain: CHAIN[DB_ENTITY]): like {DB_SERVICE[DB_ENTITY]}.items
			-- because used @ execution, we can't use G instead of DB_ENTITY
		do
			create Result.make
			across
				a_chain as l_item
			loop
				Result.extend (l_item.item)
			end
		ensure
			valid_count: a_chain.count = Result.count
		end

end
