note
	description: "Implementation of logger with file and stderr"
	colors_links: "Inspired from https://stackoverflow.com/questions/2616906/how-do-i-output-coloured-text-to-a-linux-terminal"
	stackoverflow: "https://stackoverflow.com/questions/52859429/eiffel-is-there-a-way-to-print-colorized-characters-into-a-terminal-console/52860686#52860686"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	SIT_LOGGER

inherit
	LOG_LOGGING_FACILITY
		redefine
			write
		end

create
	make_from_class

feature {NONE} -- Initialization

	make_from_class (a_class: like current_class)
		do
			current_class := a_class
			make

			enable_default_file_log -- by default the log file is "system.log"
			enable_default_stderr_log -- STDERR
			-- Add file appender
          	create app_file_log_writer
            app_file_log_writer.set_path (log_file_path)
			-- set_loger_level
			set_level ({APP_CONFIGURATION}.app_instance.log_level)
            register_log_writer (app_file_log_writer)
			-- write_debug ("make_from_class->" + {APP_CONFIGURATION}.application_instance.log_level + ":" + log_file_path.absolute_path.out + "")
            if app_file_log_writer.has_errors then
                write_emergency ("Cannot open log file '" + {APP_CONFIGURATION}.app_instance.log_file_path + "'%N")
            end
		end

feature -- Initialization

	initialize_terminal: INTEGER
	    external "C inline"
	    alias "[
	        #ifdef EIF_WINDOWS
	            {
	                HANDLE hOut = GetStdHandle(STD_OUTPUT_HANDLE);
	                if (hOut == INVALID_HANDLE_VALUE) {
	                	return GetLastError();
	                }
	                DWORD dwMode = 0;
	                if (!GetConsoleMode(hOut, &dwMode)) {
	                	return GetLastError();
	                }
	                dwMode |= ENABLE_VIRTUAL_TERMINAL_PROCESSING;
	                if (!SetConsoleMode(hOut, dwMode)) {
        				return GetLastError();
    				}
    				return 0;
	            }
		#else
		      /* Check for other platforms */
		    return 0;
	        #endif
	    ]"
	end

feature -- Status setting

	set_level (a_level: like {APP_CONFIGURATION}.log_level)
		require
			valid_log_level (a_level)
		do
			if a_level.is_equal(Log_level_debug) then
				default_log_writer_file.enable_debug_log_level
				app_file_log_writer.enable_debug_log_level
				default_log_writer_stderr.enable_debug_log_level
			elseif a_level.is_equal(Log_level_info) then
				default_log_writer_file.enable_information_log_level
				app_file_log_writer.enable_information_log_level
				default_log_writer_stderr.enable_information_log_level
			else
				default_log_writer_file.enable_error_log_level
				app_file_log_writer.enable_error_log_level
				default_log_writer_stderr.enable_error_log_level
			end
		end

	valid_log_level (a_log_level: like {APP_CONFIGURATION}.log_level): BOOLEAN
		do
			Result := a_log_level.is_equal (Log_level_debug) or
				a_log_level.is_equal (Log_level_info) or
				a_log_level.is_equal (Log_level_error)
		ensure
			instance_free: class
		end

feature -- Access

	Log_level_debug: STRING = "debug"
	Log_level_info: STRING = "info"
	Log_level_error: STRING = "error"

	current_class: STRING

    app_file_log_writer: LOG_WRITER_FILE

    is_colorized, is_terminal_output: BOOLEAN = True

	log_file_path: PATH
			-- Attached if can be created
		local
			l_file: UNIX_FILE_INFO
			l_path, l_parent_dir: PATH
			l_fu: FILE_UTILITIES
		do
			-- Parent directory check
			create l_path.make_from_string ({APP_CONFIGURATION}.app_instance.log_file_path)
			l_parent_dir := l_path.parent
			if not l_fu.directory_path_exists (l_parent_dir) then
				l_fu.create_directory_path (l_parent_dir)
			end
			create l_file.make
			l_file.update (l_parent_dir.out)
			if not l_file.exists or
				not l_file.is_access_writable
			then
--				io.putstring ("WARN: " + {APP_CONFIGURATION}.app_instance.log_file_path + " parent directory is not writtable and cannot be created%N")
				Result := l_path
			else
				Result := l_path
			end
		end


feature {NONE} -- Implementation

	write (a_priority: INTEGER; msg: STRING)
		local
			l_msg: like msg
		do
--			msg.prepend (({like Current}).generating_type.out + ":") --TODO: add client class name here
			l_msg := msg
			l_msg.prepend ("(" + current_class + "):")
			if is_colorized and is_terminal_output then
				l_msg := colorized (a_priority, l_msg).out
			end

--			print (l_msg)
--			print ("%/27/[1;31mTEST%/27/[0m<--RESET%N") -- Bold light Red
--			print ("%/27/[31;15mTEST31.1%/27/[0m<--RESET%N") -- Red on white

			Precursor(a_priority, l_msg)
		end

	colorized (a_priority: INTEGER; a_msg: STRING): TERMINAL_STRING
		local
			l_red_array: ARRAY[like a_priority]
		do
			l_red_array := <<Log_alert, Log_error, Log_critical, log_warning>>
			create Result.make_from_string (a_msg)
			if l_red_array.has (a_priority) then
				Result.set_foreground_color ({TERMINAL_STRING}.Foreground_red)
			elseif a_priority = Log_debug then
				Result.set_foreground_color ({TERMINAL_STRING}.Foreground_black)
			elseif a_priority = Log_information then
				Result.set_foreground_color ({TERMINAL_STRING}.Foreground_green)
			else
				Result.set_foreground_color ({TERMINAL_STRING}.Foreground_black)
			end
		end

end
