note
	description: "Summary description for {APP_CONFIGURATION}. Frozen because of once routines"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

frozen class
	APP_CONFIGURATION

inherit
	ANY
		redefine
			default_create
		end

create
	default_create

feature -- Initialize

	default_create
		do
			configuration_file_path := Default_configuration_file_path
			set_configuration_general_obj
			set_application_environment
			set_logger_level
			set_application_port
			set_log_file_path
			Precursor
		ensure then
			attached configuration_general_json_obj
		end

feature -- Constants

	Date_time_default_format_string: STRING = "YYYY-[0]MM-[0]DD [0]hh:[0]mi:[0]ss" -- For Database out
	Date_default_format_string: STRING = "YYYY-[0]MM-[0]DD"

	Default_port: INTEGER = 9999
	Default_configuration_file_path: STRING = "/etc/opt/solarity/ewf_entities.json"
	Default_log_file_path: STRING = "application.log"

	Current_system_json_key: STRING = "current_system"

	Current_environment_json_key: STRING = "environment"

	-- options
	Application_environment_production: like application_environment = "prod"
	Application_environment_development: like application_environment = "dev"
	Application_environment_test: like application_environment = "test"
	Default_application_environment: like application_environment = "dev"

	Default_log_level: like log_level = "info" -- {SIT_LOGGER}.Log_level_info -- https://stackoverflow.com/questions/54420105/eiffel-setting-a-constant-with-reference-to-other-class-constant

	-- JSON Value
	Datasource_name_json_key: STRING = "datasource_name"
	Database_username_json_key: STRING = "datasource_username"
	Database_password_json_key: STRING = "datasource_password"
	-- JSON Value
	App_port_json_key: STRING = "ewf_listening_port"
	Log_file_path_json_key: STRING = "log_file_path"
	-- JSON Value
	Conf_val_selected_entity_json_key: STRING = "default_selected_"
	-- JSON Value
	Logger_level_json_key: STRING = "logger_level"

	Configuration_values: ARRAY[STRING]
			--Available configuration values
		once
			Result := <<
				Conf_val_selected_entity_json_key + "",
				Log_file_path_json_key,
				Datasource_name_json_key,
				Database_username_json_key,
				Database_password_json_key,
				App_port_json_key
				>>
		end

feature -- Access

	log_level: STRING

	configuration_general_json_obj: detachable JSON_OBJECT

	application_environment_is_setted: BOOLEAN

	trace_enabled: BOOLEAN
		once
			Result := True
		end

	app_instance: APP_CONFIGURATION
		once
			create Result
		ensure
			instance_free: class
		end

	application_environment: STRING
		attribute
			Result := "dev" -- Default_application_environment
		end

	application_port: INTEGER

	log_file_path: STRING

	configuration_file_path: STRING

	vat: REAL
		require
			attached app_instance.configuration_value ("vat-value")
		do
			if attached app_instance.configuration_value ("vat-value") as l_v then
				Result := l_v.to_real
			end
		end

	solarity_json_o: JSON_OBJECT
		require
			attached configuration_object ("solarity")
		do
			if attached configuration_object ("solarity") as l_v then
				Result := l_v
			else
				create Result.make_empty
				check
					not_happening: False
				end
			end
		end

	db_connection: DB_CONNECTION
		attribute
			create Result
			if not Result.is_connected then
				Result.connect
			end
		end

feature -- Status report

	configuration_object (a_key: STRING): detachable JSON_OBJECT
		require
			application_environment_is_setted
			attached configuration_general_json_obj as l_conf_general_obj
			attached {JSON_OBJECT} l_conf_general_obj.item (application_environment)
		do
			if attached configuration_general_json_obj as l_conf_general_obj and then
				attached {JSON_OBJECT} l_conf_general_obj.item (application_environment) as l_configuration
			then
				if l_configuration.has_key (a_key) then
					if attached {JSON_OBJECT} l_configuration.item (a_key) as l_val then
						Result := l_val
					else
						io.put_string ("WARN: Unavailable value '" + a_key + "' into " + configuration_file_path + " NOT A JSON_OBJECT %N")-- TODO rewrite me
					end
				else
					io.put_string ("WARN: Unavailable value '" + a_key + "' into " + configuration_file_path + " %N")
				end
			end
		end

	configuration_value (a_value: STRING): detachable STRING
			-- uses application_environment
		require
			application_environment_is_setted
			attached configuration_general_json_obj as l_conf_general_obj
			attached {JSON_OBJECT} l_conf_general_obj.item (application_environment)
		do
			if attached configuration_general_json_obj as l_conf_general_obj and then
				attached {JSON_OBJECT} l_conf_general_obj.item (application_environment) as l_configuration
			then
				if  l_configuration.has_key (a_value) then
--					if attached l_configuration.item (a_value) as l_val then
--						io.putstring ("DEBUG:----------------->key:" + a_value + " val:" + l_val.representation)
--					end
					if attached {JSON_STRING} l_configuration.item (a_value) as l_val then
						Result := l_val.unescaped_string_8
						io.put_string ("DEBUG:Got value from json key:" + a_value + "=" + Result + "%N")
					elseif attached {JSON_NUMBER} l_configuration.item (a_value) as l_val then
						Result := l_val.item
						io.put_string ("DEBUG:Got value from json key:" + a_value + "=" + Result + "%N")
					else
						io.put_string ("WARN:Value '" + a_value + "' into " + configuration_file_path + " has not been treated as JSON_TYPE above  %N")
						check
							untreated_json_type: False
						end
					end
				else
					io.put_string ("WARN: Unavailable value '" + a_value + "' into " + configuration_file_path + " %N")
				end
			end
		end

	selected_entity_primary_key (l_class_name: STRING): detachable STRING
			-- return the selected entity primary key from json file
		do
			if attached configuration_value (Conf_val_selected_entity_json_key + l_class_name) as l_val then
				Result := l_val
			end
		end

	application_verbose: STRING
		require
			application_environment_is_setted
		once
			if is_prod then
				Result := "yes" -- TODO: LOW change me as I have to answer no on prod
			else
				Result := "yes"
			end
		end

	current_db_schema: STRING
		require
			application_environment_is_setted
		once
			Result := application_environment
		end

	is_prod: BOOLEAN
		require
			application_environment_is_setted
		once
			Result := application_environment.is_equal (Application_environment_production)
		end

	is_test: BOOLEAN
		require
			application_environment_is_setted
		once
			Result := application_environment.is_equal (Application_environment_test)
		end

	is_development: BOOLEAN
		require
			application_environment_is_setted
		once
			Result := application_environment.is_equal (Application_environment_development)
		end

	valid_application_environment (an_environment: STRING): BOOLEAN
		do
			Result := an_environment.is_equal (Application_environment_production)
				or an_environment.is_equal (Application_environment_development)
				or an_environment.is_equal (Application_environment_test)
		end

feature -- Status setting

	set_log_file_path
		require
			attached configuration_general_json_obj
		do
			log_file_path := Default_log_file_path
			if attached configuration_value (Log_file_path_json_key) as l_path then
				log_file_path := l_path
			end
		end

	set_logger_level
		require
			attached configuration_general_json_obj
		do
			log_level := Default_log_level
			if attached configuration_value (Logger_level_json_key) as l_logger_level then
				log_level := l_logger_level
				io.put_string ("INFO: set_logger_level->Found argument log_level:" + log_level + "%N")
			else
				(create {EXCEPTIONS}).raise ("set_logger_level->Unable to getenvironment from json_file:" + Logger_level_json_key)
			end
		end

	set_application_port
		require
			application_environment_is_setted
		once
			application_port := Default_port
			if attached configuration_value (App_port_json_key) as l_port and then
				l_port.is_integer
			then
				application_port := l_port.to_integer_32
			else
				check
					app_port_is_not_an_int: False
				end
			end
			io.put_string ("DEBUG: Application port is:" + application_port.out + "%N")
		end


	set_application_environment
			--Sets it from commandline and if not found from json file
		require
			attached configuration_general_json_obj as l_conf_general_obj
			attached {JSON_OBJECT} l_conf_general_obj.item (Current_system_json_key)
		local
			l_msg: STRING
		do
			if attached configuration_general_json_obj as l_conf_general_obj and then
				attached {JSON_OBJECT} l_conf_general_obj.item (Current_system_json_key) as l_current_system
			then
				if attached {JSON_STRING} l_current_system.item (current_environment_json_key) as l_current_env then
					if valid_application_environment (l_current_env.unescaped_string_8) then
						application_environment := l_current_env.unescaped_string_8
						application_environment_is_setted := True
						io.put_string ("INFO: set_application_environment->Found argument environment:" + application_environment + "%N")
					else
						io.put_string ("ERROR: set_application_environment->configuration argument:--" + Current_environment_json_key + " is not a valid_application_environment:" + l_current_env.unescaped_string_8 + "%N")
						check
							False
						end
					end
				else
					io.put_string ("ERROR: set_application_environment " + Current_system_json_key + " key not found into json file")
					check
						Current_system_json_key_not_found: False
					end
				end
			else
				l_msg := "ERROR: set_application_environment->configuration_general_json_obj is void"
				(create {EXCEPTIONS}).raise (l_msg)
			end
		ensure
			valid_application_environment (application_environment)
			application_environment_is_setted: application_environment_is_setted
		end

feature {NONE} -- Implementation

	set_configuration_general_obj
			-- from Default_configuration_file_path
		local
			l_json_parser: JSON_PARSER
			l_ut: FILE_UTILITIES
			l_path: PATH
		do
			if not attached configuration_general_json_obj then
				create l_path.make_from_string (Configuration_file_path)
				io.put_string ("DEBUG: configuration_value->path of '" + Default_configuration_file_path + "' exists:" + l_ut.file_path_exists (l_path).out + "%N")
				if
					l_ut.file_path_exists (l_path) and then
					attached (create {JSON_FILE_READER}).read_json_from (Default_configuration_file_path) as l_json_file
				then
					create l_json_parser.make_with_string (l_json_file)
					l_json_parser.parse_content
					io.put_string ("DEBUG:configuration_value->l_json_parser.is_valid:" + l_json_parser.is_valid.out + "%N")
					if
						l_json_parser.is_valid and then
						attached l_json_parser.parsed_json_object as l_general_obj
					then
						configuration_general_json_obj := l_general_obj
					else
						io.put_string ("WARN:Could not read configuration file:" + Default_configuration_file_path + " setting default values" + "%N")
						check
							False
						end
					end
				end
			end
		ensure
			configuration_general_json_obj /= Void
		end

invariant
	valid_application_environment (application_environment)
	valid_log_level: {SIT_LOGGER}.valid_log_level (log_level)
note
	misc: "Could not be LOGGABLE because is created before logger, and logger needs it to work"
end
