note
	description: "Summary description for {K_WATT_H}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	K_WATT_H

inherit
	RANDOMABLE
		undefine
			is_equal
		redefine
			default_create,
			out
		end

	LOGGABLE
		undefine
			is_equal,
			out
		redefine
			default_create
		end

	COMPARABLE
		redefine
			default_create,
			out
		end

create
	default_create,
	make_from_real_64,
	make_from_watt,
	make_from_charge_units_average,
	make_from_string,
	make_random

convert
	make_from_real_64 ({REAL_64}),
	make_from_string ({STRING}),
	to_real_64: {REAL_64}

feature {NONE} -- Initialization

	default_create
		do
			set_item(0)
			Precursor {LOGGABLE}
			Precursor {COMPARABLE}
			Precursor {RANDOMABLE}
		end

	make_from_real_64 (v: REAL_64)
		require
			valid_value (v)
		do
			set_item (v)
		end

	make_from_string (v: STRING)
		require
			v_is_integer: v.is_real_64
			valid_v_value: valid_value (v.to_real_64)
		do
			set_item (v.to_real_64)
		end

	make_random
		local
			l_random: RANDOM
		do
			create l_random.make
			default_create
			set_item (l_random.item.to_double)
		end

	make_from_watt (v: WATT)
		do
			make_from_real_64 (v.item)
			item := item / 1000
		end

	make_from_charge_units_average (some_items: LINKED_LIST[CHARGE_UNIT])
		require
			some_items_same_day: items_same_day (some_items)
			not some_items.is_empty
		local
			l_sum, l_h_sum, l_total_sum: like item.item
			l_current_h, l_h_sum_items_count: INTEGER
		do
			if not some_items.is_empty then
				l_current_h := some_items.first.date_time.hour
				across
					some_items as l_item
				loop
					l_total_sum := l_total_sum + l_item.item.value.item -- other way
					if l_current_h = l_item.item.date_time.hour then
						l_h_sum := l_h_sum + l_item.item.value.item
						l_h_sum_items_count := l_h_sum_items_count + 1
					else
						l_sum := l_sum + (l_h_sum / l_h_sum_items_count)
						l_current_h := l_current_h + 1
						l_h_sum_items_count := 1
						l_h_sum := l_item.item.value.item
					end
				end

				make_from_real_64 (l_sum / 1000) -- W -> K_W_H
				logger.write_information ("make_from_charge_units_average-> l_total_sum:" + (l_total_sum / some_items.count * 24 / 1000).out )
				logger.write_information ("make_from_charge_units_average-> date:" + some_items.first.date_time.formatted_out ("YYYY-MM-DD [0]hh:[0]mi:[0]ss") + " result value:" + item.out)
			else
				logger.write_error ("make_from_charge_units_average-> Error, some_item is empty")
			end
		end

feature -- Status setting

	set_item (v: like item)
		require
			valid_value (v)
		do
			item := v
		end


feature -- Status report

	item: REAL_64

	to_real_64: REAL_64
		do
			Result := item
		end

	valid_value (v: REAL_64): BOOLEAN
		do
			Result := True -- include negative values
		end

	out: STRING
		do
			Result := item.out
		end

	items_same_day (some_items:LINKED_LIST[CHARGE_UNIT]): BOOLEAN
			-- are all the given items on the same day?
		local
			l_day: INTEGER
		do
			Result := some_items.is_empty
			if not some_items.is_empty then
				Result := True -- if only one elt
				l_day := some_items.first.date_time.day
				across
					some_items as l_item
				until
					not Result
				loop
					if l_item.item.date_time.day /= l_day then
						Result := False
					end
				end
			end
		ensure
			instance_free: class
		end

feature -- Comparable

	is_less alias "<" (other: like Current): BOOLEAN
			-- Is current object less than `other'?
		do
			Result := item < other.item
		end


invariant
	valid_value (item)
end
