note
	description: "Summary description for {SIT_UTIL}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	SIT_UTIL

inherit
	LOGGABLE

create
	default_create

feature -- STRING Operations

	to_coma_separated (a_chain: CHAIN[ANY]): STRING
		local
			l_to_insert: STRING
		do
			Result := ""
			across
				a_chain as l_v
			loop
				l_to_insert := l_v.item.out
				if l_to_insert.is_equal ("0") then
					l_to_insert := "NULL"
				end
				Result.append (l_to_insert)
				if not l_v.is_last then
					Result.append (",")
				end
			end
		ensure
			min_valid_items_count: Result.occurrences (',') >= a_chain.count - 1
			instance_free: class
		end

	iterable_to_string (an_iterable: ITERABLE[ANY]): STRING
		do
			Result := ""
			across
				an_iterable as l_item
			loop
				Result.append(l_item.item.out + "%N")
			end
			if Result.count = 0 then
				Result := "Empty iterable"
			end
		ensure
			instance_free: class
		end

feature -- could be in class BOOLEAN

	double_implies, reversible_implies, never_both (a, b: BOOLEAN): BOOLEAN
			-- Into boolean class with never_with
	    do
	        Result := not (a and b)
		ensure
			instance_free: class
	    end

	class_name_lowercase (a_string: STRING): STRING
			-- a copy lowercased and pruned from preceding '!'
			-- TODO: HIGH add reference to what is the preceding !
			-- https://stackoverflow.com/questions/56818605/eiffel-best-way-to-compare-types-without-getting-a-catcall/56819607#56819607
		do
			Result := a_string
			if Result.index_of('!', 1) = 1 then
				Result := Result.substring (2, Result.count)
				Result.to_lower
			else
				Result := Result.as_lower
			end
		ensure
			instance_free: class
		end

feature --Regular Expression check

	match_regex ( a_text: STRING; a_pattern: STRING): BOOLEAN
			-- checks whether 'text' matches a regular expression 'pattern'
		require
			a_text /= Void
			a_pattern /= Void
		local
			dfa: LX_DFA_REGULAR_EXPRESSION
		do
			create dfa.make
			dfa.compile(a_pattern, True)
--			check      -- regex must be compiled before we can use it
--				dfa.is_compiled
--			end
			if dfa.is_compiled then
				Result := dfa.matches(a_text)
--				io.putstring ("Given expression matches:" + Result.out + " exp:'" + a_text + "' pattern:'" + a_pattern + "'%N")
--			else
--				io.putstring ("Given expression doesn't compile:'" + a_pattern + "'%N")
			end

		ensure
			instance_free: class
		end

feature -- Timezones

	timezone_offset_for_index (a_list: like {JSON_UTIL}.timezone_offsets_array_to_list; a_date_time: DATE_TIME): detachable like {JSON_UTIL}.timezone_offsets_array_to_list.item.offset
		require
			offset_list_has_date (a_list, a_date_time)
		local
			found: BOOLEAN
		do
			across
				a_list is l_item
			until
				found
			loop
				if l_item.date.is_equal (a_date_time.date) then
					Result := l_item.offset
					found := True
				end
			end
		ensure
			instance_free: class
		end

feature -- Classes and other runtime implementations

	pk_db_entity_from_class_name (a_class_name: STRING; an_id: like {PK_DB_ENTITY}.Primary_key): PK_DB_ENTITY
		do
			if a_class_name.is_case_insensitive_equal ({SIT_UTIL}.class_name_lowercase (({MEASURING_POINT}).out)) then
				create {MEASURING_POINT} Result.make (an_id)
			elseif a_class_name.is_case_insensitive_equal ({SIT_UTIL}.class_name_lowercase (({BRANCH}).out)) then
				create {BRANCH} Result.make (an_id)
			else
				create {MEASURING_POINT} Result.make (10) -- for void_safety
				check
					not_implemented_case: False
				end
			end
		ensure
			instance_free: class
		end

	raise_exception (a_msg: STRING)
		do
			(create {EXCEPTIONS}).raise (a_msg)
		ensure
			instance_free: class
		end


	offset_list_has_date (a_list: like {JSON_UTIL}.timezone_offsets_array_to_list; a_date_time: DATE_TIME): BOOLEAN
		do
			across
				a_list is l_item
			until
				Result
			loop
				Result := l_item.date.is_equal (a_date_time.date)
			end
		ensure
			instance_free: class
		end

	
end
