note
	description: "Summary description for {RELATIONSHIP}. P stands for primary/ies, and S for secondary/ies"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	RELATIONSHIP[P -> RELATED_DB_ENTITY create default_create end, S -> DB_ENTITY create default_create end]

inherit
	FALLIBLE
		redefine
			out
		end

feature -- Access

	secondary_required: BOOLEAN -- is secondary (DB NOT NULL)

	loadable_from_resultset: BOOLEAN -- is current relationship loadable from resultset?

feature -- Status setting

	set_loadable_from_resultset (v: like loadable_from_resultset)
		do
			loadable_from_resultset := v
		ensure
			loadable_from_resultset = v
		end



feature -- Status report

	primary_foreign_key: STRING
			-- key to secondary table
		do
			Result := {SIT_UTIL}.class_name_lowercase (({P}).out)
			Result.append ("_id")
		end

	secondary_foreign_key: STRING
			-- key to primary table
		do
			Result := {SIT_UTIL}.class_name_lowercase (({S}).out)
			Result.append ("_id")
		end

	primary_type: TYPE[detachable P]
		do
			Result := {P}
		end

	secondary_type: TYPE[detachable S]
		do
			Result := {S}
		end

	primary_prototype: P
			--To be able to consult fields
		do
			create Result
		end

	secondary_prototype: S
		do
			create Result
		end

	primary_join_alias: STRING
		do
			Result := primary_prototype.table_name + "_join"
		end


	secondary_join_alias: STRING
		do
			Result := secondary_prototype.table_name + "_join"
		end

	secondary_join_column (a_db_field: STRING): STRING
		require
			secondary_prototype.fields_mapping.object_comparison -- even if as invariant if somebody tries to change it here's the reason why
--			valid_a_db_field: secondary_prototype.fields_mapping_has (a_db_field) or secondary_prototype.relationships.has_field
		do
			Result := secondary_join_alias + "." + a_db_field
		end

	secondary_join_as_field_name (a_db_field: STRING): STRING
		do
			Result := secondary_join_alias + "_" + a_db_field
		end

	default_secondary: S
			-- returns a new instance of secondary
		do
			create Result
		end

feature -- Comparison

	is_types_equal (other: RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]): BOOLEAN
		do
--			Result := primary_type.generating_type.out.is_equal (other.primary_type.generating_type.out) and then
--				secondary_type.generating_type.out.is_equal (other.secondary_type.generating_type.out)
			Result := generating_type.out.is_equal (other.generating_type.out)
			if Result then
				if attached {IDENTIFIABLE} Current as l_ident and then
					attached {IDENTIFIABLE} other as l_o_ident
				then
					Result := l_ident.identifier.is_equal (l_o_ident.identifier)
				end
			end
		end

feature -- Output

	out: STRING
		do
			Result := ({P}).generating_type.out + "->" + ({S}).generating_type.out + " FALLIBLE:" + Precursor
		end

end
