note
	description: "Summary description for {REST_REQUEST}. Could not aford creating current entity to a WSF_REQUEST, thats the reason of clientelism instead of inheritence"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	REST_REQUEST

inherit
	DB_REQUEST
		redefine
			out
		end

create
	make


feature {NONE} -- Initialize


	make (a_wsf_request: like request; a_calling_entity: like calling_entity)
		do
			request := a_wsf_request
			calling_entity := a_calling_entity
			set_request_data
			default_create
			if has_request_data then
				set_request_feature
			end
			if attached {RELATED_DB_ENTITY} calling_entity then
				set_relationships_from_data
			end

			set_constraints_from_request_data
		ensure
			request = a_wsf_request
			calling_entity = a_calling_entity
		end

feature -- Access

	calling_entity: DB_ENTITY

	request_data: STRING

	request_feature: detachable REST_REQUEST_FEATURE -- only if Action_key

	request: WSF_REQUEST

feature -- Status report

	Action_key: STRING = "action"
	Get_all_key: STRING = "get_all"
	Relationships_key: STRING = "relationships"
	Constraints_key: STRING = "constraints"

	has_relationships_in_path: BOOLEAN
			-- contains at least 2 blocks '/entity/entity/[.*]'
		local
			l_pattern: STRING
		do
			l_pattern := "^\/[a-z_0-9]+\/[a-z_0-9]+\/.*"
--			Result := request.path_info.occurrences ('/') >= 3
--				and parameters_count >= 1
--			logger.write_debug( {SIT_UTIL}.match_regex ("/test/achier", "\/").out )
--			logger.write_debug( {SIT_UTIL}.match_regex ("/someArg/anotherArg/1", "^\/").out )
--			logger.write_debug( {SIT_UTIL}.match_regex ("/test/achier/", "^\/.*\/").out )
--			logger.write_debug( {SIT_UTIL}.match_regex ("/company/branch/1", L_pattern).out )

			Result := {SIT_UTIL}.match_regex (request.path_info, l_pattern)
--			logger.write_information ("has_relationships matching " + L_pattern + "=" + L_pattern.to_string_32 + ":" + Result.out)
		end


	has_request_feature: BOOLEAN
		do
			Result := attached request_feature
		end

	path_parameters_count: INTEGER
		do
			if attached request.path_parameters as l_params then
				across -- seems the only way to get it is through iterator...
					l_params as l_param
				loop
					Result := Result + 1
				end
			end
		end

	has_as_secondary (a_string: STRING): BOOLEAN
		do
			Result := request.path_info.has_substring (a_string)
		end

	primary_key: INTEGER_64
		require
			has_valid_primary_key
		do
			if attached {WSF_STRING} request.path_parameter ({PK_DB_ENTITY}.Primary_key_db_column_name) as l_pk and then l_pk.is_integer then
				Result := l_pk.value.to_integer_64
			end
		end

	has_valid_primary_key: BOOLEAN
			-- taking {DB_ENTITY}.Primary_key_db_column_name
		require
			valid_request: attached request.path_info
		do
			Result := has_primary_key_parameter
				and attached {WSF_STRING} request.path_parameter ({PK_DB_ENTITY}.Primary_key_db_column_name) as l_pk
				and then l_pk.is_integer
		end

	has_primary_key_parameter: BOOLEAN
			-- taking {DB_ENTITY}.Primary_key_db_column_name
		do
			Result := attached {WSF_STRING} request.path_parameter ({PK_DB_ENTITY}.Primary_key_db_column_name)
		end

	has_request_data: BOOLEAN
		do
			Result := not request_data.is_empty
		end

	has_constraints: BOOLEAN
		do
			Result := constraints.count > 0
		end

	is_get_all: BOOLEAN
		local
			l_json_parser: JSON_PARSER
		do
			if has_request_data then
				create l_json_parser.make_with_string (request_data)
				l_json_parser.parse_content
				if l_json_parser.is_valid then
					if
						attached l_json_parser.parsed_json_object as l_parsed_general_obj and then
						attached {JSON_OBJECT} l_parsed_general_obj.item (Get_all_key) as l_parsed_get_all_obj
					then
						Result := True
					end
				end
			end
		end

	param_with_error_and_assertion (a_parameter_index: INTEGER; an_error_msg: STRING; a_fallible: FALLIBLE ): detachable like request_feature.arguments.item
			-- returns the given parameter index if found, otherwise sets last_error with given msg
			--
			-- `a_fallible`: The fallible to set last error with `an_error_msg` with
		require
			attached request_feature as l_req_f
			l_req_f.has_arguments
			l_req_f.arguments.valid_index (a_parameter_index)
		do
			if
				attached request_feature as l_req_f and then
				attached l_req_f.arguments.at (a_parameter_index) as l_res
			then
				Result := l_res
			else
				a_fallible.set_last_error (create {INVALID_REQUEST}.make (an_error_msg + " parameter index:" + a_parameter_index.out))
			end
		ensure
			not_parameter_error: attached Result
		end

feature -- Status setting

	set_relationships_from_data
			-- sets relationships_from_data from request_data
		local
			l_json_parser: JSON_PARSER
		do
--			logger.write_debug ("set_relationships_from_data->Received object:-" + request_data + "-")
			if has_request_data and not request_data.is_empty then
				create l_json_parser.make_with_string (request_data)
				l_json_parser.parse_content
				if l_json_parser.is_valid then
					if
						attached l_json_parser.parsed_json_object as l_parsed_general_obj and then
						attached {JSON_OBJECT} l_parsed_general_obj.item (Get_all_key) as l_parsed_get_all_obj and then
						attached {JSON_ARRAY} l_parsed_get_all_obj.item (Relationships_key) as l_relationships
					then
						across
							l_relationships as l_json_rel
						loop
							if
								attached {JSON_OBJECT} l_json_rel.item as l_relationship_o
							then
								add_json_o_relationship (l_relationship_o)
							else
								logger.write_warning ("set_relationships_from_data->untreated relationship type:" + l_json_rel.out)
								check
									untreated_relationship_string: False
								end
							end
						end
					end
				else
					logger.write_debug ("set_relationships_from_data->Received object-> Error parsing object, received:-" + request_data + "- Maybe its not a feature request...")
				end
			end
		end

feature -- Element Change		

	add_constraint (a_relationship_o: JSON_OBJECT)
			-- Add constraint as PK_DB_ENTITY with id only for the moment, so that it can be taken in consideration in query
		local
			l_entity_name, l_table_name: STRING
			l_cons: CONSTRAINT
		do
			across
				a_relationship_o as l_json_o_rel
			loop
				l_entity_name := l_json_o_rel.key.unescaped_string_8
				if attached {JSON_OBJECT} l_json_o_rel.item as l_json_o and then
					attached l_json_o.number_item ({PK_DB_ENTITY}.Primary_key_db_column_name) as l_pk_json_n
				then
					if attached {PK_DB_ENTITY} {SIT_ENVIRONMENT}.app_instance.handled_entity_item_prototype (l_entity_name) as l_pk_db_e_prototype then
						if attached {RELATED_PK_DB_ENTITY} l_pk_db_e_prototype as l_rel_pkdbe_proto then
							l_table_name := l_entity_name + "_join"
						else
							l_table_name := l_pk_db_e_prototype.table_name
						end
						create {FIELD_CONSTRAINT} l_cons.make (l_table_name, l_pk_db_e_prototype.primary_key_db_column_name, {CONSTRAINT}.Equal_operator, l_pk_json_n.integer_64_item)
						constraints.extend (l_cons)
					else
						check
							do_i_have_to_handle_something: False
						end
					end
				end
			end
		ensure
			constraint_added: constraints.count = old constraints.count + 1
		end

	add_json_o_relationship (a_relationship_o: JSON_OBJECT)
			-- extends relationships_from_data if relationship found into a_relationship_o and sets them to loadable_from_resultset
			-- based on calling_entity to find and add references to relationships
		require
			not a_relationship_o.map_representation.is_empty
			attached {RELATED_DB_ENTITY} calling_entity
		local
			l_relationship_type_s, l_relationship_ident: STRING
			l_curs: TABLE_ITERATION_CURSOR[JSON_VALUE, JSON_STRING] -- like a_relationship_o.new_cursor
		do
			l_curs := a_relationship_o.new_cursor
			l_relationship_type_s := l_curs.key.item
			if attached {JSON_STRING} l_curs.item as l_json_s then
				l_relationship_ident := l_json_s.item
				if attached {RELATED_DB_ENTITY} calling_entity as l_calling_entity then
					if attached l_calling_entity.relationship_where_secondary_is_type_identifiable (l_relationship_type_s, l_relationship_ident) as l_found_rel then
						add_relationship_loadable_from_resultset (l_found_rel)
					elseif attached l_calling_entity.relationship_where_secondary_is_type (l_relationship_type_s) as l_found_rel then -- could be dangerous... lets see!! risk our life!
						add_relationship_loadable_from_resultset (l_found_rel)
					else
						logger.write_error ("add_relationship-> relationship '" + l_relationship_type_s + "' not found into relationships of :" + l_calling_entity.generating_type.out + " either add it or avoid asking for it ;-)")
						check
							not_happening: False
						end
					end
				-- else covered by require
				end
			else
				check
					strange_object: False
				end
			end

		ensure
--			relationship_added: attached relationships_from_data as l_rel implies
--				attached old relationships_from_data as l_old_rel and then
--				l_rel.count = l_old_rel.count + 1
--			relationship_added_old_void: not attached old relationships_from_data implies
--				attached relationships_from_data as l_new_rel and then l_new_rel.count = 1
--			relationships_added_old_attached: attached old relationships_from_data as l_old_rel implies
--				attached relationships_from_data as l_new_rel and then
--				l_new_rel.count = l_old_rel.count + 1 --TODO: not working
		end

feature -- Output

	out: STRING
		do
			Result := "Request:" + request.debug_output
		end

feature {NONE} -- Implementation

	set_constraints_from_request_data
			-- only if get_all is called
		require
			calling_entity /= Void
		local
			l_json_parser: JSON_PARSER
		do
			create constraints.make
			if has_request_data then
				create l_json_parser.make_with_string (request_data)
				l_json_parser.parse_content
				if l_json_parser.is_valid then
					if
						attached l_json_parser.parsed_json_object as l_parsed_general_obj and then
						attached {JSON_OBJECT} l_parsed_general_obj.item (Get_all_key) as l_parsed_get_all_obj and then
						attached {JSON_ARRAY} l_parsed_get_all_obj.item (Constraints_key) as l_constraints
					then
						across
							l_constraints as l_json_const
						loop
							if
								attached {JSON_OBJECT} l_json_const.item as l_const_o
							then
								add_constraint (l_const_o)
							else
								logger.write_warning ("set_constraints->untreated relationship type:" + l_json_const.item.out)
								check
									untreated_relationship_string: False
								end
							end
						end
					end
				else
					logger.write_debug ("set_constraints->Received object-> Error parsing object, received:-" + request_data)
				end
			end
		end

	set_request_data
			-- sets request_data from request attribute
		do
			create request_data.make_empty
			request.read_input_data_into (request_data)
		end



	set_request_feature
			-- if an associated action can be retreived from request data request_action will be attached
		require
			has_request_data
		local
			l_json_parser: JSON_PARSER
			l_request_feature: like request_feature
		do
			logger.write_information ("set_request_feature->Received object:-" + request_data + "- request:" + request.request_uri)
			if not request_data.is_empty then
				create l_json_parser.make_with_string (request_data)
				l_json_parser.parse_content
				if l_json_parser.is_valid then
					if
						attached l_json_parser.parsed_json_object as l_parsed_general_obj and then
						attached {JSON_OBJECT} l_parsed_general_obj.item (Action_key) as l_parsed_action_json_obj
					then
						create l_request_feature.make_from_json_object (l_parsed_action_json_obj)
						if l_request_feature.is_valid then
							request_feature := l_request_feature
						end
					end
				else
					logger.write_debug ("set_request_feature->Received object-> Error parsing object, received:-" + request_data + "- Maybe its not a feature request... request:" + request.request_uri)
				end
			end
		end

end
