note
	description: "Summary description for {CHARGE_TEMPLATE_DB_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CHARGE_TEMPLATE_DB_SERVICE


inherit
	DB_SERVICE[CHARGE_TEMPLATE]
		redefine
			make,
			set_callable_features
		end

create
	make

feature {NONE} -- Initialization

	make (a_db_connection: attached DB_CONNECTION)
		do
			Precursor (a_db_connection)
		end

	set_callable_features
			--redefine me if needed to extend available features of DB Service[DB_ENTITY]
		do
			precursor
			callable_features.put (agent import_from_csv (?), Rest_feature_import_from_csv)
			callable_features.put (agent max_charge_for_id (?), Rest_feature_max_charge_for_id)
		end

feature -- Access

	Csv_import_date_format: STRING = "mm/dd/yy"
--	Csv_import_date_format: STRING = "dd-mm-yyyy"
	Csv_import_time_format: STRING = "hh:mi:ss"

	Rest_feature_import_from_csv: STRING = "import_from_csv"
	Rest_feature_max_charge_for_id: STRING = "max_charge_for_id"

feature {NONE} -- Rest Callable features

	import_from_csv (a_rest_request: REST_REQUEST): detachable like items
			-- data: { "action": { "feature": "import_from_csv", "arg1": "/home/pg/data/solarity/sit-docs/sit_solar_offer/charge_templates/curva_1545344_Consumos_Horarios_Arcor.csv", "arg2": "1"} }
			--
			-- argument 1 is file_name
			-- argument 2 is charge_template_id
			-- Works with item and last_item_id
			--
			-- 14'20'' to import all on pgHwWs1
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 2
			first_argument_is_file_name: attached l_req_f.arguments.at(1) as l_arg1 and then
				not l_arg1.is_empty
			second_argument_is_integer: attached l_req_f.arguments.at(2) as l_arg2 and then
				l_arg2.is_integer and then
				l_arg2.to_integer > 0
			not header_has_been_passed
		local
			l_csv_handler: UT_CSV_HANDLER
			l_is: KL_STRING_INPUT_STREAM
			l_file: KL_TEXT_INPUT_FILE
			l_to_read_count_at_once: INTEGER
		do
			csv_current_line_index := 1
			if attached a_rest_request.request_feature as l_req_f and then
				attached l_req_f.arguments.at(1) as l_fn and then
				attached l_req_f.arguments.at(2) as l_charge_template_id_s and then
				l_charge_template_id_s.is_integer_64
			then
				create l_file.make (l_fn)
				l_to_read_count_at_once := l_file.count
				l_file.open_read
				create l_csv_handler.make_with_separator (';')
				if attached l_file.last_string as l_file_s then
					-- set item
					load_from_primary_key (l_charge_template_id_s.to_integer_64)
					-- read file
					l_file.read_string (l_to_read_count_at_once)
					create l_is.make (l_file.last_string)
					l_csv_handler.read_file (l_is, agent impl_action_for_each_csv_line (?, "imported: charge_template:" + l_charge_template_id_s + " fn:" + l_fn))
				else
					check
						file_empty: False
					end
				end
				l_file.close
				if attached item as l_item then
					if l_item.has_charge_units then
						create_or_update_item
					else
						check
							False
						end
					end
				else
					check
						item_not_void: False
					end
				end
			else
				check
					something_s_wrong_with_parameters_of_qry: False
				end
			end
		end

	max_charge_for_id (a_rest_request: REST_REQUEST): detachable like items
			-- data: { "action": { "feature": "max_charge_for_id", "arg1": "1"} }
			--
			-- argument 1 is id of charge_template
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 1
			first_argument_is_integer: attached l_req_f.arguments.at(1) as l_arg1 and then
				l_arg1.is_integer and then
				l_arg1.to_integer > 0
		local
			l_qry: COMPOSED_SQL_QUERY
			l_item_prototype_clone: like item_prototype
		do
			if attached a_rest_request.request_feature as l_req_f and then
				attached l_req_f.arguments.at(1) as l_charge_template_id_s and then
				attached {INTEGER_64} l_charge_template_id_s.to_integer_64 as l_charge_template_id
			then
				l_item_prototype_clone := item_prototype.deep_twin
				l_item_prototype_clone.set_primary_key (l_charge_template_id)
				create l_qry.make (l_item_prototype_clone)
				l_qry.join_clauses.append (l_item_prototype_clone.charge_unit_relationship.to_joins)
				l_qry.add_constraint (l_item_prototype_clone.charge_unit_relationship.to_constraint)
--				l_qry.select_clauses.extend ("MAX(" + l_item_prototype_clone.charge_unit_relationship.secondary_prototype.charge_db_column_name + ")")
				l_qry.select_clauses.extend (l_item_prototype_clone.table_name + ".*")
				l_qry.select_clauses.append (l_item_prototype_clone.charge_unit_relationship.to_api_select_clauses)
				l_qry.order_clauses.wipe_out
				l_qry.order_clauses.extend (create {ORDER_CLAUSE}.make (l_item_prototype_clone.charge_unit_relationship.secondary_prototype.table_name, l_item_prototype_clone.charge_unit_relationship.secondary_prototype.Value_db_column_name, {ORDER_CLAUSE}.way_descending))
				l_qry.set_limit (1)

				item_prototype.charge_unit_relationship.set_loadable_from_resultset (True)
				load_entities (l_qry)
				Result := items
			else
				check
					invalid_parameters: False
				end
			end
		end

feature {NONE} -- Implementation

	csv_current_line_index: INTEGER -- starting from 1

	csv_start_line_index: INTEGER = 1 -- starting from 1

	impl_action_for_each_csv_line (a_csv_line: DS_ARRAYED_LIST [STRING_8]; l_cu_name: STRING)
			-- Adds to item's charge_unit_relationship the given line
			--
			-- Tips:
			--	delete from charge_unit where measuring_point_id IS NULL;
			--	TRUNCATE charge_template_charge_unit CASCADE;
		require
			item /= Void
		local
			l_column_index, l_minutes_add: INTEGER
			l_date_time: DATE_TIME
--			l_date: DATE
--			l_time: TIME
			l_cu: CHARGE_UNIT
			l_datetime_format: STRING
			l_charge: like {CHARGE_UNIT}.value
			l_is_header_line, l_valid_void_charge: BOOLEAN
		do
--					l_datetime_format := Csv_import_date_format + " " + Csv_import_time_format
--					l_datetime_format := "dd-mm-yyyy hh:mi:ss "
			l_datetime_format := "yyyy/mm/dd hh:mi:ss"
			l_valid_void_charge := True
			l_minutes_add := 0

			if csv_current_line_index >= csv_start_line_index then
				if not header_has_been_passed then
					set_header_passed (a_csv_line)
					l_is_header_line := is_header_line (a_csv_line)
				end
				if header_has_been_passed and not l_is_header_line then
					from
						l_column_index := 1
					until
						l_column_index > a_csv_line.count
					loop
						inspect
							l_column_index
						when 1 then -- Datetime
							create l_date_time.make_from_string (a_csv_line.item (l_column_index), l_datetime_format)
							l_date_time.minute_add (l_minutes_add) -- For chile timezone
							if attached l_date_time as l_date_time_a then
								create l_cu.make_from_date_time (l_date_time_a)
							end
						when 2 then --
							check
								l_cu /= Void
								a_csv_line.item (l_column_index).is_real_32 or l_valid_void_charge
							end
							if attached l_cu as l_cu_a then
								if attached a_csv_line.item (l_column_index) as l_s then
									if l_s.is_empty and then l_valid_void_charge then
										create l_charge
									else
--								l_charge := (a_csv_line.item (l_column_index).to_real * 1000).truncated_to_integer_64
										l_charge := (l_s.to_real).truncated_to_integer_64
									end
								else
									create l_charge
									check
										not_happening: False
									end
								end

								logger.write_information (csv_current_line_index.out + ": date:" + a_csv_line.item (l_column_index - 1) + " charge raw (" +
									a_csv_line.item (l_column_index) + "): " + l_charge.out)
								l_cu_a.set_value (l_charge)
							end
						else
							do_nothing
						end

						l_column_index := l_column_index + 1
					end
					check
						l_cu /= Void
						l_cu.valid_for_insert
					end
					if attached item as l_item and then
						attached l_cu as l_cu_a
					then
						l_cu_a.set_name (l_cu_name + " line:" + csv_current_line_index.out)
						l_item.charge_unit_relationship.extend (l_cu_a)
					end
				end
			end
			csv_current_line_index := csv_current_line_index + 1
		ensure
			csv_line_index_incremented: csv_current_line_index = old csv_current_line_index + 1
		end

	header_has_been_passed: BOOLEAN

	set_header_passed (a_csv_line: DS_ARRAYED_LIST[STRING_8])
			-- sets header_has_been_passed if found into given list and still not has been setted
		do
			if not header_has_been_passed and then
				is_header_line (a_csv_line)
			then
				header_has_been_passed := True
			end
		end

	is_header_line (a_csv_line: DS_ARRAYED_LIST[STRING_8]): BOOLEAN
		do
			Result := a_csv_line.at (1).is_equal ("Date") and then
				a_csv_line.at (2).is_equal ("kW")
		end

end
