note
	description: "Summary description for {CUSTOMER_INVOICE_DB_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CUSTOMER_INVOICE_DB_SERVICE

inherit
	DB_SERVICE[CUSTOMER_INVOICE]
		redefine
			make,
			set_callable_features
		end

create
	make

feature {NONE} -- Initialization

	make (a_db_connection: attached DB_CONNECTION)
		do
			Precursor (a_db_connection)
		end

	set_callable_features
			--redefine me if needed to extend available features of DB Service[DB_ENTITY]
		do
			precursor
			callable_features.put (agent pdf_data (?), Rest_feature_pdf_data)
		end

feature -- Access

	Rest_feature_pdf_data: STRING = "pdf_data"


feature {NONE} -- Rest Callable features

	Uf_for_kwh_json_key: STRING = "uf_kwh"
	Clp_for_uf_json_key: STRING = "clp_uf"
	Clp_for_kwh_json_key: STRING = "clp_kwh"
	Solarity_json_o_key: STRING = "solarity"
	Total_without_vat_json_key: STRING = "total_without_vat"
	Total_vat_json_key: STRING = "total_vat"
	Total_with_vat_json_key: STRING = "total_with_vat"
	Last_months_generation_json_key: STRING = "last_months_generation"
	Last_months_saving_json_key: STRING = "last_months_saving"
	Last_months_data_count: INTEGER = 12 -- how many months data do you want to be exported
	Due_to_date_json_key: STRING = "due_date"
	Distribution_company_price_json_key: STRING = "distribution_company_price"
	Sit_discount_json_key: STRING = "sit_discount"
	Consumption_amount_json_key: STRING = "consumption_amount"




	pdf_data (a_rest_request: REST_REQUEST): detachable like callable_feature_result
			-- data: { "action": { "feature": "pdf_data", "arg1": "1"} }
			--
			-- argument 1 is customer_invoice id
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 1
			first_argument_is_integer: attached l_req_f.arguments.at(1) as l_arg1 and then
				l_arg1.is_integer and then
				l_arg1.to_integer > 0
		local
			l_db_request: DB_REQUEST
			l_json_o: JSON_OBJECT
			l_id: like {PK_DB_ENTITY}.primary_key
		do
			if attached a_rest_request.request_feature as l_req_f and then
				attached l_req_f.arguments.at(1) as l_id_s and then
				l_id_s.is_integer_64
			then
				create l_db_request
				-- set id
				l_db_request.constraints.extend (create {FIELD_CONSTRAINT}.make (item_prototype.table_name, item_prototype.Primary_key_db_column_name, {FIELD_CONSTRAINT}.equal_operator, l_id_s))
				-- load relationships
				item_prototype.relationships.do_all (agent (a_rel: RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]; a_rest_req: DB_REQUEST)
					do
						a_rest_req.add_relationship_loadable_from_resultset (a_rel)
					end (?, l_db_request))
				load_with_relationships (l_db_request)
				if items.count > 0 then
					item := items.first
					items.wipe_out
				end
				if attached item as l_item then
					l_json_o := l_item.to_json
					if attached l_item.branch_relationship.secondary as l_branch then
						l_id := l_branch.primary_key
						l_json_o.put (company (l_id).to_json, "company")
					else
						check
							belonging_company_not_found: False
						end
					end
					-- computed data to avoid doing it with ng
					l_json_o.put_real (l_item.uf_for_kwh, Uf_for_kwh_json_key)
					l_json_o.put_real (l_item.uf.item, Clp_for_uf_json_key)
					l_json_o.put_real (l_item.clp_for_kwh, Clp_for_kwh_json_key)
					l_json_o.put_real (l_item.total_without_vat, Total_without_vat_json_key)
					l_json_o.put_real (l_item.total_vat, Total_vat_json_key)
					l_json_o.put_real (l_item.total_with_vat, Total_with_vat_json_key)
					l_json_o.put_real (l_item.distribution_company_price, Distribution_company_price_json_key)
					l_json_o.put_string ({JSON_UTIL}.date_to_api_export (l_item.due_to_date), Due_to_date_json_key)
					add_last_months_data (l_json_o)

					l_json_o.put ({APP_CONFIGURATION}.app_instance.solarity_json_o, Solarity_json_o_key)

					Result := l_json_o.representation
				end
			end
		end

feature {NONE} -- Implementation

	company (a_branch_id: like {PK_DB_ENTITY}.primary_key): COMPANY
		local
			l_dbr: DB_REQUEST
			l_br: BRANCH
			l_comp: COMPANY
		do
			create l_br
			if attached {DB_SERVICE[BRANCH]} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type (l_br.generating_type) as l_serv then
				create l_dbr
				l_dbr.add_relationship_loadable_from_resultset (l_serv.item_prototype.company_relationship)
				l_dbr.constraints.extend (create {FIELD_CONSTRAINT}.make (l_br.table_name, l_br.primary_key_db_column_name, {CONSTRAINT}.equal_operator, a_branch_id))
				l_serv.load_with_relationships (l_dbr)
				if l_serv.items.count > 0 and then
					attached l_serv.items.first.company_relationship.secondary as l_res_comp
				then
					create l_comp
					if attached {DB_SERVICE[COMPANY]} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type (l_comp.generating_type) as l_serv_c then
						create l_dbr
						l_dbr.add_relationship_loadable_from_resultset (l_serv_c.item_prototype.billing_city_relationship)
						l_dbr.constraints.extend (create {FIELD_CONSTRAINT}.make (l_res_comp.table_name, l_res_comp.primary_key_db_column_name, {CONSTRAINT}.equal_operator, l_res_comp.primary_key))
						l_serv_c.load_with_relationships (l_dbr)
						if l_serv_c.items.count > 0 and then
							attached l_serv_c.items.first as l_res
						then
							Result := l_res
						else
							create Result
							check
								company_not_found: False
							end
						end
					else
						create Result
						check
							service_not_found: False
						end
					end
				else
					create Result
					check
						company_not_found: False
					end
				end
			else
				create Result
				check
					service_not_found: False
				end
			end
		end

	add_last_months_data (a_json_o: JSON_OBJECT)
			-- extends given JSON_OBJECT with last month generation values and discount values
		require
			attached item as l_item1
		local
			l_m_start_date, l_m_end_date: like item.a_date
			l_months_index: INTEGER
			l_req: DB_REQUEST
			l_old_item: like item
			l_old_items: like items
			l_json_array: JSON_ARRAY
			l_json_o, l_month_details_json_o: JSON_OBJECT
		do
			create l_json_array.make (Last_months_data_count)
			l_old_item := item
			l_old_items := items
			if attached item as l_item then
				from
					l_m_start_date := l_item.a_date.deep_twin
					l_m_end_date := l_item.a_date.deep_twin
					l_m_start_date.set_day (1)
					l_m_end_date.set_day (l_m_end_date.days_in_i_th_month (l_m_end_date.month, l_m_end_date.year))
					l_months_index := 1
				until
					l_months_index > Last_months_data_count
				loop
					l_m_start_date.month_add (-1)
					l_m_end_date.month_add (-1)

					create l_req
					l_req.add_relationship_loadable_from_resultset (item_prototype.electricity_rate_relationship)
					l_req.add_relationship_loadable_from_resultset (item_prototype.branch_relationship)
					l_req.constraints.extend (create {DATE_CONSTRAINT}.make_from_interval (item_prototype.table_name, l_m_start_date.to_date_time, l_m_end_date.to_date_time, item_prototype.Date_column_name))
					l_req.constraints.extend (create {FIELD_CONSTRAINT}.make (item_prototype.table_name, item_prototype.Branch_id_db_column_name, {CONSTRAINT}.equal_operator, l_item.branch_pk))
					load_with_relationships (l_req)
					if items.count > 0 then
						create l_json_o.make
						create l_month_details_json_o.make
						l_month_details_json_o.put_real (items.first.consumption_amount, Consumption_amount_json_key)
						l_month_details_json_o.put_real (items.first.sit_discount, Sit_discount_json_key)
						l_json_o.put (l_month_details_json_o, {JSON_UTIL}.date_to_api_export (items.first.a_date))
						l_json_array.extend (l_json_o)
					end
					l_months_index := l_months_index + 1
				end
				a_json_o.put (l_json_array, Last_months_generation_json_key)
			end
			items := l_old_items
			item := l_old_item -- restore item
		ensure
			item = old item
			items = old items and then items.count = old items.count
		end


end
