﻿note
	description: "Summary description for {DB_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	DB_SERVICE [G -> DB_ENTITY create default_create, make_from_db_service, make_from_json end]

inherit
	DB_SERVICE_OPERATIONS
		redefine
			default_create,
			make,
			set_last_error,
			set_last_error_from_fallible
		select
			out
		end

	SELECTABLE_SERVICE
		rename
			out as selectable_out
		redefine
			default_create
		end

	ACTION
		rename
			out as action_out
		redefine
			default_create,
			start,
			execute
		end

create
	make

feature {NONE} -- Creation

	default_create
		do
			create items.make
			create last_column_names.make
			create insert_if_not_exists_field_names.make
			create item_prototype

			Precursor {DB_SERVICE_OPERATIONS}
			Precursor {SELECTABLE_SERVICE}
			Precursor {ACTION}
		end

	make (a_db_connection: attached DB_CONNECTION)
		do
			create items.make
			create last_column_names.make
			create insert_if_not_exists_field_names.make
			create item_prototype

			Precursor (a_db_connection)
			set_callable_features

			set_base_selection_action (Current) -- setting the execute and start actions from this class to be executed
		end

feature -- Access

	Query_limit: STRING = " LIMIT 1000" -- for test perpose only

	item: detachable G

	item_prototype: G

	callable_feature_result: detachable STRING

	last_column_names: LINKED_LIST [STRING] -- Column names of last resultset (check if empty not if Void we are on Eiffel!...)

	items: SIT_LINKED_LIST [G] -- content of last resultset and ids

	insert_if_not_exists: BOOLEAN -- Does the insert statement include an if not exists statement
	insert_if_not_exists_field_names: LINKED_LIST[DB_FIELD] -- a list of field names to add to insert if not exists statement

	data_from_column_name (a_column_name: STRING): detachable like {DB_TUPLE}.item
			--returns a deep twin of item @ a_column_name if any
		require
			attached last_column_names
				and then attached cursor
		local
			i: INTEGER
			l_db_cursor: DB_TUPLE
			l_column_name: STRING
		do
			if attached last_column_names as l_column_names then
				create l_db_cursor.copy(cursor)
				from
					i := 1
				until
					attached Result or else i > l_db_cursor.count
				loop
					if attached l_db_cursor.column_name (i) as l_a_column_name and then
						l_a_column_name.is_equal (a_column_name)
					then
						l_column_name := l_a_column_name
						if attached l_db_cursor.item (i) as l_attached_value then
							Result := l_attached_value.deep_twin
						else
							logger.write_notice ("data_from_column_name->not found value for column:" + l_column_name)
						end
					end
					i := i + 1
				end
			end
		ensure
			cursor_unmoved: cursor.is_equal (old cursor)
		end

	is_multirow_resultset: BOOLEAN

feature -- Status Report

	last_inserted_id: like {PK_DB_ENTITY}.primary_key
		-- if <= 0 then invalid

	item_class_name: STRING
		do
			Result := ({attached like item}).out
		end

	item_table_name: STRING
		do
			Result := {SIT_UTIL}.class_name_lowercase (({attached like item}).out)
		end

	ready_for_select, ready_for_update: BOOLEAN
		do
			Result := not has_error
		end

	items_all_emtpy: BOOLEAN
		do
			Result := items.count = 0 and item = Void
		end

	selected (a_rest_request: REST_REQUEST): like item
			-- Default is id 1
			-- If found into configuration file, this one is choosen
			-- BORDER EFFECT: Does replace item or items with retreived item
		local
			l_i: INTEGER_64
		do
			l_i := 1
			if attached {APP_CONFIGURATION}.app_instance.selected_entity_primary_key ({SIT_UTIL}.class_name_lowercase (({like item_prototype}).out)) as l_pk then
				l_i := l_pk.to_integer_64
				if l_i = -1 then
					load_last_from_primary_key
				else
					load_from_primary_key (l_i)
				end
				if not has_error then
					Result := item
				end
			end
		end

	current_result_data_from_column_name (a_column_name: STRING) : detachable ANY
		local
			l_db_cursor: DB_TUPLE
			l_column_name: STRING
			i: INTEGER
		do
			create l_db_cursor.copy(cursor)
			from
				i := 1
			until
				i > l_db_cursor.count or
				Result /= Void
			loop
				if attached l_db_cursor.column_name (i) as l_a_column_name then
					l_column_name := l_a_column_name
					if attached l_db_cursor.item (i) as l_attached_value then
						Result := l_attached_value.deep_twin
					else
						logger.write_notice ("current_result_data_from_column_name-> has Void value with called column name:" + l_column_name)
					end
					i := i + 1
				else
					check
						column_name_not_null: False
					end
				end
			end
		ensure
			cursor_not_moved: cursor.is_equal (old cursor)
		end

	uniq_column_names_in_resultset: BOOLEAN
			-- Does resultset have uniq_column_names?
		local
			l_db_cursor: DB_TUPLE
			l_column_names_list: LINKED_SET[STRING]
			i: INTEGER
		do
			Result := True
			create l_column_names_list.make
			create l_db_cursor.copy(cursor)
			from
				i := 1
			until
				i > l_db_cursor.count or not Result
			loop
				if attached l_db_cursor.column_name (i) as l_a_column_name then
					if l_column_names_list.has (l_a_column_name) then
						logger.write_error ("Column name appears 2 times into resultset:" + l_a_column_name)
						Result := False
					else
						l_column_names_list.extend (l_a_column_name)
					end
					i := i + 1
				else
					check
						column_name_not_null: False
					end
				end
			end
		end

	service_from_entity_name (a_db_entity_name: STRING): detachable DB_SERVICE[DB_ENTITY]
		require
			valid_param: not a_db_entity_name.is_empty
		do
			-- measure_unit type
			if a_db_entity_name.is_equal ({CHARGE_UNIT}.class_name_lowercase) and then
				attached {CHARGE_UNIT_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable CHARGE_UNIT}) as l_dbs
			then
				Result := l_dbs
			elseif a_db_entity_name.is_equal ({TENSION_UNIT}.class_name_lowercase) and then
				attached {TENSION_UNIT_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable TENSION_UNIT}) as l_dbs
			then
				Result := l_dbs
			elseif a_db_entity_name.is_equal ({CURRENT_UNIT}.class_name_lowercase) and then
				attached {CURRENT_UNIT_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable CURRENT_UNIT}) as l_dbs
			then
				Result := l_dbs
			else
				check
					not_happening: False
				end
			end
		ensure
			instance_free: class
		end

feature -- Status setting

	set_item (v: attached like item)
		do
			item := v
		ensure
			item = v
		end

	set_items (v: like items)
		do
			items := v
		ensure
			items = v
		end

	set_is_multirow_resultset (v: like is_multirow_resultset)
		do
			is_multirow_resultset := v
		ensure
			is_multirow_resultset = v
		end

	set_item_prototype (v: like item_prototype)
		do
			item_prototype := v
		ensure
			item_prototype = v
		end

	set_insert_if_not_exists (v: like insert_if_not_exists)
		do
			insert_if_not_exists := v
		ensure
			insert_if_not_exists = v
		end

	load_all
			-- Will make all items available into items
		local
			l_qry: SQL_QUERY
		do
			create l_qry.make (item_prototype)
			load_entities (l_qry)
		end


	load_from_primary_key (primary_key_v: INTEGER_64)
			-- Loads given item into item otherwise item will be Void
			-- 	last_error will give a feedback of success or failure
		require
			attached valid_for_select
		local
			l_new_item: detachable like item
		do
			wipe_all_items
			execute_query (select_query_from_primary_key_query (primary_key_v))
			logger.write_debug ("load_from_primary_key->: Items count:" + items.count.out)
			if has_error then
				logger.write_error ("load_from_primary_key->Execute_query has errors with query:" + last_query)
				set_last_error_from_fallible (db_connection)
				wipe_last_error
			else
				if items.count = 1 then
					 l_new_item := items.first
					 wipe_items
				else
					create {ODBC_ERROR} last_error.make ("load_from_primary_key-> error, retreived items count were not 1 but " + items.count.out)
				end
				item := l_new_item
			end
		ensure
			valid_item: attached {PK_DB_ENTITY} item as it implies it.primary_key = primary_key_v
		end

	load_last_from_primary_key
			-- Loads given item into item otherwise item will be Void
			-- 	last_error will give a feedback of success or failure
		require
			attached valid_for_select
		local
			l_new_item: detachable like item
			l_qry: STRING
		do
			l_qry := "SELECT * FROM " + item_prototype.table_name
			if attached {PK_DB_ENTITY} item_prototype as l_pk_dbe_proto then
				l_qry.append (" ORDER BY " + l_pk_dbe_proto.Primary_key_db_column_name + " DESC")
			end
			l_qry.append (" LIMIT 1")
			wipe_all_items
			execute_query (create {STRING_SQL_QUERY}.make_from_string (item_prototype, l_qry))
			logger.write_debug ("load_last_from_primary_key->: Items count:" + items.count.out)
			if has_error then
				logger.write_error ("load_last_from_primary_key->Execute_query has errors with query:" + l_qry)
				set_last_error_from_fallible (db_connection)
				wipe_last_error
			else
				if items.count = 1 then
					 l_new_item := items.first
				else
					create {ODBC_ERROR} last_error.make ("load_from_primary_key-> error, retreived items count were not 1 but " + items.count.out)
				end
				item := l_new_item
			end
		end

	load_with_relationships (a_db_req: DB_REQUEST)
			-- modifies item_prototype.relationships with loadable_from_resultset according to a_db_req relationships
		require
			a_db_req.has_relationships
		do
			if attached {RELATED_DB_ENTITY} item_prototype as l_related_proto and then
				attached a_db_req.relationships as l_db_req_relationships
			then
				l_related_proto.set_relationship_loadable_from_resultset_list (l_db_req_relationships) -- TODO: replace with agent
			end
			if attached query_with_relationships (a_db_req) as l_qry then
				execute_query (l_qry)
			end
		end

	load_without_relationships (a_db_req: DB_REQUEST)
		do
			if attached query_with_relationships (a_db_req) as l_qry then
				execute_query (l_qry)
			end
		end

	load_items_with_relationships (an_item: attached like item; an_order_clause: detachable ORDER_CLAUSE; some_constraints: detachable LINKED_LIST[CONSTRAINT])
			-- loads items with given arguments
			-- an_item will be
		local
			l_qry: COMPOSED_SQL_QUERY
		do
			create l_qry.make (an_item)
			query_items_with_relationships (an_item, l_qry)
			if attached an_order_clause as l_order_clause then
				l_qry.order_clauses.extend (l_order_clause)
			end
			execute_query (l_qry)
		end

	set_last_error_from_fallible (o: FALLIBLE)
		local
			l_void: like item
		do
			Precursor (o)
			wipe_items
			item := l_void
		ensure then
			items.is_empty
			item = Void
		end

	set_last_error (an_error: attached like last_error)
		do
			Precursor (an_error)
		end

	call_associated_feature (a_rest_request: REST_REQUEST)
			-- Sets item or items
		require
			feature_given_from_post_exists: attached a_rest_request.request_feature as l_req_action and then
				callable_features.has_key (l_req_action.name)
		local
			l_item: like item
		do
			logger.write_debug ("call_associated_feature->" + a_rest_request.out)
			if attached a_rest_request.request_feature as l_req_feature then
				if attached callable_features.item (l_req_feature.name) as l_feature then
					if attached l_feature.item (a_rest_request) as l_feature_result then
						if attached {like item} l_feature_result as l_feature_result_item then
							wipe_items
							item := l_feature_result_item
						elseif attached {like items} l_feature_result as l_items then
							items := l_items
							item := l_item
						elseif attached {like callable_feature_result} l_feature_result as l_res then
							callable_feature_result := l_res
						else
							logger.write_warning ("call_associated_feature--> feature '" + l_req_feature.name + "' could not process result of function, type is " + l_feature_result.generating_type.out)
						end
					else
						logger.write_warning ("call_associated_feature--> feature '" + l_req_feature.name + "' did not return any result")
					end
				else
					logger.write_warning ("call_associated_feature--> associated feature '" + l_req_feature.name + "' has not been found, could not call it")
				end
			end
		end

feature -- Removal


	delete_entity (a_primary_key: INTEGER_64)
			-- Delete from given primary_key object
		require
			valid_db_connection: valid_for_modification
			a_primary_key > 0
		do
			execute_update (delete_query_from_primary_key_query(a_primary_key))
			if has_error then
				set_last_error_from_fallible (db_connection)
			else
				set_last_success_message ("Entity with ID " + a_primary_key.out + " deleted from DB successfully")
			end
		end

	wipe_items
		do
			last_column_names.wipe_out
			items.wipe_out
			logger.write_debug ("wipe_last_items")
		ensure
			items.is_empty
		end

	wipe_all_items
		local
			l_void: like item
		do
			item := l_void
			wipe_items
			logger.write_debug ("wipe_items")
		ensure
			items.is_empty and item = Void
		end

feature -- Element change

	create_or_update_entity (a_rest_request: REST_REQUEST)
			-- Put it in item if created otherwise item will be Void
			-- Working with item and modifying it
		require
			valid_db_connection: valid_for_modification
		do
			create item.make_from_json (a_rest_request.request_data)
			if attached item as l_item and then l_item.has_error then
				set_last_error_from_fallible (l_item)
			else
				create_or_update_item
			end
		end

	update_entity ( a_primary_key: like {PK_DB_ENTITY}.primary_key; a_json_string: STRING)
			-- Put it in item if created otherwise item will be Void
			-- Working with item and modifying it
		require
			valid_db_connection: valid_for_modification
		local
			l_msg: like {SIT_ERROR}.message
		do
			create item.make_from_json (a_json_string)
			if attached {PK_DB_ENTITY} item as l_pk_item then
				check
					valid_item_between_request_param_and_json_object: a_primary_key.is_equal (l_pk_item.primary_key)
				end
				if l_pk_item.has_error then
					set_last_error_from_fallible (l_pk_item)
				else
					execute_update (update_query)
						-- TODO: get and set the ID of newly created instance here
					if has_error then
						l_msg := "last update did not work with json:" + a_json_string
						if attached db_connection.last_error as l_err then
							l_msg.append("-error:" + l_err.message)
						end
						set_last_error(create {UPDATE_ERROR}.make (l_msg))
					else
						set_last_success_message ("Entity updated into DB successfully")
					end
				end
				--else covered by require
			end
		end

	create_or_update_item
			-- Create or update setted item into DB
		require
			attached_item: attached item as l_i
			valid_item_for_insert_or_update: l_i.valid_for_insert or l_i.valid_for_update
		do
			if attached item as l_item then
				create_or_update_item_without_rel
				if not has_error then
					create_secondary_association_if_any
				end
			end
		end

	update_item_s_relationships
		require
			not has_error
		do
			create_secondary_association_if_any
		end

	create_or_update_item_without_rel
			-- sets create and update relationships only if secondary_required
			-- sets last_inserted_id if no error
		require
			attached_item: attached item as l_i
			valid_item_for_insert_or_update: l_i.valid_for_insert or l_i.valid_for_update
		local
			l_qry: like insert_query
			l_is_insert: BOOLEAN
			l_msg: STRING
		do
			if attached item as l_item then
				l_is_insert := l_item.valid_for_insert
				if l_is_insert then
					l_qry := insert_query
				else
					l_qry := update_query
				end
				execute_update (l_qry)
				if has_error then
					l_msg := "last DB modification did not work with entity:" + l_item.out
					if attached db_connection.last_error as l_err then
						l_msg.append ("-error:" + l_err.out)
					end
					set_last_error( create {INSERT_ERROR}.make (l_msg) )
				else
					if l_is_insert then
						set_last_success_message ("Entity created into DB successfully")
					else
						set_last_success_message ("Entity updated into DB successfully")
					end
					-- if second query for joined entity
					if l_is_insert then
						if not insert_if_not_exists then
							set_last_inserted_id
							if attached {PK_DB_ENTITY} item as l_pk_item then
								l_pk_item.set_primary_key (last_inserted_id)
							end
						end
					elseif attached {PK_DB_ENTITY} item as l_pk_item then --- in case of update
						last_inserted_id := l_pk_item.primary_key
					else
						check
							review_your_code_should_not_happen: False
						end
					end
				end
			end
		end


	load_entities (a_qry: SQL_QUERY)
			--executes given query and puts result into items
		do
			execute_query (a_qry)
			if has_error then
				logger.write_critical ("Error while retreiving " + item_prototype.generating_type.out + " from DB")
				set_last_error_from_fallible (Current)
			end
		end


	add_sit_event_report (an_evt_title: like {SIT_EVENT}.title)
		require
			attached item
			title_not_empty: not an_evt_title.is_empty
		local
			l_evt: SIT_EVENT
			l_json: JSON_OBJECT
			l_warnings: JSON_ARRAY
		do
			-- EVENT
			if attached {SIT_EVENT_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable SIT_EVENT}) as l_dbs then
				create l_evt
				create l_json.make
				create l_warnings.make (warnings.count)

				if has_error and then
					attached last_error as l_err
				then
					l_json.put (l_err.to_json, "last_error")
				end
				-- Warnings
				across
					warnings as l_warn
				loop
					l_warnings.extend (create {JSON_STRING}.make_from_string (l_warn.item))
				end
				if l_warnings.count > 0 then
					l_json.put (l_warnings, "warnings")
				end
				-- Content
				if not l_json.is_empty then
					l_evt.set_content (l_json.representation)
				end
				l_evt.set_title (an_evt_title)
				if attached {ENUMERATE_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable ENUMERATE}) as l_enum_dbs then
					if warnings.count > 0 then
						l_evt.set_status (l_enum_dbs.enumerate_cash_from_name ({DB_CONSTANTS}.Status_warn))
					else
						l_evt.set_status (l_enum_dbs.enumerate_cash_from_name ({DB_CONSTANTS}.Status_info))
					end
				else
					check
						should_not_happen: False
					end
				end
				if attached {PK_DB_ENTITY} item as l_item then
					l_evt.add_entity (l_item)
				end

				l_dbs.add_event (l_evt)
			else
				logger.write_error ("import_from_sit_abb_csv-> Unable to locate sit_event_db_service")
				check
					unable_to_locate_sit_event_db_service: False
				end
			end
		end

feature {NONE} -- SQL

	select_all_base_query: STRING
		do
			Result := "SELECT * FROM ONLY " + item_prototype.table_name
		end

	select_all_query: STRING
		do
			Result := "SELECT * FROM ONLY " + item_prototype.table_name + " ORDER BY " + {PK_DB_ENTITY}.Primary_key_db_column_name + " ASC" + Query_limit -- TODO remove limit 50
			logger.write_debug ("select_all_query:'" + Result + "'")
		end

	select_query_from_primary_key_query (primary_key_v: like {PK_DB_ENTITY}.primary_key): SQL_QUERY
		do
			create {STRING_SQL_QUERY} Result.make_from_string (item_prototype,
				"SELECT * FROM ONLY " + item_prototype.table_name + " WHERE " + {PK_DB_ENTITY}.Primary_key_db_column_name + " = " + primary_key_v.out)
			logger.write_debug ("select_query_from_primary_key_query:'" + Result.to_sql + "'")
		end

	delete_query_from_primary_key_query (primary_key_v: like {PK_DB_ENTITY}.primary_key): SQL_UPDATE
		local
			l_qry: STRING
		do
			l_qry := "DELETE FROM ONLY " + item_prototype.table_name + " WHERE " + {PK_DB_ENTITY}.Primary_key_db_column_name + " = " + primary_key_v.out
			create {STRING_SQL_UPDATE} Result.make_from_string (item_prototype, l_qry)
			logger.write_debug ("delete_query_from_primary_key_query:'" + Result.to_sql + "'")
		end

	insert_query: SQL_MODIFY
			-- Based on fields_mapping redifine it for the fields you want to insert
			-- and on item's values
		require
			non_void_item: attached item
			valid_item: attached item as l_item and then l_item.valid_for_insert
		local
			l_key: like {DB_FIELD}.db_field_name
			l_value: ANY
			l_keys, l_values: LINKED_LIST[STRING]
			l_qry_s: STRING
			l_sql_i: SQL_INSERT
		do
			create l_keys.make
			l_keys.compare_objects
			create l_values.make
			l_qry_s := ""
			if attached item as l_item then
				if insert_if_not_exists then
					create l_sql_i.make (l_item)
					l_sql_i.set_on_conflict_action ({SQL_INSERT}.On_conflict_action_do_nothing)
					Result := l_sql_i
				else
					l_qry_s.append ("INSERT INTO " + l_item.table_name)
					-- relationships (add then before because they are prioritary to field_names (ex. enumerate_id)
					add_relationships_for_insert_update (l_keys, l_values, l_item.db_unstored_field_names, False)
					-- entity fields
					across
						l_item.fields_mapping as l_field
					loop
						l_key := l_field.item.db_field_name
						if not l_item.db_unstored_field_names.has (l_key) and then
							not l_keys.has (l_key)
						then
							l_value := l_field.item.value_to_set
							l_keys.extend (l_key)
							l_values.extend ({SQL_UTIL}.from_value_to_db_field_string (l_value, l_key.is_case_insensitive_equal ({PK_DB_ENTITY}.Primary_key_db_column_name)))
						end
					end
					l_qry_s.append ("(" + {SIT_UTIL}.to_coma_separated (l_keys) + ")")
					l_qry_s.append (" VALUES ")
					l_qry_s.append ("(" + {SIT_UTIL}.to_coma_separated (l_values) + ")")
					l_qry_s.append (";")
					create {STRING_SQL_UPDATE} Result.make_from_string (item_prototype, l_qry_s)
				end
				logger.write_debug ("insert_query->" + Result.to_sql)
			else
				create {STRING_SQL_UPDATE} Result.make_from_string (item_prototype, "")
				set_last_error(create {ATTACHED_OBJECT_EXPECTED}.make ("Trying to get insert query of a void item..."))
			end

		end

	update_query: SQL_UPDATE
			-- Based on fields_mapping redifine it for the fields you want to insert
			-- and on item's values
			-- Checks if fields are not into l_item.db_unstored_field_names
		require
			non_void_item: attached item
			valid_item: attached item as l_item and then l_item.valid_for_update
		local
			l_key: like {DB_FIELD}.db_field_name
			l_value: ANY
			l_treated_key, l_treated_value: STRING
			l_add_field_to_set, l_default: BOOLEAN
			l_keys, l_values: LINKED_LIST[STRING]
			l_updt_s: STRING
		do
			l_updt_s := ""
			if attached {PK_DB_ENTITY} item as l_item then
				create l_keys.make
				create l_values.make
				l_updt_s.append ("UPDATE " + l_item.table_name + " SET ")
				across
					l_item.fields_mapping as l_db_field
				invariant
					same_some_keys_some_values_count: l_keys.count = l_values.count
				loop
					l_key := l_db_field.item.db_field_name
					l_value := l_db_field.item.value_to_set
					l_treated_key := l_key
					l_add_field_to_set := True
					l_treated_value := ""
					if
						not attached {INTEGER_64} l_value as l_int and then
						not l_key.is_case_insensitive_equal ({PK_DB_ENTITY}.Primary_key_db_column_name) and then
						not l_item.db_unstored_field_names.has (l_key)
					then -- Don't add id to set as an update statement
						l_default := attached {INTEGER_64} l_value as l_int
							and then l_int <= 0
							and then l_key.is_equal ({PK_DB_ENTITY}.primary_key_db_column_name)
						check
							default_value_should_not_happen_on_an_update: not l_default
						end
						l_keys.extend (l_treated_key)
						l_values.extend ({SQL_UTIL}.from_value_to_db_field_string (l_value, l_default))
					end
				end
				add_relationships_for_insert_update (l_keys, l_values, l_item.db_unstored_field_names, True)
				l_updt_s.append (qry_update_set_fields (l_keys, l_values, l_item.db_unstored_field_names))

				l_updt_s.append (" WHERE " + l_item.Primary_key_db_column_name + " = " + l_item.primary_key.out + ";")
				logger.write_debug ("update_query :" + l_updt_s + "-")
			else
				check
					False
				end
				set_last_error(create {ATTACHED_OBJECT_EXPECTED}.make ("Trying to get insert query of a void item..."))
			end

			create {STRING_SQL_UPDATE} Result.make_from_string (item_prototype, l_updt_s)
		end

	query_with_relationships (a_db_req: DB_REQUEST): SQL_QUERY
			-- Returns a query with inner join from relationships
			-- based on API_EXPORTABLE_ENTITY for secondary
		note
			sample_qry: "[
				SELECT electricity_rate.*, enumerate_city.id AS city_id, enumerate_city.name AS city_name, enumerate_electricity_type.id AS electricity_type_id, enumerate_electricity_type.name AS electricity_type_name 
				FROM electricity_rate
				    INNER JOIN enumerate enumerate_city ON electricity_rate.city_id=enumerate_city.id
				    INNER JOIN enumerate enumerate_electricity_type ON electricity_rate.electricity_type_id=enumerate_electricity_type.id
				WHERE distribution_company_id=25;
			]"
		do
			create {COMPOSED_SQL_QUERY} Result.make (item_prototype)
			Result.constraints.append (a_db_req.constraints)
			Result.order_clauses.append (a_db_req.order_clauses)
			if attached a_db_req.relationships as l_req_relationships and then
				attached {COMPOSED_SQL_QUERY} Result as l_res
			then
				set_query_from_relationships (l_res, l_req_relationships)
				if attached l_req_relationships.first.primary_prototype as l_p_attached then
					set_is_multirow_resultset (False) -- Electricity_rates|
				else
					check
						strange_case_lp_not_attached: False
					end
				end
			end -- if attached
		ensure
--			is_multirow_resultset
		end

	query_items_with_relationships (an_item: attached like item; a_qry: COMPOSED_SQL_QUERY)
			-- returns a query containing given item and relationships
		do
			if attached {RELATED_DB_ENTITY} an_item as l_related_dbe then
				set_query_from_relationships (a_qry, l_related_dbe.relationships)
				set_is_multirow_resultset (True)
			end
		end


feature {NONE} -- Implementation of entities loading

	qry_update_set_fields (some_keys, some_values, some_unstored_field_names: LINKED_LIST[STRING]): STRING
		require
			same_some_keys_some_values_count: some_keys.count = some_values.count
		local
			l_val_c: LINKED_LIST_ITERATION_CURSOR [STRING_8]
		do
			Result := ""
			l_val_c := some_values.new_cursor
			across
				some_keys as l_key
			loop
				check
					invalid_some_keys_content: not some_unstored_field_names.has (l_key.item)
				end
				Result.append (l_key.item + " = " + l_val_c.item + ", ")
				l_val_c.forth
			end
			if Result.ends_with (", ") then
				Result.remove_tail (2)
			end
		ensure
			dont_modify_parameters: old some_keys.is_equal (some_keys) and old some_values.is_equal (some_values)
		end


	load_entity ( a_select_statement: STRING )
		obsolete "use load_entities instead"
		require
			valid_for_select
		local
			l_new_item, just_void: detachable like item
		do
			wipe_items
			execute_query (create {STRING_SQL_QUERY}.make_from_string (item_prototype, a_select_statement))
			if has_error then
				set_last_error_from_fallible (db_connection)
				wipe_last_error
			else
				create l_new_item.make_from_db_service (Current)
				if l_new_item.has_error then
					set_last_error_from_fallible (l_new_item)
					l_new_item := just_void
				end
			end
			item := l_new_item
		ensure
			no_error_implies_new_item: not has_error implies item /= old item
		end

	start
			-- This method is used by the class DB_SELECTION, and is executed after the first
			-- iteration step of 'load_result', it provides some facilities to control, manage, and/or
			-- display data resulting of a query.
			-- In this example, it simply prompts column name on standard output.
		require else
			valid_db_connection: valid_resultset
			attached last_column_names
		local
			i: INTEGER
			tuple: DB_TUPLE
		do
			current_row_pk := Default_current_row_pk
			last_row_pk := Default_current_row_pk
			wipe_items
			if valid_resultset then
				create tuple.copy (cursor)
				from
					i := 1
				until
					i > tuple.count
				loop
					if attached tuple.column_name (i) as s then
						last_column_names.extend (s)
--						logger.write_debug ("DB Query Column " + i.out + ":" + s)
					end
					i := i + 1
				end
			else
				set_last_error (create {ATTACHED_OBJECT_EXPECTED}.make("start: unable to get database cursor"))
			end
			logger.write_information ("Start parsing received data")
		ensure then
			current_row_pk_reseted: current_row_pk = Default_current_row_pk
			last_row_pk = Default_current_row_pk
		end

	current_row_pk: like {PK_DB_ENTITY}.primary_key
	last_row_pk: like {PK_DB_ENTITY}.primary_key
	Default_current_row_pk: like {PK_DB_ENTITY}.primary_key = -1

	execute
			-- Adds to items a created one from Current DB_SERVICE
		do
			if not has_error then
				if next_resultset_is_no_item and then
					attached {PK_DB_ENTITY} item_prototype as l_item_prototype
				then -- For last_inserted_id
					if attached current_result_data_from_column_name (l_item_prototype.Primary_key_db_column_name) as l_pk then
						last_inserted_id := {SIT_TYPE_CONVERTER}.force_as_integer_64 (l_pk)
					else
						check
							cannot_get_last_inserted_id_from_no_pk_db_entity: False
						end
					end
				else
					execute_impl
				end
			end
		end

feature {NONE} -- Implementation

	ids_list_from_string (s: STRING): LINKED_LIST[like {PK_DB_ENTITY}.primary_key]
			-- s: [3,5,6] or 4
		require
			valid_param: not s.is_empty
		local
			l_mp_ids_array_s: LIST[STRING]
		do
			create Result.make
			if s.item (1).is_equal ('[') then
				s.remove_head (1)
				s.remove_tail (1)
				l_mp_ids_array_s := s.split (',')
				across
					l_mp_ids_array_s as l_item
				loop
					Result.extend (l_item.item.to_integer_64)
				end
			else
				Result.extend (s.to_integer_64)
			end
		end

	next_resultset_is_no_item: BOOLEAN
		-- next resultset has to be interpreted as other than item (for implementation), setted by set_last_inserted_id

	set_last_inserted_id
			-- only working with Postgresql
			-- https://stackoverflow.com/a/2944481/2118777
		require
			current_is_pk_db_entity: attached {PK_DB_ENTITY} item_prototype
		local
			l_table, l_column: STRING
		do
			next_resultset_is_no_item := True
			if attached {PK_DB_ENTITY} item_prototype as l_item_proto then
				l_table := l_item_proto.sequence_name
				l_column := l_item_proto.Primary_key_db_column_name
				execute_query (create {STRING_SQL_QUERY}.make_from_string (item_prototype, "SELECT currval(pg_get_serial_sequence('" + l_table + "','" + l_column + "'));"))
				logger.write_debug ("set_last_inserted_id-> last_inserted_id:" + last_inserted_id.out)
			end
			next_resultset_is_no_item := False
		ensure
			last_inserted_id > 0
			not next_resultset_is_no_item
		end

	execute_impl
		require
			not has_error
		local
			l_new_item, l_void: like item
		do
			if is_multirow_resultset and then
				attached {RELATED_PK_DB_ENTITY} item_prototype as l_item_prototype and then
				attached {DB_SERVICE[RELATED_PK_DB_ENTITY]} Current as l_current_as_related_pk_db_entity
			then -- setting attributes from same entity but different results of joined entities
				if attached current_result_data_from_column_name (l_item_prototype.Primary_key_db_column_name) as l_pk then
					current_row_pk := {SIT_TYPE_CONVERTER}.force_as_integer_64 (l_pk)
				else
					check
						pk_not_found_into_resultset: False
					end
				end
				if current_row_pk /= last_row_pk and then
					attached item as l_item
				then -- if line is new entity
					items.extend (l_item)
					item := l_void
				end
				if item = Void then
					item := item_prototype.deep_twin
				end
				if attached {RELATED_PK_DB_ENTITY} item as l_item_related_dbe then
					l_item_related_dbe.set_from_db_service (l_current_as_related_pk_db_entity)
				end
				-- treat error
				if attached {RELATED_PK_DB_ENTITY} item as l_item and then l_item.has_error then
					logger.write_error ("execute error during execute of DB_Service, could not set item from db service")
					set_last_error_from_fallible (l_item)
				end
				last_row_pk := current_row_pk
			else
				create l_new_item.make_from_db_service (Current)
				if l_new_item.has_error then
					logger.write_error ("execute error during execute of DB_Service, could not create item from db service")
					set_last_error_from_fallible (l_new_item)
				else
					items.extend (l_new_item)
					if items.count \\ 1000 = 0 then
						logger.write_information ("Adding new item to collection (" + items.count.out + "):" + l_new_item.out)
					end

--					logger.write_debug ("Adding new item to collection:" + l_new_item.out)
				end
			end
		end


	create_secondary_association_if_any
		require
			attached_item: attached item as l_i
			valid_item_for_insert_or_update: l_i.valid_for_insert or l_i.valid_for_update
		local
			l_msg: STRING
			l_updt: STRING_SQL_UPDATE
			l_void: like item
		do
			if attached {RELATED_DB_ENTITY} item as l_item and then
				(l_item.valid_for_insert or l_item.valid_for_update)
			then
				across
					l_item.relationships as l_rel
				until
					has_error
				loop
					if attached {MANY_TO_ONE_JOIN[RELATED_PK_DB_ENTITY, DB_ENTITY]} l_rel.item as l_rel_mtoj then
						across
							l_rel_mtoj.primaries as l_prim
						loop
							if attached {PK_DB_ENTITY} l_prim.item as l_prim_pkdbe then
								if attached {PK_DB_ENTITY} l_rel_mtoj.secondary as l_sec then
									check
										last_inserted_id > 0
									end
									create l_updt.make_from_string (item_prototype, "INSERT INTO " + l_rel_mtoj.join_table_name +
										"(" + l_rel_mtoj.secondary_foreign_key + ", " + l_rel_mtoj.primary_foreign_key + ")" +
										" VALUES (" + l_sec.primary_key.out + ", " + last_inserted_id.out + ")")
									execute_update (l_updt)
									if has_error then
										l_msg := "last DB modification did not work with entity:" + l_item.out
										if attached db_connection.last_error as l_err then
											l_msg.append ("-error:" + l_err.out)
										end
										set_last_error( create {INSERT_ERROR}.make (l_msg) )
									else
										set_last_success_message ("Entity created into DB successfully")
									end
--								else -- just dont add it
								end
							else
								check
									either_prim_or_sec_not_a_pk_db_entity: False
								end
							end
						end
					elseif attached {ONE_TO_MANY_JOIN[RELATED_PK_DB_ENTITY, PK_DB_ENTITY]} l_rel.item as l_rel_otmj then
						if attached l_rel_otmj.secondary_prototype.generating_type as l_sec_type and then
							attached {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type (l_sec_type) as l_secondary_db_service
						then
							across
								l_rel_otmj.secondaries as l_sec
							loop
								if attached {PK_DB_ENTITY} l_sec.item as l_sec_pkdbe and then
									attached {PK_DB_ENTITY} l_item as l_prim
								then
									-- create secondary and sets result
									l_secondary_db_service.set_item (l_sec_pkdbe)
									l_secondary_db_service.create_or_update_item_without_rel
									last_inserted_id := l_secondary_db_service.last_inserted_id
									if attached l_secondary_db_service.item as l_sec_item then
										l_sec_item.deep_copy (l_sec.item)
									end
									-- create association on join table
									check
										last_inserted_id > 0
									end
									create l_updt.make_from_string (item_prototype, "INSERT INTO " + l_rel_otmj.join_table_name +
										"(" + l_rel_otmj.secondary_foreign_key + ", " + l_rel_otmj.primary_foreign_key + ")" +
										" VALUES (" + last_inserted_id.out + ", " + l_prim.primary_key.out + ")")
									execute_update (l_updt)
									if has_error then
										l_msg := "last DB modification did not work with entity:" + l_item.out
										if attached db_connection.last_error as l_err then
											l_msg.append ("-error:" + l_err.out)
										end
										set_last_error( create {INSERT_ERROR}.make (l_msg) )
									else
										set_last_success_message ("Entity created into DB successfully")
									end
								else
									check
										either_prim_or_sec_not_a_pk_db_entity: False
									end
								end
							end
						else
							check
								secondary_db_service_not_found: False
							end
						end
					elseif attached {ONE_TO_MANY[RELATED_PK_DB_ENTITY, PK_DB_ENTITY]} l_rel.item as l_rel_otm then
						if attached {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type (l_rel_otm.secondary_prototype.generating_type) as l_secondary_db_service then
							l_secondary_db_service.set_insert_if_not_exists (insert_if_not_exists)
							across
								l_rel_otm.secondaries as l_sec
							loop
								l_secondary_db_service.set_item (l_sec.item)
								l_secondary_db_service.create_or_update_item_without_rel
							end
							l_secondary_db_service.set_insert_if_not_exists (False)
						else
							check
								secondary_service_not_found: False
							end
						end
					end
				end
			end
			if has_error then
				item := l_void
			end
		end

	add_relationships_for_insert_update (some_keys, some_values, some_unstored_field_names: LINKED_LIST[STRING]; an_is_update: BOOLEAN)
			-- working around item's relationships
		local
			l_relationship: like {RELATED_DB_ENTITY}.relationships.item
			l_key, l_val, l_ident: detachable STRING
		do
			if attached {RELATED_DB_ENTITY} item as l_item then
				across
					l_item.relationships as l_relationship_c
				loop
					l_key := Void
					l_val := Void
					l_relationship := l_relationship_c.item
					-- identifiable
					if attached {IDENTIFIABLE} l_relationship as l_rel then
						l_ident := l_rel.foreign_key
					else
						l_ident := Void
					end
					-- relationship types
					if attached {MANY_TO_ONE_JOIN[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship then
						do_nothing
					elseif attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship as l_m_t_o then
--						if not an_is_update then -- on update don't have to set required associations (yes on companies->billing_city
-- on update dont add a relationship which is not set
							if
								attached {PK_DB_ENTITY} l_m_t_o.secondary as l_secondary and then
								l_secondary.primary_key > 0 -- adding only if relationship has been setted and id > 0
							then
								if attached l_ident as l_ident_a then
									l_key := l_ident_a
								else
									l_key := l_m_t_o.secondary_foreign_key
								end
								l_val := l_secondary.primary_key.out
							end
--						end
					elseif attached {ONE_TO_MANY[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship as l_o_t_m then
						do_nothing -- As per shema sit-docs/sit_solar_offer/20190110_154523_relationships_schema_with_insert_update_strategy.jpg update only current Item on many secondaries
					elseif attached {ONE_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_relationship as l_o_t_o then
						check
							not_empty_primary: l_o_t_o.primary /= Void -- primaries loaded
						end
						if
							attached {PK_DB_ENTITY} l_o_t_o.secondary as l_secondary and then
							l_secondary.primary_key > 0 -- adding only if relationship has been setted and id > 0
						then
							if attached l_ident as l_ident_a then
								l_key := l_ident_a
							else
								l_key := l_o_t_o.secondary_foreign_key
							end
							l_val := l_secondary.primary_key.out
						end
					else
						check
							relationship_not_treated: false
						end
					end
					if
						attached l_key as lk and then
						attached l_val as lv and then
						not some_unstored_field_names.has (lk) and then
						not some_keys.has (l_key)
					then
						if attached l_ident then
							some_keys.extend (l_ident)
						else
							some_keys.extend (lk)
						end
						some_values.extend (lv)
					end
				end
			end
		ensure
			same_some_keys_some_values_count: some_keys.count = some_values.count
		end

	set_query_from_relationships (a_qry: COMPOSED_SQL_QUERY; some_relationships: LINKED_LIST[RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]])
		local
			l_join_alias, l_join_column: STRING
			l_rest_rel: RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]
			l_p: like {RELATIONSHIP[RELATED_DB_ENTITY, DB_ENTITY]}.primary_prototype
			l_primary_table_added: BOOLEAN
		do
			across
				some_relationships as l_rel_c
			loop
				l_rest_rel := l_rel_c.item
				l_p := l_rest_rel.primary_prototype
				if attached {PK_DB_ENTITY} l_rest_rel.secondary_prototype as l_s then
					if not l_primary_table_added then
						a_qry.select_clauses.extend (l_p.table_name + ".*")
						l_primary_table_added := True
					end
					if attached {IDENTIFIABLE} l_rest_rel as l_identifiable_rel then
						if attached {IDENTIFIABLE_ONE_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rest_rel as l_rest_rel_l then
							l_join_alias := l_rest_rel_l.secondary_join_alias + "_" + l_identifiable_rel.identifier -- more than one joins...
							l_join_column := l_identifiable_rel.foreign_key
							a_qry.join_clauses.extend (create {INNER_JOIN}.make_from_fields (l_s.table_name, l_join_alias, l_p.table_name, l_join_column, l_join_alias, l_s.Primary_key_db_column_name))
							a_qry.add_api_exportable_fields_to_select (l_join_alias, l_identifiable_rel.identifier, l_rest_rel)
						elseif attached {IDENTIFIABLE_MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rest_rel as l_rest_rel_l then
							l_join_alias := l_rest_rel_l.secondary_join_alias + "_" + l_identifiable_rel.identifier -- more than one joins...
							l_join_column := l_identifiable_rel.foreign_key -- + "_id" -- Not applicable to items_from_date_time_interval_request
							a_qry.join_clauses.extend (create {LEFT_JOIN}.make_from_fields (l_s.table_name, l_join_alias, l_p.table_name, l_join_column, l_join_alias, l_s.Primary_key_db_column_name))
							a_qry.add_api_exportable_fields_to_select (l_join_alias, l_identifiable_rel.identifier, l_rest_rel)
						else
							check
								untreated_relationship_type: False
							end
						end
					elseif attached {ONE_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rest_rel as l_rest_rel_l then
						l_join_alias := l_rest_rel_l.secondary_join_alias
						l_join_column := l_rest_rel_l.secondary_foreign_key
						a_qry.join_clauses.extend (create {INNER_JOIN}.make_from_fields (l_s.table_name, l_join_alias, l_p.table_name, l_join_column, l_join_alias, l_s.Primary_key_db_column_name))
						a_qry.add_api_exportable_fields_to_select (l_join_alias, l_rest_rel_l.secondary_join_alias, l_rest_rel_l)
					elseif attached {ONE_TO_MANY_JOIN[RELATED_PK_DB_ENTITY, PK_DB_ENTITY]} l_rest_rel as l_rest_rel_l then
						-- innerJoin first table to join table
						a_qry.join_clauses.extend (create {INNER_JOIN}.make_from_fields (l_rest_rel_l.join_table_name, Void,  l_p.table_name, l_rest_rel_l.primary_prototype.Primary_key_db_column_name, l_rest_rel_l.join_table_name, l_rest_rel_l.primary_foreign_key))
						-- innerJoin join table to second table
						l_join_alias := l_rest_rel_l.secondary_join_alias
						a_qry.join_clauses.extend (create {INNER_JOIN}.make_from_fields (l_s.table_name, l_join_alias, l_rest_rel_l.join_table_name, l_rest_rel_l.secondary_foreign_key, l_join_alias, l_rest_rel_l.secondary_prototype.Primary_key_db_column_name))
						a_qry.add_api_exportable_fields_to_select (l_join_alias, l_join_alias, l_rest_rel_l)
					elseif attached {MANY_TO_ONE[RELATED_DB_ENTITY, DB_ENTITY]} l_rest_rel as l_rest_rel_l then
						l_join_alias := l_rest_rel_l.secondary_join_alias
						l_join_column := l_rest_rel_l.secondary_foreign_key
						a_qry.join_clauses.extend (create {INNER_JOIN}.make_from_fields (l_s.table_name, l_join_alias, l_p.table_name, l_join_column, l_join_alias, l_s.Primary_key_db_column_name))
						a_qry.add_api_exportable_fields_to_select (l_join_alias, l_rest_rel_l.secondary_join_alias, l_rest_rel_l)
					else
						check
							relationship_not_identifiable: False -- check weather when you added the relationship it is a descendent of IDENTIFIABLE
						end
					end
				end --if attached as PK_DB_ENTITY
			end -- accross	
		end

invariant
	valid_item: attached item implies last_error = Void
	valid_items: items.count > 0 implies last_error = Void
	valid_last_error_items_item: attached last_error implies item = Void and items.count = 0
	uniq_column_names_in_resultset: uniq_column_names_in_resultset

end -- class
