note
	description: "Summary description for {DB_SERVICE_BASE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	DB_SERVICE_OPERATIONS

inherit
	FALLIBLE


feature {NONE} -- Initialization

	make (a_db_connection: DB_CONNECTION)
		require
			valid_db_connection: a_db_connection.is_connected
		do
			last_query := ""
			db_connection := a_db_connection
			create base_selection.make
			create base_update.make
			default_create
			-- 'base_selection' provides a SELECT query mechanism.
			-- 'base_update' provides updating facilities.

			base_selection.set_trace
			base_update.set_trace
		ensure
			db_connection_setted: db_connection = a_db_connection and db_connection.is_connected
		end

feature -- Access

	last_query: STRING

feature {NONE} -- Database operations access

	db_connection: DB_CONNECTION

	base_selection: DB_SELECTION

	base_update: DB_CHANGE

feature -- Status Report

	valid_for_modification: BOOLEAN
		do
			Result := db_connection.is_connected
		end

	valid_for_select: BOOLEAN
		do
			Result := db_connection.is_connected
		end

	valid_resultset: BOOLEAN
		do
			Result := attached base_selection.cursor
		end

	valid_base_selection: BOOLEAN
		do
			Result := base_selection.is_connected
		end

	cursor: attached like base_selection.cursor
		require
			valid_resultset
		do
			create Result.make
			if attached base_selection.cursor as l_curs then
				Result := l_curs
			else
				Check
					invalid_cursor_call: False
				end
			end
		end

feature -- Status Setting

	set_base_selection_action (an_action: ACTION)
		require
			valid_base_selection
		do
			base_selection.set_action (an_action)
		end

feature -- Operations

	execute_query (a_query: SQL_QUERY)
			-- calling start and execute
		do
			last_query := a_query.to_sql
			base_selection.query (last_query)
			logger.write_notice ("execute_query------->-" + last_query + "-")
			if base_selection.is_ok then
				base_selection.load_result -- calling start and execute
			else
				create {ODBC_ERROR} last_error.make ("Error while retreiving item from DB query not ok:" + last_query)
				check
					error_not_happening: False
				end
			end
			base_selection.terminate
		ensure
			last_query.is_equal (a_query.to_sql)
		end

	execute_update (an_sql_modify: SQL_MODIFY)
		local
			l_msg: STRING
		do
			last_query := an_sql_modify.to_sql
			if base_update.is_ok then
				base_update.modify (last_query)
				if base_update.is_ok then
					l_msg := "Updated successfully"
					if base_update.is_affected_row_count_supported then
						l_msg.append (" modified:" + base_update.affected_row_count.out)
					else
						logger.write_warning ("execute_update: is_affected_row_count_supported is False")
					end
					set_last_success_message (l_msg)
					logger.write_information ("execute_update->Success:" + l_msg + " qry:" + last_query)
				else
					set_last_error (create {ODBC_ERROR}.make("Error updating qry:" + last_query + "-code:" + base_update.error_code.out + "-msg:" + base_update.error_message_32))
					check
						base_update_is_ok_not_ok: False
					end
				end
				if not base_update.warning_message_32.is_empty then
					logger.write_warning ("execute_update:warning_message_32:" + base_update.warning_message_32)
				end
			else
				set_last_error (create {ODBC_ERROR}.make("Error updating qry:" + last_query + "-code:" + base_update.error_code.out + "-msg:" + base_update.error_message_32))
				check
					error_not_happening: False
				end
			end
		ensure
			last_query.is_equal (an_sql_modify.to_sql)
		end


end
