note
	description: "Summary description for {ENERGY_UNIT_DB_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	ENERGY_UNIT_DB_SERVICE

inherit
	MEASURE_UNIT_DB_SERVICE[ENERGY_UNIT]
		redefine
			set_callable_features
		end

create
	make

feature {NONE} -- Initialization


	set_callable_features
			--redefine me if needed to extend available features of DB Service[DB_ENTITY]
		do
			precursor
		end

feature -- Access

	Rest_feature_items_monthly_energy_for_chart: STRING = "items_monthly_energy_for_chart"

feature -- Rest Callable features

	items_daily_summarized (a_parent_entity: RELATED_PK_DB_ENTITY; a_date_time: like {MEASURE_UNIT}.date_time; some_date_time_offsets: like {JSON_UTIL}.timezone_offsets_array_to_list): like items
			-- Returns a list of computed average items with timestamps of given zone on arg3 of request for a given day.
			--
			-- modifies items, items_index
		local
			l_current_day_for_average: DATE
			l_average_item: like item
			l_start_dt, l_end_dt: like item.date_time
			l_items_for_average: LINKED_LIST[CHARGE_UNIT]
			l_constraint: FIELD_CONSTRAINT
		do
			create Result.make
			-- Start end date_times
			create l_start_dt.make_by_date_time (a_date_time.date.deep_twin, create {TIME}.make (0, 0, 1))
			l_start_dt.date.set_day (1)
			l_start_dt.minute_add ({SIT_UTIL}.timezone_offset_for_index (some_date_time_offsets, l_start_dt))
			create l_end_dt.make_by_date_time (a_date_time.date.deep_twin, create {TIME}.make (0, 0, 0))
			l_end_dt.day_add (2) --one day before, but 0:0:0 is day before :-(
			l_end_dt.minute_add ({SIT_UTIL}.timezone_offset_for_index (some_date_time_offsets, l_end_dt))
			l_end_dt.date.set_day (l_end_dt.days_in_i_th_month (l_end_dt.date.month, l_end_dt.date.year))

			if attached {ENUMERATE_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable ENUMERATE}) as l_enum_dbs and then
				attached {CHARGE_UNIT_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable CHARGE_UNIT}) as l_chu_dbs
			then
				-- load charge_units
				logger.write_information ("items_monthly_energy_for_chart-> loading items for month " + a_date_time.date.month.out + " now ")
				l_chu_dbs.specific_fields_to_load.extend (l_chu_dbs.item_prototype.date_time_db_column_name)
				l_chu_dbs.specific_fields_to_load.extend (l_chu_dbs.item_prototype.value_db_column_name)
				l_chu_dbs.set_execute_loads_specific_fields (True)
				create l_constraint.make (
						l_chu_dbs.item_prototype.table_name,
						l_chu_dbs.item_prototype.An_index_fk,
						{CONSTRAINT}.equal_operator,
						l_enum_dbs.enumerate_cash_from_name ({DB_CONSTANTS}.Enumerate_neutral).primary_key)
				if attached l_chu_dbs.items_from_date_time_interval (
						l_start_dt,
						l_end_dt,
						a_parent_entity,
						l_constraint,
						False)
					as l_items
				then
					logger.write_information ("items_monthly_energy_for_chart-> items found for charge_units count:" + l_items.count.out)
					-- compute averages
					if not l_items.is_empty then
						l_current_day_for_average := l_items.first.date_time.date.deep_twin -- for void safety only will be set after setting timezone
						create l_items_for_average.make
						logger.write_information ("items_monthly_energy_for_chart-> computing average now")
						across
							l_items as l_item
						loop
							-- remove timezone
							l_item.item.set_time_zone_offset (- {SIT_UTIL}.timezone_offset_for_index (some_date_time_offsets, l_item.item.date_time))
							if l_item.is_first then
								l_current_day_for_average := l_item.item.date_time.date.deep_twin
							end
							if attached l_item.item.date_time as l_cur_item_dt then
								-- fills l_items_for_average with current day
								if l_current_day_for_average.day = l_item.item.date_time.day then
									l_items_for_average.extend (l_item.item)
									check
										{K_WATT_H}.items_same_day (l_items_for_average)
									end
								else
									if not l_items_for_average.is_empty then
										create l_average_item
										l_average_item.set_value (create {K_WATT_H}.make_from_charge_units_average (l_items_for_average))
										l_average_item.set_date_time (create {DATE_TIME}.make_by_date_time (l_current_day_for_average.deep_twin, create {TIME}.make (0, 0, 1)))
										Result.extend (l_average_item)
										l_current_day_for_average.day_add (1)
										l_items_for_average.wipe_out
									end
								end
							else
								logger.write_error ("items_monthly_energy_for_chart-> item without date_time!:" + l_item.item.out)
								check
									item_without_date_time: False
								end
							end
						end
						-- Adding last day
						if not l_items_for_average.is_empty then
							create l_average_item
							l_average_item.set_value (create {K_WATT_H}.make_from_charge_units_average (l_items_for_average))
							l_average_item.set_date_time (create {DATE_TIME}.make_by_date_time (l_current_day_for_average.deep_twin, create {TIME}.make (0, 0, 1)))
							Result.extend (l_average_item)
							l_current_day_for_average.day_add (1)
							l_items_for_average.wipe_out
						end
						logger.write_information ("items_monthly_energy_for_chart Result count:" + Result.count.out)
					end
				end
			end
		end


feature {NONE} -- Implementation


	average_value (some_items: like items): like item.value
			-- returns a null charge if any and otherwise an average of given l_items
		local
			l_val_result: like item.value.item
			l_result: like item.value
		do
			create Result
			create l_val_result
			across
				some_items as l_item
			loop
				l_val_result := l_val_result + l_item.item.value.item
			end
			l_result := l_val_result / some_items.count
			if some_items.count > 0 then
				Result.set_item (l_result)
			end
			logger.write_debug ("average_charge: " + {SIT_UTIL}.iterable_to_string(some_items) + " result:" + Result.out)
		end

end
