note
	description: "Summary description for {ENUMERATE_DB_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	ENUMERATE_DB_SERVICE

inherit
	DB_SERVICE[ENUMERATE]
		redefine
			make,
			set_callable_features,
			select_all_query
		end

create
	make

feature {NONE} -- Creation

	make (a_db_connection: attached DB_CONNECTION)
		do
			Precursor (a_db_connection)
		end

feature -- Access

	enumerate_cash_from_name (a_name: STRING): ENUMERATE
		local
			l_enum: ENUMERATE
			l_qry: COMPOSED_SQL_QUERY
		do
			across
				enumerates_cash as l_item
			loop
				if l_item.item.name.is_equal(a_name) then
					l_enum := l_item.item
				end
			end
			if not attached l_enum as l_enum_a then
				create l_qry.make (item_prototype)
				l_qry.add_constraint (create {FIELD_CONSTRAINT}.make (Void, {ENUMERATE}.Name_db_column_name, {CONSTRAINT}.Like_operator, a_name))
				load_entities (l_qry)
				if items.count > 0 then
					Result := items.first
				else
					create Result
					check
						item_not_found: False
					end
				end

				enumerates_cash.extend (Result)
			else
				Result := l_enum_a
			end
		end

feature -- Rest Callable settings

	Rest_feature_items_from_category_name: STRING = "items_from_category_name"
	Rest_feature_items_root_categories: STRING = "items_root_categories"
	Rest_feature_items_branches_only: STRING = "items_branches_only"
	Rest_feature_items_leaves_only: STRING = "items_leaves_only"
	Rest_feature_items_sub_categories_tree_from_id: STRING = "items_sub_categories_tree_from_id"
	Rest_feature_items_sub_items_from_id: STRING = "items_sub_items_from_id"

	set_callable_features
			--redefine me if needed to extend available features of DB Service[DB_ENTITY]
		do
			precursor
			callable_features.put (agent items_from_category_name (?), Rest_feature_items_from_category_name)
			callable_features.put (agent items_root_categories, Rest_feature_items_root_categories)
			callable_features.put (agent items_branches_only, Rest_feature_items_branches_only)
			callable_features.put (agent items_leaves_only, Rest_feature_items_leaves_only)
			callable_features.put (agent items_sub_categories_tree_from_id, Rest_feature_items_sub_categories_tree_from_id)
			callable_features.put (agent items_sub_items_from_id, Rest_feature_items_sub_items_from_id)
		end

feature -- Rest Callable features

	items_sub_categories_tree_from_id (a_rest_request: REST_REQUEST): detachable like items
			-- returns sub items from given ID
			-- data: { "action": { "feature": "items_sub_categories_tree_from_id", "arg1": "1" } }
		note
			query_sample: "[
				WITH RECURSIVE enumerates_leaves AS (
				 SELECT
				        *
				    FROM
				        enumerate
				    WHERE
				        id = 10
				 UNION
				    SELECT
				        e.*
				    FROM
				        enumerate e
				 INNER JOIN 
				    enumerates_leaves s ON s.id = e.enumerate_id
				) SELECT * FROM enumerates_leaves WHERE enumerates_leaves.enumerate_id IS NOT NULL;
				]"
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.name.is_equal (Rest_feature_items_sub_categories_tree_from_id) and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 1
			second_argument_is_integer: attached a_rest_request.request_feature as l_req_arg2 and then
				attached l_req_arg2.arguments.first as l_arg2 and then
				l_arg2.is_integer and then
				l_arg2.to_integer > 0
		local
			l_category_id: STRING
			l_qry: STRING_SQL_QUERY
			l_top_categories_items: like items
		do
			if attached a_rest_request.request_feature as l_req_f then
				l_category_id := l_req_f.arguments.first
				create l_qry.make_from_string (item_prototype, "WITH RECURSIVE enumerates_leaves AS (SELECT%
				    %    * %
				    %FROM %
				    %    enumerate %
				    %WHERE %
				    %    " + {like item_prototype}.Primary_key_db_column_name + " = " + l_category_id + " %
					%UNION %
					%   SELECT %
					%       e.* %
					%   FROM %
					%       enumerate e %
					%INNER JOIN  %
					%   enumerates_leaves s ON s." + {like item_prototype}.Primary_key_db_column_name + " = e." + {like item_prototype}.Category_db_column_name + " %
					%) SELECT * FROM enumerates_leaves WHERE enumerates_leaves." + {like item_prototype}.Category_db_column_name + " IS NOT NULL;")

				load_entities (l_qry)
				-- setting categories with relationships
				create l_top_categories_items.make
				-- should be with agents but too complex for a first example: https://stackoverflow.com/questions/54380282/eiffel-is-there-a-way-with-do-all-or-do-if-to-search-for-an-element-in-a-collec/54593853#54593853
				-- http://openproject/openproject/projects/solar-offers/work_packages/456/activity
--				items.do_all (agent (l_item: like item)
--					do
--						if not items.after and then
--							attached l_item as l_i and then
--							l_i.primary_key.to_integer_64.is_equal (l_i.category_id)
--						then
--							l_i.item.set_category (items.item)
--							l_top_categories_items.extend (items.item)
--						else
--							l_item.item.set_category (Void)
--						end
--					end)

				-- with loop
				across
					items as l_item
				loop
					search_item_with_id (l_item.item.category_id.to_integer)
					if not items.after then
						l_item.item.set_category (items.item)
						l_top_categories_items.extend (items.item)
					else
						l_item.item.set_category (Void)
					end
				end
				items.remove_all_from (l_top_categories_items)
				Result := items
			end
		end

	items_sub_items_from_id (a_rest_request: REST_REQUEST): detachable like items
			-- returns sub categories from given ID
			-- data: { "action": { "feature": "items_sub_items_from_id", "arg1": "1" } }
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.name.is_equal (Rest_feature_items_sub_items_from_id) and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 1
			second_argument_is_integer: attached a_rest_request.request_feature as l_req_arg2 and then
				attached l_req_arg2.arguments.first as l_arg2 and then
				l_arg2.is_integer and then
				l_arg2.to_integer > 0
			not items.object_comparison
		local
			l_category_id: STRING
			l_qry: STRING_SQL_QUERY
		do
			if attached a_rest_request.request_feature as l_req_f then
				l_category_id := l_req_f.arguments.first
				create l_qry.make_from_string (item_prototype,"SELECT * FROM enumerate WHERE enumerate." + {like item_prototype}.Category_db_column_name + "=" + l_category_id + ";")
				load_entities (l_qry)
				Result := items
			end
		end

	items_from_category_name (a_rest_request: REST_REQUEST): detachable like items
			-- data: { "action": { "feature": "items_from_category_name", "arg1": "city" } }
			--
			-- argument 1 is category name (enumerate.name)
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.name.is_equal (Rest_feature_items_from_category_name) and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 1
			third_argument_is_integer: attached a_rest_request.request_feature as l_req_arg2 and then
				attached l_req_arg2.arguments.at(2) as l_arg2 and then
				not l_arg2.is_empty
		local
			l_category_name: STRING
		do
			if attached a_rest_request.request_feature as l_req_f then
				l_category_name := l_req_f.arguments.first
				Result := items_from_category_name_impl (l_category_name)
			end
		end

	items_root_categories (a_rest_request: REST_REQUEST): detachable like items
			-- returns root categories only
			-- data: { "action": { "feature": "items_root_categories" } }
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				not l_req_f.has_arguments and then
				l_req_f.name.is_equal (Rest_feature_items_root_categories)
		local
			l_qry: STRING_SQL_QUERY
		do
			create l_qry.make_from_string (item_prototype, "SELECT * FROM enumerate WHERE enumerate." + {like item_prototype}.Category_db_column_name + " IS NULL")
			load_entities (l_qry)
			Result := items
		end

	items_branches_only (a_rest_request: REST_REQUEST): detachable like items
			-- Returns only the categories
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				not l_req_f.has_arguments and then
				l_req_f.name.is_equal (Rest_feature_items_branches_only)
		local
			l_qry: STRING_SQL_QUERY
		do
		    create l_qry.make_from_string (item_prototype,"SELECT * FROM enumerate p WHERE EXISTS ( %
			        % SELECT 1 FROM enumerate c %
			        % WHERE c." + {like item_prototype}.Category_db_column_name + " = p." + {like item_prototype}.Primary_key_db_column_name + " %
			    %) UNION SELECT * FROM enumerate p WHERE p.enumerate_id IS NULL")
			load_entities (l_qry)
			Result := items
		end

	items_leaves_only (a_rest_request: REST_REQUEST): detachable like items
			-- Returns only the enumerates which are leaves
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				not l_req_f.has_arguments and then
				l_req_f.name.is_equal (Rest_feature_items_leaves_only)
		local
			l_qry: STRING_SQL_QUERY
		do
		    create l_qry.make_from_string (item_prototype, "SELECT * FROM enumerate p WHERE NOT EXISTS ( %
			        % SELECT 1 FROM enumerate c %
			        % WHERE c." + {like item_prototype}.Category_db_column_name + " = p." + {like item_prototype}.Primary_key_db_column_name + " %
			    %)")
			load_entities (l_qry)
			Result := items
		end

feature {NONE} -- Implementation

	items_from_category_name_impl (a_category_name: STRING): like items_from_category_name
		require
			not_empty_cat_name: not a_category_name.is_empty
		local
			l_qry: STRING_SQL_QUERY
		do
			create l_qry.make_from_string (item_prototype, "SELECT enum.id, enum.name, cat.id AS category_id, cat.name AS category_name FROM enumerate enum" +
				" INNER JOIN enumerate cat ON enum." + {like item_prototype}.Category_db_column_name + " = cat.id" +
				" WHERE cat.name like '" + a_category_name + "'")
			load_entities (l_qry)
			Result := items
		end

	search_item_with_id (an_id: INTEGER)
			-- Moves items cursor if not found is_after
		require
			valid_id: an_id > 0
		local
			l_found: BOOLEAN
		do
			from
				items.start
			until
				items.after or l_found
			loop
				l_found := items.item.primary_key = an_id
				if not l_found then
					items.forth
				end
			end
		ensure
			cursor_on_element_if_found: not items.after implies items.item.primary_key = an_id
		end

feature {NONE} -- SQL

	select_all_query: STRING
		do
			Result := "SELECT enum.id, enum.name, cat.id AS category_id, cat.name AS category_name FROM enumerate enum" +
				" LEFT JOIN enumerate cat ON enum." + {like item_prototype}.Category_db_column_name + " = cat.id" +
				" ORDER BY enum." + item_prototype.Primary_key_db_column_name + " ASC"
			logger.write_debug ("select_all_query from ENUMERATE_DB_SERVICE:'" + Result + "'")
		end

feature {NONE} -- Cash

	enumerates_cash: LINKED_LIST[ENUMERATE]
		once
			create Result.make
		end

end
