note
	description: "Summary description for {CHARTABLE_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	MEASURE_UNIT_DB_SERVICE[G -> MEASURE_UNIT create default_create, make_from_db_service, make_from_json, make_from_date_time end]

inherit
	DB_SERVICE[G]
		redefine
			make,
			execute
		end

feature {NONE} -- Initialization

	make (a_db_connection: attached DB_CONNECTION)
		do
			create specific_fields_to_load.make
			Precursor (a_db_connection)
		end


feature -- Status report

	items_from_date_time_interval (a_start_date_time, an_end_date_time: DATE_TIME; a_related_pk_dbe: RELATED_PK_DB_ENTITY; a_constraint: detachable CONSTRAINT; a_with_relationships: BOOLEAN ): like items
	-- TODO: MIDDLE multiple types conformance with a_relationship_entity put me again when solved https://stackoverflow.com/questions/56799559/eiffel-a-way-to-check-type-conformance-with-a-given-class-name
			-- Returns all items from given interval refering to given a_relationship_entity
			-- Will be loaded with an_index relationship
			--
			-- modifies items
			-- a_constraint: any constraint to add to query
		require
--			a_relationship_entity_is_related_db_entity: a_relationship_entity.conforms_to_multiple ([{RELATED_DB_ENTITY}])
			a_start_date_time.is_less_equal (an_end_date_time)
			a_related_pk_dbe.primary_key > 0
		local
			l_dbr: DB_REQUEST
		do
			create l_dbr
			-- CONSTRAINTS
			l_dbr.constraints.extend (create {DATE_CONSTRAINT}.make_from_interval (item_prototype.table_name, a_start_date_time, an_end_date_time, item_prototype.Date_time_db_column_name))

			if attached a_constraint as l_c then
				-- an_index
				l_dbr.constraints.extend (l_c)
			end
			-- measuring_point
--			l_dbr.add_relationship_loadable_from_secondary_type (a_related_pk_dbe, a_related_pk_dbe.generating_type)
			l_dbr.add_relationship_from_secondary_type (item_prototype, a_related_pk_dbe.generating_type)
			l_dbr.add_relationship_loadable_from_resultset (item_prototype.an_index_relationship)

			l_dbr.constraints.extend (a_related_pk_dbe.primary_key_constraint_join)

			l_dbr.order_clauses.extend (create {ORDER_CLAUSE}.make (item_prototype.table_name, "timestamp", {ORDER_CLAUSE}.Way_ascending))
			if a_with_relationships and l_dbr.has_relationships then
				load_with_relationships (l_dbr)
			else
				load_without_relationships (l_dbr)
				-- TODO: HIGH replace with items_optimized
			end
			across
				items is l_item
			loop
				l_item.set_to_json_relationships_level_max (2)
			end

			Result := items
		end

	items_from_date_time_interval_average (a_start_date_time, an_end_date_time: DATE_TIME; a_minutes_increment: INTEGER; a_time_zone_offset: INTEGER; a_relationship_entity: PK_DB_ENTITY; some_constraints: detachable like {DB_REQUEST}.constraints; a_with_relationships: BOOLEAN ): like items
			-- I/Os are offsetted
			--
			-- a_with_relationships true to have some enumerates embeded
		require
			a_start_date_time.is_less_equal (an_end_date_time)
			a_relationship_entity.primary_key > 0
		local
			l_date_times_of_day: like date_times_from_interval
			l_date_times_of_day_index, l_time_zone: INTEGER
			l_found_item: like item
			l_start_dt, l_end_dt, l_qry_start_dt, l_qry_end_dt: like {ENERGY_UNIT}.date_time
			l_constraints: like some_constraints
			l_dt_c: like {DB_REQUEST}.constraints.item
			l_items: like items
			l_to_retreive_fields_list: like specific_fields_to_load
		do
			items_index := 1 -- average_item_from_interval(...) is in charge of incrementing this index					

			l_date_times_of_day := date_times_from_interval (a_start_date_time, an_end_date_time, a_minutes_increment)
			l_qry_start_dt := l_date_times_of_day.first.date_time.deep_twin
			l_qry_start_dt.minute_add (-l_time_zone)
			l_qry_end_dt := l_date_times_of_day.last.date_time.deep_twin
			l_qry_end_dt.minute_add (-l_time_zone)

			-- Qry
			create l_to_retreive_fields_list.make
			l_to_retreive_fields_list.extend (item_prototype.Value_db_column_name)
			l_to_retreive_fields_list.extend (item_prototype.Date_time_db_column_name)
			  -- Constraints
			if attached some_constraints then
				l_constraints := some_constraints
			else
				create l_constraints.make
			end
			create {DATE_CONSTRAINT} l_dt_c.make_from_interval (item_prototype.table_name, l_qry_start_dt, l_qry_end_dt, item_prototype.date_time_db_column_name)

			l_constraints.extend (a_relationship_entity.primary_key_constraint)
			l_constraints.extend (l_dt_c)
			l_items := items_optimized (l_constraints, l_to_retreive_fields_list)
			l_constraints.prune (l_dt_c)

			set_items_time_zone_offsets (items, l_time_zone)
			from -- loop into items_from_date_time_interval and sets l_date_times_of_day with averages of l_minutes_increment
				l_date_times_of_day_index := 2
			until
				l_date_times_of_day_index > l_date_times_of_day.count
			loop
				-- computing average into f_found_item
				l_start_dt := l_date_times_of_day.at (l_date_times_of_day_index - 1).date_time
				l_end_dt := l_date_times_of_day.at (l_date_times_of_day_index).date_time
				l_found_item := average_item_from_interval (l_items, l_start_dt, l_end_dt)
				l_date_times_of_day.at (l_date_times_of_day_index).set_but_date_time (l_found_item) -- setting data from found item (in the past) to current item
--						l_next_charge_unit := l_date_times_of_day.at (l_date_times_of_day_index + 1)
--						l_next_charge_unit.set_but_date_time (l_found_item)

				l_date_times_of_day_index := l_date_times_of_day_index + 1
			end
			Result := l_date_times_of_day
		end

	start_date_time_constraint (a_start_date_time: like item_prototype.date_time): CONSTRAINT
			-- Greater than a_start_date_time
		do
			create {FIELD_CONSTRAINT} Result.make (
					item_prototype.table_name,
					item_prototype.date_time_db_column_name,
					{CONSTRAINT}.Greater_than_operator,
					a_start_date_time
			)
		end

	end_date_time_constraint (an_end_date_time: like item_prototype.date_time): CONSTRAINT
			-- Smaller or equal than an_end_date_time
		do
			create {FIELD_CONSTRAINT} Result.make (
					item_prototype.table_name,
					item_prototype.date_time_db_column_name,
					{CONSTRAINT}.Smaller_or_equal_operator,
					an_end_date_time
			)
		end

	an_index_constraint (an_enumerate_name: like {DB_CONSTANTS}.enumerate_neutral): CONSTRAINT
		do
			if attached {ENUMERATE_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable ENUMERATE}) as l_enum_dbs then
				create {FIELD_CONSTRAINT} Result.make (
						item_prototype.table_name,
						item_prototype.An_index_fk,
						{CONSTRAINT}.equal_operator,
						l_enum_dbs.enumerate_cash_from_name (an_enumerate_name).primary_key
				)
			else
				set_last_error (create {SIT_INTERNAL_ERROR}.make ("an_index_constraint-> Enumerates db service not found"))
				create {VOID_SAFETY_CONSTRAINT} Result
				check
					enumerate_db_service_not_found: False
				end
			end
		end

	items_optimized ( some_constraints: like {COMPOSED_SQL_QUERY}.constraints; a_fields_list: like specific_fields_to_load ): like items
			-- modifies specific_fields_to_load and execute_loads_specific_fields
		require
			specific_fields_to_load.is_empty
		local
			l_qry: COMPOSED_SQL_QUERY
		do
			create l_qry.make (item_prototype)
			set_execute_loads_specific_fields (True)

			specific_fields_to_load.append (a_fields_list)
			l_qry.constraints.append (some_constraints)

			execute_query (l_qry)
			Result := items

			set_execute_loads_specific_fields (False)
			specific_fields_to_load.wipe_out
		ensure
			execute_loads_specific_fields = False
			specific_fields_to_load.is_empty
		end

	valid_measure_unit_db_service_heir (a_measure_unit_heir: STRING): BOOLEAN
		do
			Result := a_measure_unit_heir.is_equal ({CHARGE_UNIT}.class_name_lowercase) or
				a_measure_unit_heir.is_equal ({TENSION_UNIT}.class_name_lowercase) or
				a_measure_unit_heir.is_equal ({CURRENT_UNIT}.class_name_lowercase)
		ensure
			instance_free: class
		end

feature -- Status setting

	set_execute_loads_specific_fields (v: like execute_loads_specific_fields)
		do
			execute_loads_specific_fields := v
		ensure
			execute_loads_specific_fields = v
		end

feature -- Entities loading

	execute
			-- Adds to items a created one from Current DB_SERVICE
		do
			if execute_loads_specific_fields then
				if not has_error then
					impl_execute_load_specific_fields
				end
			else
				Precursor
			end
		end


feature -- Optimized load

	execute_loads_specific_fields: BOOLEAN

	specific_fields_to_load: LINKED_LIST[STRING]

feature {NONE} -- Implementation

	set_items_time_zone_offsets (some_items: like items; a_time_zone_offset: INTEGER)
		do
			across
				items as l_item
			loop
				l_item.item.set_time_zone_offset (a_time_zone_offset)
			end
		end

	items_index: INTEGER

	average_item_from_interval (some_items: like items; a_start_dt, an_end_dt: DATE_TIME): attached like item
			-- loop into some_items and look (or compute) @ the corresponding item. If not found, returns a default item
			--
			-- Otherwise an average is calculated between a_start_dt and an_end_dt both included
			-- modifies items_index
		local
			l_found: BOOLEAN
			l_item: like item
			l_res_items_for_average: like items
		do
			create l_res_items_for_average.make
			from
				--items_index := 1
			until
				l_found or items_index > some_items.count
			loop
				l_item := some_items.at (items_index)
				if l_item.date_time >= a_start_dt and l_item.date_time < an_end_dt then
					l_res_items_for_average.extend (l_item.deep_twin)
				end
				l_found := l_item.date_time > an_end_dt
				if not l_found then
					items_index := items_index + 1
				end
			end
			create Result
			if l_res_items_for_average.count > 0 then
				-- Dont set times!
				Result.set_but_date_time (l_res_items_for_average.last)
			end
			Result.set_value (average_value (l_res_items_for_average))
			logger.write_debug ("item_from_interval->from:" + a_start_dt.formatted_out ({JSON_UTIL}.Date_time_format) + " to " + an_end_dt.formatted_out ({JSON_UTIL}.Date_time_format) + "%N" + {SIT_UTIL}.iterable_to_string(l_res_items_for_average) + "- res:" + Result.out)
		end

	average_value (some_items: like items): like item.value
			-- returns a null charge if any and otherwise an average of given l_items
		deferred
		end

	date_times_from_interval (a_start_date_time, an_end_date_time: DATE_TIME; some_minutes_increment: INTEGER): like items
			--list of items with charge as 0, and times from 0 to end of day with 'some_minutes_increment'
		local
			l_date_time, l_max_date_time, l_date_time_for_result, l_is_last_dt, l_tmp: DATE_TIME
			l_item: like item
			l_is_first, l_is_last: BOOLEAN
		do
			create Result.make
			l_max_date_time := an_end_date_time.deep_twin
			l_max_date_time.set_time(create {TIME}.make(23, 59, 59)) -- end of day
			from
				l_is_first := True
				l_date_time := a_start_date_time.deep_twin
				l_date_time.time.set_hour (0)
				l_date_time.time.set_minute (0)
				l_date_time.time.set_second (1)
				l_date_time_for_result := l_date_time.deep_twin
			until
				l_date_time.is_greater_equal (l_max_date_time)
			loop
				if l_is_last then
					l_tmp := l_date_time_for_result.deep_twin
					l_tmp.minute_add (some_minutes_increment)
					create l_item.make_from_date_time (l_tmp)
					Result.extend (l_item)
					l_date_time_for_result := l_max_date_time
				else
					l_date_time_for_result := l_date_time.deep_twin
				end
				if not l_is_first and not l_is_last then
					l_date_time_for_result.time.set_second (0)
				end

				create l_item.make_from_date_time (l_date_time_for_result.deep_twin)
				Result.extend (l_item)

				l_date_time.minute_add (some_minutes_increment)
				l_is_last_dt := l_date_time.deep_twin
				l_is_last_dt.minute_add (some_minutes_increment)
				l_is_first := False
				l_is_last := l_is_last_dt.is_greater_equal (l_max_date_time)
			end
			logger.write_debug ("date_times_from_interval->" + {SIT_UTIL}.iterable_to_string (Result))
		end

	impl_execute_load_specific_fields
			-- loads only specific fields given into specific_fields_to_load
		require
			execute_loads_specific_fields
			not specific_fields_to_load.is_empty
			not has_error
			not is_multirow_resultset
		local
			l_new_item: like item
		do
			create l_new_item
			if l_new_item.has_error then
				logger.write_error ("impl_execute_load_specific_fields-> error during execute of DB_Service, could not create item from db service")
				set_last_error_from_fallible (l_new_item)
			else
				across
					specific_fields_to_load as l_db_field_name
				loop
					if attached data_from_column_name (l_db_field_name.item) as l_db_value then
						l_new_item.call_setter_from_db_field_name (l_db_field_name.item, l_db_value)
					else
						logger.write_error ("impl_execute_load_specific_fields->requested field:" + l_db_field_name.item + " not found into resultset")
					end
				end
				items.extend (l_new_item)
				if items.count \\ 1000 = 0 then
					logger.write_information ("impl_execute_load_specific_fields->Adding new item to collection (" + items.count.out + "):" + l_new_item.out)
				end
--				logger.write_debug ("Adding new item to collection:" + l_new_item.out)
			end
		end



end
