note
	description: "Summary description for {MEASURING_POINT_DB_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	MEASURING_POINT_DB_SERVICE


inherit
	DB_SERVICE[MEASURING_POINT]
		redefine
			make,
			set_callable_features
		end

create
	make

feature {NONE} -- Initialization

	make (a_db_connection: attached DB_CONNECTION)
		do
			Precursor (a_db_connection)
		end

	set_callable_features
			--redefine me if needed to extend available features of DB Service[DB_ENTITY]
		do
			precursor
			callable_features.put (agent items_monthly_energy_for_chart (?), Rest_feature_items_monthly_energy)
			callable_features.put (agent import_from_abb_platform_csv (?), Rest_feature_import_from_csv)
			callable_features.put (agent import_from_sit_abb_csv (?), Rest_feature_import_from_sit_csv_abb)
			callable_features.put (agent import_from_sit_legrand_14668_csv (?), Rest_feature_import_from_sit_csv_legrand_14668)
			callable_features.put (agent import_from_sit_schneider_pm5500_csv (?), Rest_feature_import_from_sit_csv_pm5500)

			callable_features.extend (agent items_from_date_time_interval_request (?), Rest_feature_items_from_date_time_interval)
			callable_features.put (agent items_from_date_time_averages (?), Rest_feature_items_from_date_time_averages)

			callable_features.put (agent max_charge_for_id (?), Rest_feature_max_charge_for_id)
		end

feature -- Access

	-- Imports
	Rest_feature_import_from_csv: STRING = "import_from_abb_csv"
	Rest_feature_import_from_sit_csv_abb: STRING = "import_from_sit_csv_abb"
	Rest_feature_import_from_sit_csv_legrand_14668: STRING = "import_from_sit_csv_legrand_14668"
	Rest_feature_import_from_sit_csv_pm5500: STRING = "import_from_sit_csv_schneider_pm5500_csv"

	-- Reports
	Rest_feature_items_monthly_energy: STRING = "items_monthly_energy"
	Rest_feature_items_from_date_time_interval: STRING = "items_from_date_time_interval"
	Rest_feature_items_from_date_time_averages: STRING = "items_from_date_time_average"

	Rest_feature_max_charge_for_id: STRING = "max_charge_for_id"

feature {NONE} -- Rest Callable features

	items_from_date_time_interval_request (a_rest_request: REST_REQUEST): detachable like items
			-- returns a list of all measure_unit between given interval, with fk as given relationship
			--			
			-- data: { "action": { "feature": "items_from_date_time_interval_request", "arg1": "2018-12-14 00:00:00", "arg2": "2018-12-14 23:59:59", "arg3": "[1,3,4]", "arg4": "[{"date_time":"2019-04-06","offset":-180},{"date_time":"2019-04-07","offset":-240}]", "arg5": "measuring_point", "arg6": "charge_unit"} }
			--
			-- argument 1 is start_date_time
			-- argument 2 is end_date_time
			-- argument 3 is for measuring_point ids [10,3,...]
			-- argument 4 is for timezone in minutes
			-- argument 5 is type of measure_unit you need (see {MEASURE_UNIT_DB_SERVICE}.valid_measure_unit_db_service)
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 5
			fifth_argument_is_not_void_and_valid: attached l_req_f.arguments.at(5) as l_arg5 and then
				{CURRENT_UNIT_DB_SERVICE}.valid_measure_unit_db_service_heir (l_arg5)
		local
			l_error_pref: STRING
		do
			l_error_pref := "items_from_date_time_interval->Request error: "
			if
				attached a_rest_request.param_with_error_and_assertion (4, l_error_pref + "timezones", Current) as l_timezones_s and then
				attached {JSON_UTIL}.timezone_offsets_array_to_list (l_timezones_s) as l_timezone_offsets
			then
				if
					attached a_rest_request.param_with_error_and_assertion (1, l_error_pref + "start_date", Current) as l_start_date_s and then
					attached {JSON_UTIL}.date_time_to_offsetted_date_time (l_start_date_s, l_timezones_s) as l_start_dt
				then
					if
						attached a_rest_request.param_with_error_and_assertion (2, l_error_pref + "end_date", Current) as l_end_date_s and then
						attached {JSON_UTIL}.date_time_to_offsetted_date_time (l_end_date_s, l_timezones_s) as l_end_dt
					then
						if attached a_rest_request.param_with_error_and_assertion (3, l_error_pref + "measuring_points_ids", Current) as l_m_p_ids_s then
							if attached a_rest_request.param_with_error_and_assertion (5, l_error_pref + "l_measure_unit_heir", Current) as l_measure_unit_heir then
								Result := items_from_date_time_interval (l_start_dt, l_end_dt, ids_list_from_string (l_m_p_ids_s), l_measure_unit_heir, l_timezone_offsets, Void, True)
							end
						end
					end
				end
			end
		end


	items_from_date_time_averages (a_rest_request: REST_REQUEST): like items
			-- Returns a list of computed average items with timestamps of given zone on arg3 of request for a given day.
			-- Average is calculated for minutes between every day (arg2) from a returned date_time till the next one (so that the current charge_unit contains the average of timestamp + arg2)
			-- The returned timezones are UTC (so the returned data are shifted with given timezone)
			--
			-- data: { "action": { "feature": "items_from_date_time_for_chart", "arg1": "2018-12-10 00:00:01", "arg2": "2018-12-10 23:59:59" "arg3": "[15,3]", "arg4": "1", "arg5": "[{"date_time":"2019-04-06",...]", "arg6": "measuring_point", "arg7": "charge_unit"} }
			--
			-- argument 1 is start_date_time
			-- argument 2 is end_date_time
			-- argument 3 is for minutes between every charge_unit, an empty chargeunit will be returned in case of no data found for this timestamp
			-- argument 4 are measuring_point ids [12,43,4]
			-- argument 5 is a timezone offsets
			-- argument 6 is type of measure_unit you need (see {CHARGE_UNIT_DB_SERVICE}.valid_measure_unit_db_service)
			--
			-- modifies items, items_index
		require
			a_rest_request.has_request_feature
			invalid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 6
			third_argument_is_integer_minutes: attached l_req_f.arguments.at(3) as l_arg3 and then
				l_arg3.is_integer and then
				l_arg3.to_integer > 0
			forth_argument_is_integer_fk: attached l_req_f.arguments.at(4) as l_arg4 and then
				l_arg4.is_integer and then
				l_arg4.to_integer > 0
			sixth_argument_is_not_void: attached l_req_f.arguments.at(6) as l_arg6
			seventh_argument_is_not_void_and_valid: attached l_req_f.arguments.at(7) as l_arg7 and then
				{CHARGE_UNIT_DB_SERVICE}.valid_measure_unit_db_service_heir (l_arg7)
		local
			l_constraint: FIELD_CONSTRAINT
			l_error_pref: like last_error.message
		do
			l_error_pref := "items_from_date_time_averages->Request error: "
			create Result.make
			if
				attached a_rest_request.param_with_error_and_assertion (5, l_error_pref + "timezones", Current) as l_timezones_s and then
				attached {JSON_UTIL}.timezone_offsets_array_to_list (l_timezones_s) as l_timezone_offsets
			then
				if
					attached a_rest_request.param_with_error_and_assertion (1, l_error_pref + "start_date", Current) as l_start_date_s and then
					attached {JSON_UTIL}.date_time_to_offsetted_date_time (l_start_date_s, l_timezones_s) as l_start_dt
				then
					if
						attached a_rest_request.param_with_error_and_assertion (2, l_error_pref + "end_date", Current) as l_end_date_s and then
						attached {JSON_UTIL}.date_time_to_offsetted_date_time (l_end_date_s, l_timezones_s) as l_end_dt
					then
						if
							attached a_rest_request.param_with_error_and_assertion (3, l_error_pref + "minutes_increment", Current) as l_minutes_increment_s and then
							attached l_minutes_increment_s.to_integer_64 as l_minutes_increment
						then
							if
								attached a_rest_request.param_with_error_and_assertion (4, l_error_pref + "measuring_points_ids", Current) as l_m_p_ids_s and then
								attached ids_list_from_string (l_m_p_ids_s) as l_measuring_point_ids
							then
								if
									attached a_rest_request.param_with_error_and_assertion (6, l_error_pref + "measure_unit_type", Current) as l_measure_unit_type and then
									attached {MEASURE_UNIT_DB_SERVICE[MEASURE_UNIT]} service_from_entity_name (l_measure_unit_type) as l_measure_unit_db_service
								then
									if attached {ENUMERATE_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable ENUMERATE}) as l_enum_dbs then
										-- prepare constraint
										l_constraint := create {FIELD_CONSTRAINT}.make (l_measure_unit_db_service.item_prototype.table_name, l_measure_unit_db_service.item_prototype.An_index_fk, {CONSTRAINT}.equal_operator, l_enum_dbs.enumerate_cash_from_name ({DB_CONSTANTS}.Enumerate_neutral).primary_key)
										Result := items_from_date_time_interval (
												l_start_dt,
												l_end_dt,
												l_measuring_point_ids,
												l_measure_unit_type,
												l_timezone_offsets,
												l_constraint,
												True
											)
									end
								else
									set_last_error (create {INVALID_REQUEST}.make (l_error_pref + " service not found for measure_unit_type "))
									check
										service_not_found: False
									end
								end
							end
						end
					end
				end
			end
		end

	items_monthly_energy_for_chart (a_rest_request: REST_REQUEST): like items
			-- Returns a list of measuring_points with implementation of ENERGY_UNIT_DB_SERVICE.items_monthly_energy_for_chart for each one
			--
			-- data: { "action": { "feature": "items_monthly_energy_for_chart", "arg1": "2019-05-12", "arg2": "[4, 8]", "arg3": "[{"date_time":"2019-04-30","offset":-240},{"date_time":"2019-05-01","offset":-240},...]" } }
			--
			-- argument 1 is date of month to return
			-- argument 2 is measuring_point_ids array
			-- argument 3 timezone offsets array [{"date_time":"2019-04-30","offset":-240}, ...]
			--
			-- modifies items, items_index
		require
			a_rest_request.has_request_feature
			invalid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 3
			second_argument_is_string: attached l_req_f.arguments.at(2) as l_arg2 and then
				not l_arg2.is_empty
			third_argument_is_not_empty: attached l_req_f.arguments.at(3) as l_arg3 and then
				not l_arg3.is_empty
		local
			l_energy_units: like {ENERGY_UNIT_DB_SERVICE}.items
			l_error_pref: like last_error.message
		do
			l_error_pref := "items_monthly_energy_for_chart->Request error: "
			create Result.make
			if
				attached a_rest_request.param_with_error_and_assertion (3, l_error_pref + "timezones", Current) as l_timezones_s and then
				attached {JSON_UTIL}.timezone_offsets_array_to_list (l_timezones_s) as l_timezone_offsets
			then
				if
					attached a_rest_request.param_with_error_and_assertion (1, l_error_pref + "date", Current) as l_date_s and then
					attached {JSON_UTIL}.date_to_offsetted_date_time (l_date_s, l_timezones_s) as l_date_time
				then
					if attached {ENERGY_UNIT_DB_SERVICE} {SIT_ENVIRONMENT}.app_instance.db_service_from_entity_type ({detachable ENERGY_UNIT}) as l_energy_unit_db_s then
						if
							attached a_rest_request.param_with_error_and_assertion (2, l_error_pref + "measuring_points_ids", Current) as l_m_p_ids and then
							attached ids_list_from_string (l_m_p_ids) as l_measuring_point_ids
						then
							-- set energy_units
							across
								l_measuring_point_ids as l_measuring_point_id
							loop
								load_from_primary_key (l_measuring_point_id.item)
								if attached item as l_measuring_point then
									l_energy_units := l_energy_unit_db_s.items_daily_summarized (l_measuring_point, l_date_time, l_timezone_offsets)
									l_measuring_point.energy_unit_relationship.secondaries.append (l_energy_units)
									Result.extend (l_measuring_point)
								else
									set_last_error (create {INVALID_REQUEST}.make (l_error_pref + " measuring point not found for id:" + l_measuring_point_id.item.out))
								end
							end
						end
					else
						set_last_error (create {INVALID_REQUEST}.make (l_error_pref + " service not found for measure_unit_type "))
						check
							service_not_found: False
						end
					end
				end
			end
		end

	import_from_sit_schneider_pm5500_csv (a_rest_request: REST_REQUEST): detachable like items
		local
			l_title: like {SIT_EVENT}.title
			l_csv_report: SCHNEIDER_PM5500
		do
			create l_csv_report.make (a_rest_request, Current)
			l_csv_report.process

			create l_title.make_from_string (Rest_feature_import_from_sit_csv_pm5500)
			if attached a_rest_request.request_feature as l_f then
				l_title.append (", file:" + l_f.arguments.at(1) + " measuring_point_id:" + l_f.arguments.at(2))
			end
			add_sit_event_report (l_title)
		end

	import_from_sit_abb_csv (a_rest_request: REST_REQUEST): detachable like items
		local
			l_title: like {SIT_EVENT}.title
			l_csv_report: ABB_SIT
		do
			create l_csv_report.make (a_rest_request, Current)
			l_csv_report.process

			l_title := "import_from_sit_abb_csv"
			if attached a_rest_request.request_feature as l_f then
				l_title.append (", file:" + l_f.arguments.at(1) + " measuring_point_id:" + l_f.arguments.at(2))
			end
			add_sit_event_report (l_title)
		end

	import_from_abb_platform_csv (a_rest_request: REST_REQUEST): detachable like items
		local
			l_csv_report: ABB_PLATFORM
			l_title: like {SIT_EVENT}.title
		do
			create l_csv_report.make (a_rest_request, Current)
			l_csv_report.process

			-- Event
			l_title := "import_from_abb_platform_csv"
			if attached a_rest_request.request_feature as l_f then
				l_title.append (", file:" + l_f.arguments.at(1) + " measuring_point_id:" + l_f.arguments.at(2))
			end
			add_sit_event_report (l_title)
		end

	import_from_sit_legrand_14668_csv (a_rest_request: REST_REQUEST): detachable like items
		local
			l_csv_report: LEGRAND_14668
			l_title: like {SIT_EVENT}.title
		do
			create l_csv_report.make (a_rest_request, Current)
			l_csv_report.process

			-- Event
			l_title := "import_from_sit_legrand_14668_csv"
			if attached a_rest_request.request_feature as l_f then
				l_title.append (", file:" + l_f.arguments.at(1) + " measuring_point_id:" + l_f.arguments.at(2))
			end
			add_sit_event_report (l_title)
		end

	max_charge_for_id (a_rest_request: REST_REQUEST): detachable like items
			-- data: { "action": { "feature": "max_charge_for_id", "arg1": "1"} }
			--
			-- argument 1 is id of measuring_point
		require
			a_rest_request.has_request_feature
			valid_arguments: attached a_rest_request.request_feature as l_req_f and then
				l_req_f.has_arguments and then
				l_req_f.arguments.count = 1
			first_argument_is_integer: attached l_req_f.arguments.at(1) as l_arg1 and then
				l_arg1.is_integer and then
				l_arg1.to_integer > 0
		local
			l_qry: COMPOSED_SQL_QUERY
			l_item_prototype_clone: like item_prototype
		do
			if attached a_rest_request.request_feature as l_req_f and then
				attached l_req_f.arguments.at(1) as l_measuring_point_id_s and then
				attached {INTEGER_64} l_measuring_point_id_s.to_integer_64 as l_measuring_point_id
			then
				l_item_prototype_clone := item_prototype.deep_twin
				l_item_prototype_clone.set_primary_key (l_measuring_point_id)
				create l_qry.make (l_item_prototype_clone)
--				l_qry.join_clauses.append (l_item_prototype_clone.charge_unit_relationship.to_joins)
--				l_qry.add_constraint (l_item_prototype_clone.charge_unit_relationship.to_constraint)
--				l_qry.select_clauses.extend ("MAX(" + l_item_prototype_clone.charge_unit_relationship.secondary_prototype.charge_db_column_name + ")")
				l_qry.select_clauses.extend (l_item_prototype_clone.table_name + ".*")
				l_qry.select_clauses.append (l_item_prototype_clone.charge_unit_relationship.to_api_select_clauses)
				l_qry.order_clauses.wipe_out
				l_qry.order_clauses.extend (create {ORDER_CLAUSE}.make (l_item_prototype_clone.charge_unit_relationship.secondary_prototype.table_name, l_item_prototype_clone.charge_unit_relationship.secondary_prototype.Value_db_column_name, {ORDER_CLAUSE}.way_descending))
				l_qry.set_limit (1)

				item_prototype.charge_unit_relationship.set_loadable_from_resultset (True)
				load_entities (l_qry)
				Result := items
			else
				check
					invalid_parameters: False
				end
			end
		end

feature {NONE} -- Implementation misc


	items_from_date_time_interval (a_start_dt, an_end_dt: DATE_TIME; a_measuring_point_ids_array: like ids_list_from_string; a_measure_unit_type: STRING; some_timezone_offsets: like {JSON_UTIL}.timezone_offsets_array_to_list; a_constraint: detachable CONSTRAINT; a_with_relationships: BOOLEAN ): like items
			-- returns a collection of items from DB and corresponding service
		local
			l_measure_units: like {MEASURE_UNIT_DB_SERVICE[MEASURE_UNIT]}.items_from_date_time_interval
		do
			create Result.make
			if attached {MEASURE_UNIT_DB_SERVICE[MEASURE_UNIT]} service_from_entity_name (a_measure_unit_type) as l_measure_unit_db_service then
				across
					a_measuring_point_ids_array as l_mp_id
				loop
					load_from_primary_key (l_mp_id.item)
					if attached item as l_measuring_point then
						l_measure_units := l_measure_unit_db_service.items_from_date_time_interval (a_start_dt, an_end_dt, l_measuring_point, a_constraint, a_with_relationships)
						if attached {ONE_TO_MANY[RELATED_DB_ENTITY, RELATED_DB_ENTITY]} l_measuring_point.relationship_where_secondary_is_type (a_measure_unit_type) as l_rel then
							across -- adjust timezone
								l_measure_units is l_item
							loop
								l_item.date_time.minute_add ({SIT_UTIL}.timezone_offset_for_index (some_timezone_offsets, l_item.date_time))
							end
							l_rel.secondaries.append (l_measure_units)
						else
							check
								not_happening: False
							then
								{SIT_UTIL}.raise_exception ("items_from_date_time_interval-> relationship not found")
							end
						end

						Result.extend (l_measuring_point)
					end
				end
			else
				set_last_error (create {INVALID_REQUEST}.make ("items_from_date_time_interval->Request error: Argument for measure_unit_type is not a measure_unit :" + a_measure_unit_type))
				check
					not_happening: False
				end
			end
		end

end
