note
	description: "Summary description for {TENSION_UNIT_DB_SERVICE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	SIT_EVENT_DB_SERVICE

inherit
	DB_SERVICE[SIT_EVENT]
		redefine
			make,
			create_secondary_association_if_any,
			set_callable_features
		end

create
	make

feature {NONE} -- Initialization

	make (a_db_connection: attached DB_CONNECTION)
		do
			Precursor (a_db_connection)
		end

	set_callable_features
		do
			Precursor
			callable_features.put (agent get_events_from_class_and_id, Feature_get_events_from_class_and_id)
		end


feature -- Access

	Feature_get_events_from_class_and_id: STRING = "get_events_from_class_and_id"

feature -- Status setting

	add_event (an_event: SIT_EVENT)
		do
			set_item (an_event)
			create_or_update_item
			if has_error and then
				attached last_error as l_err
			then
				logger.write_error (l_err.out)
				check
					not_happening: False
				end
			else
				item := Void
			end
		ensure
			last_inserted_id_changed: last_inserted_id /= old last_inserted_id
		end

feature {NONE} -- Implementation

	get_events_from_class_and_id (a_rest_request: REST_REQUEST): detachable like items
			-- returns a list of all events with given arguments
			-- T
			--			
			-- data: { "action": { "feature": "get_events_from_class_and_id", "arg1": "measuring_point", "arg2": "5"} }
			--
			-- argument 1 is class_name (Optional)
			-- argument 2 is id of object for given class_name (Optional)
		require
			a_rest_request.has_request_feature
		local
			l_qry: COMPOSED_SQL_QUERY
			l_rel: like item_prototype.relationships
		do
			if attached a_rest_request.request_feature as l_req_f then
				create l_qry.make (item_prototype)
				item_prototype.relationships.prune (item_prototype.entities_relationship)
				l_rel := item_prototype.relationships.deep_twin
				item_prototype.relationships.extend (item_prototype.entities_relationship) --restore
				item_prototype.status_relationship.set_loadable_from_resultset (True)

				set_query_from_relationships (l_qry, l_rel)
				if l_req_f.arguments.valid_index (1) and then
					attached l_req_f.arguments.at (1) as l_class_name
				then
					l_qry.join_clauses.extend (create {INNER_JOIN}.make_from_fields ("event_entity",
						Void,
						"event_entity",
						"event_id",
						"event",
						item_prototype.Primary_key_db_column_name))
					-- l_qry.from_clauses.extend ("event_event_entity")
					l_qry.constraints.extend (create {FIELD_CONSTRAINT}.make ("event_entity", "entity_class", {CONSTRAINT}.Ilike_operator, l_class_name))
					if l_req_f.arguments.valid_index (2) and then
						attached l_req_f.arguments.at (2) as l_pk and then
						l_pk.is_integer
					then
						l_qry.constraints.extend (create {FIELD_CONSTRAINT}.make ("event_entity", "entity_id", {CONSTRAINT}.Equal_operator, l_pk.to_integer))
					end
				end
				execute_query (l_qry)
			end
		end

	create_secondary_association_if_any
		require else
			valid_sit_event_id: last_inserted_id > 0
		local
			l_updt: STRING_SQL_UPDATE
			l_last_inserted_event_id: like {PK_DB_ENTITY}.Primary_key
			l_msg: STRING
		do
			l_last_inserted_event_id := last_inserted_id
			if attached item as l_item then
				across
					l_item.entities_relationship.secondaries as l_pkdbe
				loop
					-- create association on join table
					create l_updt.make_from_string (item_prototype, "INSERT INTO " + l_item.entities_relationship.join_table_name +
						"(event_id, entity_id, entity_class)" +
						" VALUES (" + l_last_inserted_event_id.out + ", " + l_pkdbe.item.primary_key.out + ", '" + l_pkdbe.item.class_name_lowercase + "')")
					execute_update (l_updt)
					if has_error then
						l_msg := "create_secondary_association_if_any-> last DB modification did not work with entity:" + l_pkdbe.out
						if attached db_connection.last_error as l_err then
							l_msg.append ("-error:" + l_err.out)
						end
						set_last_error( create {INSERT_ERROR}.make (l_msg) )
					else
						set_last_success_message ("Entity created into DB successfully")
					end
				end
				l_item.entities_relationship.secondaries.wipe_out
				last_inserted_id := l_last_inserted_event_id
				Precursor
			elseif attached items as l_items then
				check
					not_implemented: False
				end
			end
		end

end
