note
	description: "Summary description for {JSON_UTIL}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	JSON_UTIL

inherit
	LOGGABLE

feature -- instance_free for most

	last_error: detachable SIT_ERROR

	adapted_for_parser ( s : STRING ): STRING
		do
			Result := s.twin
			if not s.item (1).is_equal ('[') then
				Result.prepend("[")
				Result.append ("]")
			end
		end


	item_from_json_array(a_json_array: JSON_ARRAY; l_field_name: STRING): detachable ANY
		do
			logger.write_debug ("Debug Output:" + a_json_array.debug_output)
			across
				a_json_array as l_json_array_item
			loop
				if attached {JSON_STRING} l_json_array_item as l_json_array_s then
					Result := l_json_array_s
				else
					create {PARSE_ERROR} last_error.make ("l_json_array_item is of unknown Type:" + l_json_array_item.item.representation)
				end
			end
		end

feature -- DATE_TIME / DATE

	Date_time_format: STRING = "YYYY-[0]MM-[0]DD [0]hh:[0]mi:[0]ss"
	Date_format: STRING = "YYYY-[0]MM-[0]DD"

	date_string_to_date_time (a_json_date_s: STRING): DATE_TIME
		do
			create Result.make_by_date_time (create {DATE}.make_from_string (a_json_date_s, Date_format), create {TIME}.make (0, 0, 1))
		ensure
			instance_free: class
		end

	date_time_string_to_date_time (a_json_date_time_s: STRING): DATE_TIME
		do
			create Result.make_from_string (a_json_date_time_s, Date_time_format)
		ensure
			instance_free: class
		end

	date_to_offsetted_date_time (a_json_date_s: STRING; a_dt_offsets_list: STRING): DATE_TIME
		do
			Result := date_string_to_date_time (a_json_date_s)
			Result.minute_add (- {SIT_UTIL}.timezone_offset_for_index ({JSON_UTIL}.timezone_offsets_array_to_list (a_dt_offsets_list), Result))
		ensure
			instance_free: class
		end

	date_time_to_offsetted_date_time (a_json_date_time_s: STRING; a_dt_offsets_list: STRING): DATE_TIME
		do
			Result := date_time_string_to_date_time (a_json_date_time_s)
			Result.minute_add (- {SIT_UTIL}.timezone_offset_for_index ({JSON_UTIL}.timezone_offsets_array_to_list (a_dt_offsets_list), Result))
		ensure
			instance_free: class
		end

	to_date (a_json_string: STRING): DATE
		do
			create Result.make_from_string (a_json_string, Date_format)
		ensure
			instance_free: class
		end

	date_to_api_export (v: DATE): STRING
		do
			Result := v.formatted_out (Date_format)
		ensure
			instance_free: class
		end

	date_time_to_api_export (v: DATE_TIME): STRING
		do
			Result := v.formatted_out ({APP_CONFIGURATION}.Date_time_default_format_string)
		ensure
			instance_free: class
		end

	timezone_offsets_array_to_list (a_json_s: STRING): LINKED_LIST[DATE_OFFSET]
			-- a_json_s:
		require
			not a_json_s.is_empty
		local
			l_json_parser: JSON_PARSER
			l_date: DATE
		do
			create Result.make
			create l_json_parser.make_with_string (a_json_s)
			l_json_parser.parse_content
			if l_json_parser.is_valid and then
					attached l_json_parser.parsed_json_value as l_parsed_json_value and then
					attached {JSON_ARRAY} l_parsed_json_value as l_parsed_json_obj
			then
				across
					l_parsed_json_obj as l_item
				loop
					if attached {JSON_OBJECT} l_item.item as l_obj then
						if attached {JSON_STRING} l_obj.item ("date_time") as l_dt_s then
							create l_date.make_from_string (l_dt_s.unescaped_string_8, "yyyy-mm-dd")
							if attached l_obj.item ("offset") as l_offset and then
								l_offset.representation.is_integer
							then
								Result.extend (create {DATE_OFFSET}.make_from_date (l_date, l_offset.representation.to_integer))
							else
								logger.write_error ("cannot parse offset from json object")
								check
									not_happening: False
								end
							end

						end
					end
					do_nothing
				end
			else
				logger.write_error ("Error parsing given json string: " + a_json_s)
				check
					not_happening: False
				end
			end
		ensure
			valid_result: Result.count > 1
			instance_free: class
		end

end
