note
	description: "[
				application execution
			]"
	date: "$Date: 2016-10-21 10:45:18 -0700 (Fri, 21 Oct 2016) $"
	revision: "$Revision: 99331 $"

class
	EWF_APP_EXECUTION


inherit
	WSF_FILTERED_ROUTED_EXECUTION
		redefine
			initialize,
			clean
		end

	LOGGABLE

create
	make

feature {NONE} -- Initialization

	initialize
		do
			logger.write_debug ("------------------------> Starting Request -------------------------->")
			Precursor
		end

feature -- Access

	db_connection: detachable DB_CONNECTION

feature -- Status setting

	set_db_connection (v: attached like db_connection)
		do
			db_connection := v
		ensure
			db_connection = v
		end

	add_handler (a_handler: SIT_HANDLER[DB_ENTITY])
		do
			{SIT_ENVIRONMENT}.app_instance.add_handler (a_handler)
			{SIT_ENVIRONMENT}.app_instance.add_db_service (a_handler.db_service)
		end

feature -- Filter

	create_filter
			-- Create `filter'
		do
				--| Example using Maintenance filter.
			create {WSF_MAINTENANCE_FILTER} filter
		end

	setup_filter
			-- Setup `filter'
		local
			l_filter: like filter
		do
			logger.write_debug ("Setting CORS filter")
			create {WSF_CORS_FILTER} l_filter
			l_filter.set_next (create {WSF_LOGGING_FILTER})

				--| Chain more filters like {WSF_CUSTOM_HEADER_FILTER}, ...
				--| and your owns filters.

			filter.append (l_filter)
		end


feature -- Router

	setup_router
			-- Setup `router'
		require else
			attached db_connection as l_db_c and then l_db_c.is_connected
		local
			fhdl: WSF_FILE_SYSTEM_HANDLER
		do
				--| As example:
				--|   /doc is dispatched to self documentated page
				--|   /* are dispatched to serve files/directories contained in "www" directory

				--| Self documentation
			router.handle ("/api/doc", create {WSF_ROUTER_SELF_DOCUMENTATION_HANDLER}.make (router), router.methods_GET)

			db_connection := {APP_CONFIGURATION}.app_instance.db_connection
			logger.write_debug ("setting up router")
			if attached db_connection as l_db_c then
				if not l_db_c.is_connected then
					l_db_c.connect
				end
				  -- Add all handlers here so that their execute method can be called
				add_handler (create {SIT_HANDLER[COMPANY]}.make (l_db_c, router))
				add_handler (create {SIT_HANDLER[BRANCH]}.make (l_db_c, router))
				add_handler (create {SIT_HANDLER[CONSUMPTION_SECTOR]}.make (l_db_c, router))
				add_handler (create {SIT_HANDLER[MEASURING_POINT]}.make_from_db_service (l_db_c, router, create {MEASURING_POINT_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[DISTRIBUTION_COMPANY]}.make (l_db_c, router))
				add_handler (create {SIT_HANDLER[ELECTRICITY_RATE]}.make (l_db_c, router))
				add_handler (create {SIT_HANDLER[ENUMERATE]}.make_from_db_service (l_db_c, router, create {ENUMERATE_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[ELECTRICITY_INVOICE]}.make (l_db_c, router))
				add_handler (create {SIT_HANDLER[CHARGE_TEMPLATE]}.make_from_db_service (l_db_c, router, create {CHARGE_TEMPLATE_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[UF]}.make (l_db_c, router))
				add_handler (create {SIT_HANDLER[CUSTOMER_INVOICE]}.make_from_db_service (l_db_c, router, create {CUSTOMER_INVOICE_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[CHARGE_UNIT]}.make_from_db_service (l_db_c, router, create {CHARGE_UNIT_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[TENSION_UNIT]}.make_from_db_service (l_db_c, router, create {TENSION_UNIT_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[CURRENT_UNIT]}.make_from_db_service (l_db_c, router, create {CURRENT_UNIT_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[ENERGY_UNIT]}.make_from_db_service (l_db_c, router, create {ENERGY_UNIT_DB_SERVICE}.make (l_db_c)))
				add_handler (create {SIT_HANDLER[SIT_EVENT]}.make_from_db_service (l_db_c, router, create {SIT_EVENT_DB_SERVICE}.make (l_db_c)))
			else
				logger.write_error ("Db connection not set, not creating handlers!")
			end
				--| Files publisher
			create fhdl.make_hidden ("www")
			fhdl.set_directory_index (<<"index.html">>)
			router.handle ("", fhdl, router.methods_GET)
		end

feature -- Cleaning

	clean
		do
			Precursor
			logger.write_debug ("clean")
			logger.write_debug ("<--------- ENDING Request ---------------")
			if attached db_connection as l_db_c then
				l_db_c.disconnect
			end
		end


end
